@@ -20,13 +20,6 @@
 #include "gwmdesktop.h"
 
 
-#define CONFIG_OBOX_BORDER 6
-#define CONFIG_OBOX_SPACING 8
-#define CONFIG_IBOX_BORDER 6
-#define CONFIG_IBOX_SPACING 4
-#define CONFIG_ITEM_BORDER 0
-#define CONFIG_ITEM_SPACING 4
-  
 
 /* --- prototypes --- */
 static void	gp_init_gui		(void);
@@ -41,8 +34,7 @@ static gboolean	gp_task_notifier	(gpointer	    func_data,
 					 GwmhTaskInfoMask   imask);
 static void	gp_about		(void);
 static void	gp_config_popup		(void);
-static gpointer	gp_config_find_value	(const gchar       *path,
-					 gboolean           tmp_value);
+static gpointer	gp_config_find_value	(const gchar *path);
 static void	gp_load_config		(const gchar	   *privcfgpath);
 static gboolean	gp_save_session		(gpointer	    func_data,
 					 const gchar	   *privcfgpath,
@@ -60,87 +52,70 @@ static GtkWidget	*gp_desk_widget[MAX_DESKTOPS] = { NULL, };
 static guint             gp_n_desk_widgets = 0;
 static GdkWindow	*gp_atom_window = NULL;
 static GtkOrientation    gp_orientation = GTK_ORIENTATION_HORIZONTAL;
-/* static guint             gp_panel_size = 12; */
 static guint		 GP_TYPE_HBOX = 0;
 static guint		 GP_TYPE_VBOX = 0;
 static guint		 GP_ARROW_DIR = 0;
 static gchar		*DESK_GUIDE_NAME = NULL;
 
 static ConfigItem gp_config_items[] = {
-  CONFIG_PAGE (N_ ("Display")),
-  CONFIG_SECTION (sect_layout,				  	N_ ("Layout")),
-  CONFIG_BOOL (switch_arrow,	FALSE,
-	       N_ ("Switch tasklist arrow")),
-  CONFIG_BOOL (show_pager,	TRUE,
-	       N_ ("Show desktop pager")),
-  CONFIG_BOOL (current_only,	FALSE,
-	       N_ ("Only show current desktop in pager")),
-  CONFIG_BOOL (raise_grid,		FALSE,
-	       N_ ("Raise area grid over tasks")),
-  CONFIG_SECTION (sect_tooltips,			 	N_ ("Tooltips")),
-  CONFIG_BOOL (tooltips,	TRUE,
-	       N_ ("Show Desk-Guide tooltips")),
-  CONFIG_RANGE (tooltips_delay,	500,	1,	5000,
-		N_ ("Desk-Guide tooltip delay [ms]")),
+  CONFIG_SECTION (tooltip_settings,			 	N_ ("Tooltips")),
   CONFIG_BOOL (desktips,	TRUE,
-	       N_ ("Show desktop name tooltips")),
+	       N_ ("Enable Desktop Names")),
   CONFIG_RANGE (desktips_delay,	10,	1,	5000,
-		N_ ("Desktop name tooltip delay [ms]")),
+		N_ ("Desktop Name Popup Delay [ms]")),
+  CONFIG_BOOL (tooltips,	TRUE,
+	       N_ ("Enable Tooltips")),
+  CONFIG_RANGE (tooltips_delay,	500,	1,	5000,
+		N_ ("Tooltips Delay [ms]")),
+  CONFIG_SECTION (layout,				  	N_ ("Layout")),
+  CONFIG_BOOL (switch_arrow,	FALSE,
+	       N_ ("Switch Arrow")),
+  CONFIG_BOOL (current_only,	FALSE,
+	       N_ ("Show only current Desktop")),
+  CONFIG_BOOL (show_pager,	TRUE,
+	       N_ ("Show Desktop Pager")),
   
   CONFIG_PAGE (N_ ("Tasks")),
-  CONFIG_SECTION (sect_task_visibility,			  	N_ ("Visibility")),
+  CONFIG_SECTION (task_visibility,			  	N_ ("Visibility")),
   CONFIG_BOOL (show_hidden_tasks,	TRUE,
-	       N_ ("Show hidden tasks (HIDDEN)")),
+	       N_ ("Show `HIDDEN' Tasks")),
   CONFIG_BOOL (show_shaded_tasks,	TRUE,
-	       N_ ("Show shaded tasks (SHADED)")),
+	       N_ ("Show `SHADED' Tasks")),
   CONFIG_BOOL (show_skip_winlist,	FALSE,
-	       N_ ("Show tasks which hide from window list (SKIP-WINLIST)")),
+	       N_ ("Show `SKIP-WINLIST' Tasks")),
   CONFIG_BOOL (show_skip_taskbar,	FALSE,
-	       N_ ("Show tasks which hide from taskbar (SKIP-TASKBAR)")),
-  /*  CONFIG_SECTION (sect_null_1, NULL), */
+	       N_ ("Show `SKIP-TASKBAR' Tasks")),
+  CONFIG_SECTION (cosmetics,				  	N_ ("Cosmetics")),
+  CONFIG_BOOL (raise_grid,		FALSE,
+	       N_ ("Raise Area Grid Above Tasks")),
+  /*  CONFIG_SECTION ("", NULL), */
   
   CONFIG_PAGE (N_ ("Geometry")),
-  CONFIG_SECTION (sect_horizontal,                     	     	N_ ("Horizontal Layout")),
+  CONFIG_SECTION (horizontal,                        	     	N_ ("Horizontal Layout")),
   CONFIG_RANGE (area_height,	44,	4,	1024,
 		N_ ("Desktop Height [pixels]")),
-  /*
-    CONFIG_BOOL (abandon_area_height,		TRUE,
-	       N_ ("Override desktop height with panel size")),
-  */
   CONFIG_BOOL (div_by_vareas,		TRUE,
-	       N_ ("Divide height by number of vertical areas")),
+	       N_ ("Divide Height By Number Of Vertical Areas")),
   CONFIG_RANGE (row_stackup,	1,	1,	64,
-		N_ ("Rows of Desktops")),
-  CONFIG_BOOL (div_by_nrows,		TRUE,
-	       N_ ("Divide height by number of rows")),
-  CONFIG_SECTION (sect_vertical,                       	     	N_ ("Vertical Layout")),
+		N_ ("Number of Rows to pack Desktops in")),
+  CONFIG_SECTION (vertical,                        	     	N_ ("Vertical Layout")),
   CONFIG_RANGE (area_width,	44,	4,	1024,
-		N_ ("Desktop Width [pixels]")),
-  /*
-    CONFIG_BOOL (abandon_area_width,		TRUE,
-	       N_ ("Override desktop width with panel size")),
-  */
+		N_ ("Desktop width [pixels]")),
   CONFIG_BOOL (div_by_hareas,		TRUE,
-	       N_ ("Divide width by number of horizontal areas")),
+	       N_ ("Divide Width By Number Of Horizontal Areas")),
   CONFIG_RANGE (col_stackup,	1,	1,	64,
-		N_ ("Columns of Desktops")),
-  CONFIG_BOOL (div_by_ncols,		TRUE,
-	       N_ ("Divide width by number of columns")),
+		N_ ("Number of Columns to pack Desktops in")),
 
-  CONFIG_PAGE (N_ ("Advanced")),
-  CONFIG_SECTION (sect_drawing,                   	     	N_ ("Drawing")),
+  CONFIG_PAGE (N_ ("Advanced Options")),
   CONFIG_BOOL (double_buffer,	TRUE,
-	       N_ ("Draw desktops double-buffered (recommended)")),
-  CONFIG_SECTION (sect_workarounds,         	     	N_ ("Window Manager Workarounds")),
+	       N_ ("Draw Desktops Double Buffered")),
+  CONFIG_SECTION (bug_fixes,                        	     	N_ ("Bug Fixes")),
   CONFIG_BOOL (skip_movement_offset,		TRUE,
-	       N_ ("Window manager moves decoration window instead\n"
+	       N_ ("Window Manager Moves Decoration Window Instead\n"
 		   "(AfterStep, Enlightenment, FVWM, IceWM, SawMill)")),
   CONFIG_BOOL (unified_areas,			TRUE,
-	       N_ ("Window manager changes active area on all desktops\n"
+	       N_ ("Window Manager Changes Active Area On All Desktops\n"
 		   "(FVWM, SawMill)")),
-  CONFIG_BOOL (violate_client_msg,			TRUE,
-	       N_ ("Window manager expects pager to modify area+desktop properties directly\n"
-		   "(Enlightenment, FVWM, SawMill)")),
 };
 static guint  gp_n_config_items = (sizeof (gp_config_items) /
 				   sizeof (gp_config_items[0]));
@@ -224,8 +199,6 @@ main (gint   argc,
 		  "signal::change-orient", fixme_applet_widget_get_panel_orient, NULL,
 		  "signal::change-orient", gp_destroy_gui, NULL,
 		  "signal::change-orient", gp_init_gui, NULL,
-		  "signal::change-size", gp_destroy_gui, NULL,
-		  "signal::change-size", gp_init_gui, NULL,
 		  "object_signal::save-session", gp_save_session, NULL,
 		  "signal::destroy", gtk_main_quit, NULL,
 		  NULL);
@@ -273,7 +246,7 @@ gp_load_config (const gchar *privcfgpath)
 {
   static guint loaded = FALSE;
   guint i;
-  gchar *section = "sect_general";
+  gchar *section = "general";
   
   if (loaded)
     return;
@@ -317,8 +290,7 @@ gp_load_config (const gchar *privcfgpath)
 }
 
 static gpointer
-gp_config_find_value (const gchar *path,
-		      gboolean     tmp_value)
+gp_config_find_value (const gchar *path)
 {
   guint i;
   
@@ -327,7 +299,7 @@ gp_config_find_value (const gchar *path,
       ConfigItem *item = gp_config_items + i;
       
       if (path == item->path)
-	return tmp_value ? item->tmp_value : item->value;
+	return item->value;
     }
   
   g_warning (G_GNUC_PRETTY_FUNCTION "(): unable to find config value for <%s>", path);
@@ -341,7 +313,7 @@ gp_save_session (gpointer     func_data,
 		 const gchar *globcfgpath)
 {
   guint i;
-  gchar *section = "sect_general";
+  gchar *section = "general";
   
   gnome_config_push_prefix (privcfgpath);
   
@@ -469,39 +441,19 @@ gp_create_desk_widgets (void)
     g_error ("MAX_DESKTOPS limit reached, adjust source code");
 
   /* some gwmh configuration hacks ;( */
-  gwmh_desk_set_hack_values (BOOL_CONFIG (unified_areas),
-			     BOOL_CONFIG (violate_client_msg));
+  gwmh_desk_set_hack_values (BOOL_CONFIG (unified_areas));
 
   if (!gp_desk_box)
     return;
 
   /* configure Desktop widget class for us */
-  if (gp_orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-      /*
-	if (BOOL_CONFIG (abandon_area_height))
-	area_size = gp_panel_size;
-	else
-      */
-      area_size = RANGE_CONFIG (area_height);
-      if (BOOL_CONFIG (div_by_vareas))
-	area_size /= (gdouble) N_VAREAS;
-      if (BOOL_CONFIG (div_by_nrows))
-	area_size /= (gdouble) RANGE_CONFIG (row_stackup);
-    }
-  else /* gp_orientation == GTK_ORIENTATION_VERTICAL */
-    {
-      /*
-	if (BOOL_CONFIG (abandon_area_width))
-	area_size = gp_panel_size;
-	else
-      */
-      area_size = RANGE_CONFIG (area_width);
-      if (BOOL_CONFIG (div_by_hareas))
-	area_size /= (gdouble) N_HAREAS;
-      if (BOOL_CONFIG (div_by_ncols))
-	area_size /= (gdouble) RANGE_CONFIG (col_stackup);
-    }
+  area_size = (gp_orientation == GTK_ORIENTATION_HORIZONTAL
+	       ? RANGE_CONFIG (area_height)
+	       : RANGE_CONFIG (area_width));
+  if (gp_orientation == GTK_ORIENTATION_HORIZONTAL && BOOL_CONFIG (div_by_vareas))
+    area_size /= (gdouble) N_VAREAS;
+  if (gp_orientation == GTK_ORIENTATION_VERTICAL && BOOL_CONFIG (div_by_hareas))
+    area_size /= (gdouble) N_HAREAS;
   gwm_desktop_class_config (gtk_type_class (GWM_TYPE_DESKTOP),
 			    BOOL_CONFIG (double_buffer),
 			    gp_orientation,
@@ -625,13 +577,11 @@ gp_widget_ignore_button (GtkWidget      *widget,
 static void 
 gp_init_gui (void)
 {
-  /*  static guint panel_sizes[] = { 24, 48, 64, 80 }; */
   GtkWidget *button, *abox;
   gboolean arrow_at_end = FALSE;
   GtkWidget *main_box;
   
   gtk_widget_set_usize (gp_container, 0, 0);
-  /*  gp_panel_size = panel_sizes[applet_widget_get_panel_size (APPLET_WIDGET (gp_applet))]; */
   switch (applet_widget_get_panel_orient (APPLET_WIDGET (gp_applet)))
     {
     case ORIENT_UP:
@@ -792,19 +742,6 @@ gp_about (void)
   gdk_window_raise (dialog->window);
 }
 
-static void
-gp_config_check (GtkWidget *widget)
-{
-  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
-
-  /*
-    gtk_widget_set_sensitive (CONFIG_WIDGET (toplevel, area_height),
-			    !BOOL_TMP_CONFIG (abandon_area_height));
-  gtk_widget_set_sensitive (CONFIG_WIDGET (toplevel, area_width),
-			    !BOOL_TMP_CONFIG (abandon_area_width));
-  */
-}
-
 static void
 gp_config_toggled (GtkToggleButton *button,
 		   ConfigItem      *item)
@@ -817,21 +754,18 @@ static inline GtkWidget*
 gp_config_add_boolean (GtkWidget  *vbox,
 		       ConfigItem *item)
 {
-  GtkWidget *widget, *toplevel = gtk_widget_get_toplevel (vbox);
+  GtkWidget *widget;
   
   widget = gtk_widget_new (GTK_TYPE_CHECK_BUTTON,
 			   "visible", TRUE,
 			   "label", _ (item->name),
 			   "active", GPOINTER_TO_INT (item->value),
-			   "border_width", CONFIG_ITEM_BORDER,
 			   "signal::toggled", gp_config_toggled, item,
-			   "object_signal_after::toggled", gp_config_check, toplevel,
 			   NULL);
   gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
   gtk_widget_set (GTK_BIN (widget)->child,
 		  "xalign", 0.0,
 		  NULL);
-  gtk_object_set_data (GTK_OBJECT (toplevel), item->path, widget);
   
   return widget;
 }
@@ -849,7 +783,7 @@ gp_config_add_range (GtkWidget  *vbox,
 		     ConfigItem *item)
 {
   GtkObject *adjustment;
-  GtkWidget *hbox, *label, *spinner, *toplevel = gtk_widget_get_toplevel (vbox);
+  GtkWidget *hbox, *label, *spinner;
   
   adjustment = gtk_adjustment_new (GPOINTER_TO_UINT (item->tmp_value),
 				   item->min,
@@ -857,10 +791,10 @@ gp_config_add_range (GtkWidget  *vbox,
 				   1, 1,
 				   (item->max - item->min) / 10);
   hbox = gtk_widget_new (GTK_TYPE_HBOX,
-			 "homogeneous", FALSE,
+			 "homogeneous", TRUE,
 			 "visible", TRUE,
-			 "spacing", CONFIG_ITEM_SPACING,
-			 "border_width", CONFIG_ITEM_BORDER,
+			 "spacing", GNOME_PAD_SMALL,
+			 "border_width", GNOME_PAD_SMALL,
 			 NULL);
   label = gtk_widget_new (GTK_TYPE_LABEL,
 			  "visible", TRUE,
@@ -869,18 +803,14 @@ gp_config_add_range (GtkWidget  *vbox,
 			  "parent", hbox,
 			  NULL);
   spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0, 0);
-  gtk_widget_set (spinner,
-		  "visible", TRUE,
-		  "width", 80,
-		  NULL);
+  gtk_widget_show (spinner);
   gtk_object_set_user_data (GTK_OBJECT (adjustment), spinner);
   gtk_box_pack_end (GTK_BOX (hbox), spinner, FALSE, TRUE, 0);
-  gtk_object_set (adjustment,
-		  "signal::value_changed", gp_config_value_changed, item,
-		  "object_signal_after::value_changed", gp_config_check, toplevel,
-		  NULL);
+  gtk_signal_connect (adjustment,
+		      "value_changed",
+		      GTK_SIGNAL_FUNC (gp_config_value_changed),
+		      item);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-  gtk_object_set_data (GTK_OBJECT (toplevel), item->path, hbox);
   
   return hbox;
 }
@@ -901,21 +831,17 @@ gp_config_add_section (GtkBox      *parent,
       box = gtk_widget_new (GTK_TYPE_VBOX,
 			    "visible", TRUE,
 			    "parent", widget,
-			    "border_width", CONFIG_IBOX_BORDER,
-			    "spacing", CONFIG_IBOX_SPACING,
 			    NULL);
     }
   else
     {
       box = gtk_widget_new (GTK_TYPE_VBOX,
 			    "visible", TRUE,
-			    "border_width", 0,
-			    "spacing", CONFIG_OBOX_SPACING,
 			    NULL);
       widget = box;
     }
 
-  gtk_box_pack_start (parent, widget, FALSE, TRUE, 0);
+  gtk_box_pack_start (parent, widget, TRUE, TRUE, 0);
   
   return box;
 }
@@ -936,6 +862,7 @@ gp_config_create_page (GSList		*item_slist,
       GSList *node = item_slist;
       
       item_slist = node->next;
+      g_slist_free_1 (node);
       
       page_name = _ (item->name);
     }
@@ -944,18 +871,19 @@ gp_config_create_page (GSList		*item_slist,
   
   page = gtk_widget_new (GTK_TYPE_VBOX,
 			 "visible", TRUE,
-			 "border_width", CONFIG_OBOX_BORDER,
-			 "spacing", CONFIG_OBOX_SPACING,
+			 "border_width", GNOME_PAD_SMALL,
+			 "spacing", GNOME_PAD_SMALL,
 			 NULL);
-  gnome_property_box_append_page (pbox, page, gtk_label_new (_ (page_name)));
   
   while (item_slist)
     {
-      ConfigItem *item = item_slist->data;
+      GSList *node = item_slist;
+      ConfigItem *item = node->data;
       
       if (!item->path)						/* page */
 	break;
-      item_slist = item_slist->next;
+      item_slist = node->next;
+      g_slist_free_1 (node);
       
       if (item->min == -2 && item->max == -2)			/* section */
 	vbox = gp_config_add_section (GTK_BOX (page), _ (item->name));
@@ -973,6 +901,8 @@ gp_config_create_page (GSList		*item_slist,
 	}
     }
   
+  gnome_property_box_append_page (pbox, page, gtk_label_new (_ (page_name)));
+  
   return item_slist;
 }
 
@@ -985,13 +915,12 @@ gp_config_popup (void)
   
   if (!dialog)
     {
-      GSList *fslist, *slist = NULL;
+      GSList *slist = NULL;
       guint i;
       
       for (i = 0; i < gp_n_config_items; i++)
 	slist = g_slist_prepend (slist, gp_config_items + i);
       slist = g_slist_reverse (slist);
-      fslist = slist;
       
       dialog = gnome_property_box_new ();
       gtk_widget_set (dialog,
@@ -1000,15 +929,15 @@ gp_config_popup (void)
 		      "signal::apply", gp_config_apply_tmp_values, NULL,
 		      "signal::apply", gp_init_gui, NULL,
 		      "signal::destroy", gp_config_reset_tmp_values, NULL,
-		      "signal::destroy", gtk_widget_destroyed, &dialog,
 		      NULL);
       
       while (slist)
 	slist = gp_config_create_page (slist, GNOME_PROPERTY_BOX (dialog));
-      g_slist_free (fslist);
-
+      
+      gtk_widget_set (dialog,
+		      "signal::destroy", gtk_widget_destroyed, &dialog,
+		      NULL);
       gtk_quit_add_destroy (1, GTK_OBJECT (dialog));
-      gp_config_check (dialog);
     }
   
   gtk_widget_show (dialog);
