@@ -109,7 +109,7 @@ static void		gwmh_task_queue_full	  (GwmhTask          *task,
 						   GwmhTaskInfoMask   imask,
 						   GwmhTaskInfoMask   notify_mask);
 static void		gwmh_desk_update	  (GwmhDeskInfoMask   imask);
-static gboolean		gwmh_task_update	  (GwmhTask          *task,
+static void		gwmh_task_update	  (GwmhTask          *task,
 						   GwmhTaskInfoMask   imask,
 						   gboolean           skip_notify);
 static void		gwmh_desk_notify	  (GwmhDeskInfoMask   imask);
@@ -155,7 +155,6 @@ static GwmhDesk         gwmh_desk = {
   FALSE /* unified_area */,
 };
 static gboolean		gwmh_hack_think_unified_area = FALSE;
-static gboolean		gwmh_hack_violate_client_msg = TRUE;
 
 
 /* --- functions --- */
@@ -1030,7 +1029,7 @@ gwmh_desk_update (GwmhDeskInfoMask imask)
     gwmh_desk_notify (ichanges);
 }
 
-static gboolean
+static void
 gwmh_task_update (GwmhTask        *task,
 		  GwmhTaskInfoMask imask,
 		  gboolean         skip_notify)
@@ -1084,11 +1083,8 @@ gwmh_task_update (GwmhTask        *task,
       get_task_root_and_frame (task);
       if (!task->xframe || !task->sroot)
 	{
-	  /* eek, that's ugly, we need to get rid of this task as
-	   * soon as possible, but *not* right here.
-	   */
-	  gwmh_desk_queue_update (GWMH_DESK_INFO_CLIENT_LIST);
-	  return FALSE;
+	  task_delete (task);
+	  return;
 	}
       if (task->xframe != xframe ||
 	  (task->sroot ? GSTC_PARENT_XWINDOW (task->sroot) : 0) != !xroot)
@@ -1341,8 +1337,6 @@ gwmh_task_update (GwmhTask        *task,
   
   if (!skip_notify && ichanges)
     gwmh_task_notify (task, GWMH_NOTIFY_INFO_CHANGED, ichanges);
-
-  return TRUE;
 }
 
 static gboolean
@@ -1607,19 +1601,12 @@ task_new (GdkWindow *window)
   /* gwmh_task_update () will do this for us:
    * get_task_root_and_frame (task);
    */
-  if (gwmh_task_update (task, GWMH_TASK_INFO_ALL, TRUE))
-    {
-      gwmh_task_notify (task, GWMH_NOTIFY_NEW, 0);
-      gwmh_task_update (task, 0, FALSE);
+  gwmh_task_update (task, GWMH_TASK_INFO_ALL, TRUE);
 
-      return task;
-    }
-  else
-    {
-      task_delete (task);
+  gwmh_task_notify (task, GWMH_NOTIFY_NEW, 0);
+  gwmh_task_update (task, 0, FALSE);
 
-      return NULL;
-    }
+  return task;
 }
 
 static void
@@ -1676,18 +1663,6 @@ task_delete (GwmhTask *task)
   gwmh_desk.client_list = g_list_remove (gwmh_desk.client_list, task);
   g_free (task->name);
   task->name = "DELETED";
-
-  if (g_list_find (gwmh_desk.client_list, task))
-    {
-      g_warning ("deleted task still in client_list\n"); /* FIXME */
-      g_assert_not_reached ();
-    }
-  if (g_slist_find (gwmh_task_queue, task))
-    {
-      g_warning ("deleted task still in queue\n"); /* FIXME */
-      g_assert_not_reached ();
-    }
-    
   g_free (task);
 }
 
@@ -1729,18 +1704,13 @@ client_list_sync (Window *xwindows,
 
       if (window)
 	{
-	  GwmhTask *task = task_new (window);
-
-	  if (task)
-	    {
-	      client_list = g_list_prepend (client_list, task);
-	      clients_changed = TRUE;
-	    }
+	  client_list = g_list_prepend (client_list,
+					task_new (window));
+	  clients_changed = TRUE;
 	  gdk_window_unref (window);
 	}
     }
 
-  g_assert (g_list_length (gwmh_desk.client_list) == g_list_length (client_list)); /* FIXME */
   if (clients_changed)
     {
       g_list_free (gwmh_desk.client_list);
@@ -1857,13 +1827,14 @@ gwmh_task_deiconify (GwmhTask *task)
 {
   g_return_if_fail (task != NULL);
 
-  if (gwmh_task_update (task, GWMH_TASK_INFO_ICONIFIED, FALSE) &&
-      GWMH_TASK_ICONIFIED (task))
+  gwmh_task_update (task, GWMH_TASK_INFO_ICONIFIED, FALSE);
+
+  if (GWMH_TASK_ICONIFIED (task))
     {
       gdk_error_trap_push ();
-      
+
       XMapWindow (GDK_DISPLAY (), task->xwin);
-      
+
       gwmh_sync ();
       gdk_error_trap_pop ();
     }
@@ -1874,8 +1845,7 @@ gwmh_task_focus (GwmhTask *task)
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_FOCUSED, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_FOCUSED, FALSE);
 
   if (wm_protocol_check_support (task->xwin, XA_WM_TAKE_FOCUS))
     send_client_message_32 (task->xwin, task->xwin,
@@ -1899,12 +1869,11 @@ gwmh_task_show (GwmhTask *task)
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task,
-			 (GWMH_TASK_INFO_ICONIFIED |
-			  GWMH_TASK_INFO_FOCUSED |
-			  GWMH_TASK_INFO_GSTATE),
-			 FALSE))
-    return;
+  gwmh_task_update (task,
+		    (GWMH_TASK_INFO_ICONIFIED |
+		     GWMH_TASK_INFO_FOCUSED |
+		     GWMH_TASK_INFO_GSTATE),
+		    FALSE);
 
   if (!GWMH_TASK_SKIP_FOCUS (task) &&
       wm_protocol_check_support (task->xwin, XA_WM_TAKE_FOCUS))
@@ -1952,8 +1921,7 @@ gwmh_task_set_gstate_flags (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_GSTATE, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_GSTATE, FALSE);
 
   if ((GWMH_TASK_GSTATE (task) & flags) != flags)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -1968,8 +1936,7 @@ gwmh_task_unset_gstate_flags (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_GSTATE, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_GSTATE, FALSE);
 
   if (GWMH_TASK_GSTATE (task) & flags)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -1984,8 +1951,7 @@ gwmh_task_set_ghint_flags (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_GHINTS, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_GHINTS, FALSE);
 
   if ((GWMH_TASK_GHINTS (task) & flags) != flags)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -2000,8 +1966,7 @@ gwmh_task_unset_ghint_flags (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_GHINTS, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_GHINTS, FALSE);
 
   if (GWMH_TASK_GHINTS (task) & flags)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -2016,8 +1981,7 @@ gwmh_task_set_app_state (GwmhTask        *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_APP_STATE, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_APP_STATE, FALSE);
 
   if (GWMH_TASK_APP_STATE (task) != app_state)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -2087,8 +2051,7 @@ gwmh_task_set_layer (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_LAYER, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_LAYER, FALSE);
 
   if (task->layer != layer)
     send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
@@ -2105,8 +2068,7 @@ gwmh_task_set_area (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_DESKTOP | GWMH_TASK_INFO_AREA, FALSE))
-    return;
+  gwmh_task_update (task, GWMH_TASK_INFO_DESKTOP | GWMH_TASK_INFO_AREA, FALSE);
   
   if (desktop >= gwmh_desk.n_desktops ||
       harea >= gwmh_desk.n_hareas ||
@@ -2119,10 +2081,7 @@ gwmh_task_set_area (GwmhTask *task,
   gwmh_sync ();
   gwmh_freeze_syncs ();
 
-  /* ugly hack for buggy window managers
-   * that violate the spec and expect us to modify properties directly
-   */
-  if (gwmh_hack_violate_client_msg)
+  if (1) /* ugly hack for buggy window managers */
     {
       long data[2];
       
@@ -2140,19 +2099,17 @@ gwmh_task_set_area (GwmhTask *task,
 
       gdk_error_trap_pop ();
     }
-  else
-    {
-      if (desktop != task->desktop)
-	send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
-				GWMHA_WIN_WORKSPACE,
-				SubstructureNotifyMask,
-				2, desktop, CurrentTime);
-      send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
-			      GWMHA_WIN_AREA,
-			      SubstructureNotifyMask,
-			      3, harea, varea, CurrentTime);
-    }
-  
+
+  if (desktop != task->desktop)
+    send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
+			    GWMHA_WIN_WORKSPACE,
+			    SubstructureNotifyMask,
+			    2, desktop, CurrentTime);
+  send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
+			  GWMHA_WIN_AREA,
+			  SubstructureNotifyMask,
+			  3, harea, varea, CurrentTime);
+
   gwmh_thaw_syncs ();
 }
 
@@ -2162,14 +2119,12 @@ gwmh_task_set_desktop (GwmhTask *task,
 {
   g_return_if_fail (task != NULL);
 
-  if (!gwmh_task_update (task, GWMH_TASK_INFO_DESKTOP, FALSE) ||
-      desktop >= gwmh_desk.n_desktops || task->desktop == desktop)
+  gwmh_task_update (task, GWMH_TASK_INFO_DESKTOP, FALSE);
+  
+  if (desktop >= gwmh_desk.n_desktops || task->desktop == desktop)
     return;
   
-  /* ugly hack for buggy window managers
-   * that violate the spec and expect us to modify properties directly
-   */
-  if (gwmh_hack_violate_client_msg)
+  if (1) /* ugly hack for buggy window managers */
     {
       long data[2];
       
@@ -2179,14 +2134,14 @@ gwmh_task_set_desktop (GwmhTask *task,
       if (desktop != task->desktop)
 	XChangeProperty (GDK_DISPLAY (), task->xwin, GWMHA_WIN_WORKSPACE,
 			 XA_CARDINAL, 32, PropModeReplace, (unsigned char*) data, 1);
-      
+
       gdk_error_trap_pop ();
     }
-  else
-    send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
-			    GWMHA_WIN_WORKSPACE,
-			    SubstructureNotifyMask,
-			    2, desktop, CurrentTime);
+
+  send_client_message_32 (GDK_ROOT_WINDOW (), task->xwin,
+			  GWMHA_WIN_WORKSPACE,
+			  SubstructureNotifyMask,
+			  2, desktop, CurrentTime);
 }
 
 GwmhDesk*
@@ -2230,11 +2185,9 @@ gwmh_desk_set_current_desktop (guint desktop)
 }
 
 void
-gwmh_desk_set_hack_values (gboolean unified_area,
-			   gboolean violate_client_msg)
+gwmh_desk_set_hack_values (gboolean unified_area)
 {
   gwmh_hack_think_unified_area = unified_area != FALSE;
-  gwmh_hack_violate_client_msg = violate_client_msg != FALSE;
 
   gwmh_desk_update (GWMH_DESK_INFO_N_AREAS);
 }
