@@ -15,13 +15,15 @@ struct _FishProp {
 	char *image;
 	int frames;
 	float speed;
+	int rotate;
 };
 
 static FishProp defaults = {
-	"Wanda",
-	NULL,
-	3,
-	1.0
+	"Wanda", /*name*/
+	NULL, /*image*/
+	3, /*frames*/
+	1.0, /*speed*/
+	TRUE /*rotate*/
 };
 
 typedef struct _Fish Fish;
@@ -39,16 +41,63 @@ struct _Fish {
 	GtkWidget * fortune_less; 
 	GtkWidget * aboutbox;
 	GtkWidget * pb;
+	PanelSizeType size;
+	PanelOrientType orient;
 };
 
+#define IS_ROT(f) ((f)->prop.rotate && ((f)->orient == ORIENT_LEFT || (f)->orient == ORIENT_RIGHT))
+
 GtkWidget *bah_window = NULL;
 
+static void
+load_image_file(Fish *fish)
+{
+	GdkImlibImage *pix;
+	int w, h;
+
+	if(fish->pix)
+		gdk_pixmap_unref(fish->pix);
+	
+	pix = gdk_imlib_load_image(fish->prop.image);
+	if(pix) {
+		if(fish->size==SIZE_TINY && pix->rgb_height>24) {
+			h = 24;
+			w = pix->rgb_width*(24.0/pix->rgb_height);
+		} else {
+			h = pix->rgb_height;
+			w = pix->rgb_width;
+		}
+
+		if(IS_ROT(fish)) {
+			int t;
+			t = w; w = h; h = t;
+			gdk_imlib_rotate_image(pix,90);
+		}
+
+		gdk_imlib_render (pix, w, h);
+		fish->w = w;
+		fish->h = h;
+		fish->pix = gdk_imlib_move_image(pix);
+		gdk_imlib_destroy_image(pix);
+	} else {
+ 		g_assert(bah_window);
+ 		g_assert(bah_window->window);
+		if(IS_ROT(fish)) {
+			fish->w = 24;
+			fish->h = fish->prop.frames*24;
+		} else {
+			fish->w = fish->prop.frames*24;
+			fish->h = 24;
+		}
+ 		fish->pix = gdk_pixmap_new(bah_window->window,
+ 					   fish->w,fish->h,-1);
+	}
+}
 
 static void
 load_properties(Fish *fish)
 {
 	char buf[256];
-	GdkImlibImage *pix;
 	if(!defaults.image)
 		defaults.image = gnome_unconditional_pixmap_file ("fish/fishanim.png");
 
@@ -71,27 +120,12 @@ load_properties(Fish *fish)
 	if(fish->prop.speed<0.1) fish->prop.speed = 0.1;
 	if(fish->prop.speed>10.0) fish->prop.speed = 10.0;
 
+	g_snprintf(buf,256,"fish/rotate=%s",defaults.rotate?"true":"false");
+	fish->prop.rotate = gnome_config_get_bool(buf);
+
 	gnome_config_pop_prefix();
 
-	if(fish->pix)
-		gdk_pixmap_unref(fish->pix);
-	
-	pix = gdk_imlib_load_image(fish->prop.image);
-	if(pix) {
-		gdk_imlib_render (pix, pix->rgb_width,
-				  pix->rgb_height);
-		fish->w = pix->rgb_width;
-		fish->h = pix->rgb_height;
-		fish->pix = gdk_imlib_move_image(pix);
-		gdk_imlib_destroy_image(pix);
-	} else {
-		g_assert(bah_window);
-		g_assert(bah_window->window);
-		fish->w = fish->prop.frames*48;
-		fish->h = 48;
-		fish->pix = gdk_pixmap_new(bah_window->window,
-					   fish->w,fish->h,-1);
-	}
+	load_image_file(fish);
 }
 
 static char *
@@ -111,11 +145,19 @@ fish_draw(GtkWidget *darea, Fish *fish)
 	if(!GTK_WIDGET_REALIZED(fish->darea))
 		return;
 	
-	gdk_draw_pixmap(fish->darea->window,
-			fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
-			fish->pix,
-			(fish->w*fish->curpix)/fish->prop.frames,
-			0, 0, 0, -1, -1);
+	if(IS_ROT(fish))
+		gdk_draw_pixmap(fish->darea->window,
+				fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
+				fish->pix,
+				0,
+				(fish->h*fish->curpix)/fish->prop.frames,
+				0, 0, -1, -1);
+	else
+		gdk_draw_pixmap(fish->darea->window,
+				fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
+				fish->pix,
+				(fish->w*fish->curpix)/fish->prop.frames,
+				0, 0, 0, -1, -1);
 }
 
 static int
@@ -133,10 +175,8 @@ static void
 apply_properties(Fish *fish) 
 {
 	char * tmp;
-	/* xgettext:no-c-format */
 	const char * title_format = _("%s the Fish");
 	const char * label_format = _("%s the GNOME Fish Says:");
-	GdkImlibImage *pix;
 
 	if (fish->fortune_dialog != NULL) { 
 		tmp = splice_name(title_format, fish->prop.name);
@@ -148,27 +188,27 @@ apply_properties(Fish *fish)
 		g_free(tmp);
 	}
 	
-	if(fish->pix)
-		gdk_pixmap_unref(fish->pix);
-	
-	pix = gdk_imlib_load_image(fish->prop.image);
-	gdk_imlib_render (pix, pix->rgb_width,
-			  pix->rgb_height);
-	fish->w = pix->rgb_width;
-	fish->h = pix->rgb_height;
-	fish->pix = gdk_imlib_move_image(pix);
-	gdk_imlib_destroy_image(pix);
+	load_image_file(fish);
 
-	gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
-			      fish->w/fish->prop.frames, fish->h);
-
-	gtk_widget_set_usize(fish->darea, fish->w/fish->prop.frames, fish->h);
+	if(IS_ROT(fish)) {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea), fish->w,
+				      fish->h/fish->prop.frames);
+		gtk_widget_set_usize(fish->darea, fish->w,
+				     fish->h/fish->prop.frames);
+	} else {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
+				      fish->w/fish->prop.frames, fish->h);
+		gtk_widget_set_usize(fish->darea, fish->w/fish->prop.frames,
+				     fish->h);
+	}
 
 	if(fish->timeout_id)
 		gtk_timeout_remove(fish->timeout_id);
         fish->timeout_id = gtk_timeout_add(fish->prop.speed*1000,
 					   fish_timeout,fish);
 	fish->curpix = 0;
+
+	fish_timeout(fish);
 }
 
 static void
@@ -183,6 +223,8 @@ apply_cb(GnomePropertyBox * pb, int page, Fish *fish)
 		gtk_object_get_data(GTK_OBJECT(pb), "frames");
 	GtkAdjustment *speed =
 		gtk_object_get_data(GTK_OBJECT(pb), "speed");
+	GtkWidget *rotate = gtk_object_get_data(GTK_OBJECT(pb),
+						"rotate");
 
 	if (page != -1) return; /* Only honor global apply */
 
@@ -198,6 +240,8 @@ apply_cb(GnomePropertyBox * pb, int page, Fish *fish)
 	}
 	fish->prop.frames = frames->value;
 	fish->prop.speed = speed->value;
+	fish->prop.rotate =
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rotate));
 
 	apply_properties(fish);
 }
@@ -287,6 +331,14 @@ properties_dialog(AppletWidget *aw, gpointer data)
 				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
 				  GTK_OBJECT(fish->pb));
 
+	w = gtk_check_button_new_with_label(_("Rotate on vertical panels"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),fish->prop.rotate);
+	gtk_signal_connect_object(GTK_OBJECT(w), "toggled",
+				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
+				  GTK_OBJECT(fish->pb));
+	gtk_object_set_data(GTK_OBJECT(fish->pb),"rotate",w);
+	gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 0);
+
 	gnome_property_box_append_page(GNOME_PROPERTY_BOX(fish->pb), vbox,
 				       gtk_label_new(_("Fish")));
 
@@ -304,7 +356,7 @@ properties_dialog(AppletWidget *aw, gpointer data)
 static void 
 update_fortune_dialog(Fish *fish)
 {
-        gboolean fortune_exists;
+	char *fortune_command;
 
 	if ( fish->fortune_dialog == NULL ) {
 		fish->fortune_dialog = 
@@ -346,16 +398,17 @@ update_fortune_dialog(Fish *fish)
 	if (!GTK_WIDGET_VISIBLE(fish->fortune_dialog))
 		gtk_widget_show(fish->fortune_dialog);
 
-	fortune_exists = g_file_exists("/usr/games/fortune") || 
-	                 (gnome_is_program_in_path("fortune") != NULL);
+	fortune_command = g_file_exists("/usr/games/fortune")?
+		g_strdup("/usr/games/fortune"):
+		gnome_is_program_in_path("fortune");
 
-	if (fortune_exists)
+	if(fortune_command) {
                 gnome_less_show_command(GNOME_LESS(fish->fortune_less),
-					g_file_exists("/usr/games/fortune")?
-					"/usr/games/fortune":"fortune");
-	else
+					fortune_command);
+		g_free(fortune_command);
+	} else
                 gnome_less_show_string(GNOME_LESS(fish->fortune_less),
-				       "You do not have fortune installed.");
+				       _("You do not have fortune installed."));
 }
 
 static int 
@@ -374,13 +427,23 @@ fish_clicked_cb(GtkWidget * widget, GdkEventButton * e, Fish *fish)
 static int
 fish_expose(GtkWidget *darea, GdkEventExpose *event, Fish *fish)
 {
-	gdk_draw_pixmap(fish->darea->window,
-			fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
-			fish->pix,
-			((fish->w*fish->curpix)/fish->prop.frames)+
-			event->area.x, event->area.y,
-			event->area.x, event->area.y,
-			event->area.width, event->area.height);
+	if(IS_ROT(fish))
+		gdk_draw_pixmap(fish->darea->window,
+				fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
+				fish->pix,
+				event->area.x,
+				((fish->h*fish->curpix)/fish->prop.frames)+
+				event->area.y,
+				event->area.x, event->area.y,
+				event->area.width, event->area.height);
+	else
+		gdk_draw_pixmap(fish->darea->window,
+				fish->darea->style->fg_gc[GTK_WIDGET_STATE(fish->darea)],
+				fish->pix,
+				((fish->w*fish->curpix)/fish->prop.frames)+
+				event->area.x, event->area.y,
+				event->area.x, event->area.y,
+				event->area.width, event->area.height);
         return FALSE;
 }
 
@@ -394,8 +457,13 @@ create_fish_widget(Fish *fish)
 	style = gtk_widget_get_style(fish->applet);
 	
 	fish->darea = gtk_drawing_area_new();
-	gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
-			      fish->w/fish->prop.frames, fish->h);
+	if(IS_ROT(fish)) {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea), fish->w,
+				      fish->h/fish->prop.frames);
+	} else {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
+				      fish->w/fish->prop.frames, fish->h);
+	}
 	gtk_widget_set_events(fish->darea, gtk_widget_get_events(fish->darea) |
 			      GDK_BUTTON_PRESS_MASK);
 	gtk_signal_connect(GTK_OBJECT(fish->darea), "button_press_event",
@@ -467,6 +535,7 @@ applet_save_session(GtkWidget *w,
 	gnome_config_set_string("fish/image",fish->prop.image);
 	gnome_config_set_int("fish/frames",fish->prop.frames);
 	gnome_config_set_float("fish/speed",fish->prop.speed);
+	gnome_config_set_bool("fish/rotate",fish->prop.rotate);
 	gnome_config_pop_prefix();
 
 	gnome_config_sync();
@@ -493,7 +562,52 @@ applet_destroy(GtkWidget *applet,Fish *fish)
 		gtk_widget_destroy(fish->pb);
 	g_free(fish);
 }
+
+static void
+applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
+{
+	Fish *fish = data;
 	
+	fish->orient = o;
+	
+	load_image_file(fish);
+	
+	if(IS_ROT(fish)) {
+		gtk_widget_set_usize(fish->darea, fish->w,
+				     fish->h/fish->prop.frames);
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea), fish->w,
+				      fish->h/fish->prop.frames);
+	} else {
+		gtk_widget_set_usize(fish->darea, fish->w/fish->prop.frames,
+				     fish->h);
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
+				      fish->w/fish->prop.frames, fish->h);
+	}
+	fish_timeout(fish);
+}
+
+static void
+applet_change_size(GtkWidget *w, PanelSizeType o, gpointer data)
+{
+	Fish *fish = data;
+
+	fish->size = o;
+
+	load_image_file(fish);
+
+	if(IS_ROT(fish)) {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea), fish->w,
+				      fish->h/fish->prop.frames);
+		gtk_widget_set_usize(fish->darea, fish->w,
+				     fish->h/fish->prop.frames);
+	} else {
+		gtk_drawing_area_size(GTK_DRAWING_AREA(fish->darea),
+				      fish->w/fish->prop.frames, fish->h);
+		gtk_widget_set_usize(fish->darea, fish->w/fish->prop.frames,
+				     fish->h);
+	}
+	fish_timeout(fish);
+}
 
 static CORBA_Object
 wanda_activator(PortableServer_POA poa,
@@ -506,14 +620,24 @@ wanda_activator(PortableServer_POA poa,
 
   fish = g_new0(Fish,1);
   
+  fish->size = SIZE_STANDARD;
+  fish->orient = ORIENT_UP;
+
   fish->applet = applet_widget_new(goad_id);
-  
+
   bah_window = gtk_window_new(GTK_WINDOW_POPUP);
   gtk_widget_set_uposition(bah_window,gdk_screen_width()+1,
 			   gdk_screen_height()+1);
   gtk_widget_show_now(bah_window);
   
   load_properties(fish);
+  
+  gtk_signal_connect(GTK_OBJECT(fish->applet),"change_orient",
+		     GTK_SIGNAL_FUNC(applet_change_orient),
+		     fish);
+  gtk_signal_connect(GTK_OBJECT(fish->applet),"change_size",
+		     GTK_SIGNAL_FUNC(applet_change_size),
+		     fish);
 
   /*gtk_widget_realize(applet);*/
   create_fish_widget(fish);
