@@ -99,7 +99,6 @@ struct terminal_config {
 	int transparent;
 	int shaded;
 	int background_pixmap;
-	int terminal_id;			/* terminal id for this terminal */
 	char *pixmap_file;
         char *window_title;                     /* the window title */
 	char *wordclass;			/* select-by-word character class */
@@ -117,9 +116,6 @@ GtkWidget *initial_term = NULL;
 /* A list of all the open terminals */
 GList *terminals = 0;
 
-/* current new terminal window id */
-static int terminal_id = 0;
-
 typedef struct {
 	GtkWidget *prop_win;
 	GtkWidget *blink_checkbox;
@@ -185,7 +181,7 @@ void toggle_secure_keyboard_cmd (GtkWidget *w, ZvtTerm *term);
 static int popup_menu_cmd (ZvtTerm *term, GdkEventButton *event,
 			   GtkWidget *menu, GnomeUIInfo *uiinfo);
 
-GtkWidget *new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry, int termid);
+GtkWidget *new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry);
 GtkWidget *new_terminal     (GtkWidget *widget, ZvtTerm *term);
 
 static void parse_an_arg (poptContext state,
@@ -201,20 +197,14 @@ about_terminal_cmd (void)
         GtkWidget *about;
 
         const gchar *authors[] = {
-	     N_("Zvt terminal widget: "
-		"    Michael Zucchi (zucchi@zedzone.mmc.com.au)"),
-	     N_("GNOME terminal: "
-		"    Miguel de Icaza (miguel@kernel.org)"),
-	     N_("    Erik Troan (ewt@redhat.com)"),
+		"Zvt terminal widget: "
+		"    Michael Zucchi (zucchi@zedzone.mmc.com.au)",
+		"GNOME terminal: "
+		"    Miguel de Icaza (miguel@kernel.org)",
+		"    Erik Troan (ewt@redhat.com)",
 		NULL
 	};
 
-#ifdef ENABLE_NLS
-	authors[0]=_(authors[0]);
-	authors[1]=_(authors[1]);
-	authors[2]=_(authors[2]);
-#endif
-
         about = gnome_about_new (_("GNOME Terminal"), VERSION,
 				 "(C) 1998, 1999 the Free Software Foundation",
 				 authors,
@@ -505,7 +495,6 @@ load_config (char *class)
 	cfg->window_title = NULL;
 
 	cfg->termname = NULL;
-	cfg->terminal_id = 0;
 
 	cfg->update_records = ZVT_TERM_DO_UTMP_LOG|ZVT_TERM_DO_WTMP_LOG;
 
@@ -1625,10 +1614,7 @@ get_shell_name (char **shell, char **name, int isLogin)
 	g_assert (*shell != NULL);
 
 	only_name = strrchr (*shell, '/');
-	if (only_name != NULL)
-		only_name++;
-	else
-		only_name = *shell;
+	only_name++;
 	
 	if (isLogin){
 		len = strlen (only_name);
@@ -1953,7 +1939,7 @@ show_pty_error_dialog (int errcode)
 }
 
 GtkWidget *
-new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry, int termid)
+new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 {
 	GtkWidget *app, *hbox, *scrollbar;
 	ZvtTerm   *term;
@@ -1972,8 +1958,6 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry, i
 
 	cfg = terminal_config_dup (cfg_in);
 
-	cfg->terminal_id = termid;
-
 	/* Setup the environment for the gnome-terminals:
 	 *
 	 * TERM is set to xterm (which is what zvt emulates)
@@ -2027,7 +2011,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry, i
 	if (cfg->window_title) {
 	  gtk_window_set_title(GTK_WINDOW(app), cfg->window_title);
  	}
-	sprintf(winclass, "GnomeTerminal.%d", termid);
+	sprintf(winclass, "GnomeTerminal:%p", app);
 	gtk_window_set_wmclass (GTK_WINDOW (app), "GnomeTerminal", winclass);
 	gtk_window_set_policy(GTK_WINDOW (app), TRUE, TRUE, TRUE);
 
@@ -2202,7 +2186,7 @@ new_terminal (GtkWidget *widget, ZvtTerm *term)
 
 	cfg = gtk_object_get_data (GTK_OBJECT (term), "config");
 
-	return new_terminal_cmd (NULL, cfg, NULL, terminal_id++);
+	return new_terminal_cmd (NULL, cfg, NULL);
 }
 
 static gboolean
@@ -2228,9 +2212,7 @@ load_session ()
 		char *class, *class_path;
 		int argc;
 		char *prefix = g_strdup_printf ("%s%d/", file, i);
-		int termid;
-		char termconf[16];
-
+	  
 		gnome_config_push_prefix (prefix);
 
 		/* NAUGHTY: The ICCCM requires that the WM stores
@@ -2261,14 +2243,10 @@ load_session ()
 		if (gnome_config_get_bool ("do_wtmp=true"))
 			cfg->update_records |= ZVT_TERM_DO_WTMP_LOG;
 
-		termid = gnome_config_get_int("terminal_id=-1");
-		if (termid!=-1)
-			terminal_id = termid;
-
 		g_free(class);
 		gnome_config_pop_prefix();
 
-		new_terminal_cmd (argv, cfg, geom, terminal_id++);
+		new_terminal_cmd (argv, cfg, geom);
 
 		g_free(cfg);
 		g_free (geom);
@@ -2341,7 +2319,6 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 		g_string_free (geom, TRUE);
 		
 		gnome_config_set_string("class", cfg->class);
-		gnome_config_set_int("terminal_id", cfg->terminal_id);
 		
 		if (top == initial_term){
 			int n;
@@ -2680,7 +2657,7 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	terminal_config_free (cmdline_config);
 	
 	if (!load_session ())
-		new_terminal_cmd (initial_command, default_config, initial_global_geometry, terminal_id++);
+		new_terminal_cmd (initial_command, default_config, initial_global_geometry);
 
 	terminal_config_free (default_config);
 
