@@ -131,7 +131,7 @@ write_one_client (const Client *client)
 				       &numbers[number_count]);
 	  if (found)
 	    {
-	      saved = !(!strcmp(properties[i].name, SmRestartStyleHint) &&
+	      saved = !(properties[i].name == SmRestartStyleHint &&
 			numbers[number_count] == SmRestartNever);
 	      
 	      number_names[number_count++] = (char*)properties[i].save_name;
@@ -338,7 +338,7 @@ read_one_client (Client *client)
 	      prop->vals[0].value = (SmPointer) value;
 	      APPEND (client->properties, prop);      
 
-	      if (!strcmp(properties[i].name, GsmPriority))
+	      if (properties[i].name == GsmPriority)
 		client->priority = number;
 	    }
 	  break;
@@ -403,8 +403,8 @@ set_session_name (const char *name)
 	{
 	  if (saved_session_name)
 	    g_free (saved_session_name);
-
-	  if (name)	  
+	  
+	  if (name)
 	    saved_session_name = g_strdup (name);
 	  
 	  name = TRASH_SESSION;
@@ -428,13 +428,13 @@ set_session_name (const char *name)
 void
 set_trash_mode (gboolean new_mode)
 {
-  if (!trashing != !new_mode)
+  if (trashing != new_mode)
     {
       trashing = new_mode;
 
       if (trashing)
 	set_session_name (session_name);
-      else
+      else if (saved_session_name)
 	set_session_name (saved_session_name);
     }
 }
