@@ -76,7 +76,7 @@ struct _helpWindow {
 };
 
 
-static GtkWidget * makeEntryArea(HelpWindow w);
+static void makeEntryArea(HelpWindow w);
 
 /* Callbacks */
 static void quit_cb(void);
@@ -104,6 +104,8 @@ static void setCurrent(HelpWindow w);
 
 static void pageUp(GtkWidget *w, HelpWindow win);
 static void pageDown(GtkWidget *w, HelpWindow win);
+static void spaceUp(GtkWidget *w, HelpWindow win);
+static void spaceDown(GtkWidget *w, HelpWindow win);
 static void focusEnter(GtkWidget *w, HelpWindow win);
 
 static void dndDrop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
@@ -196,13 +198,13 @@ GnomeUIInfo toolbar[] = {
     GNOMEUIINFO_ITEM_STOCK(N_("Reload"), N_("Reload"), reload_page,
 			   GNOME_STOCK_PIXMAP_REFRESH),
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM(N_("Index"), N_("Show Documentation Index"), 
-		     help_gotoindex, contents_xpm),
+    GNOMEUIINFO_ITEM_STOCK(N_("Index"), N_("Show Documentation Index"), 
+		     help_gotoindex, GNOME_STOCK_PIXMAP_HOME),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM(N_("History"), N_("Show History Window"),
 		     ghelpShowHistory, contents_xpm),
-    GNOMEUIINFO_ITEM(N_("BMarks"), N_("Show Bookmarks Window"),
-		     ghelpShowBookmarks, contents_xpm),
+    GNOMEUIINFO_ITEM_STOCK(N_("BMarks"), N_("Show Bookmarks Window"),
+		     ghelpShowBookmarks, GNOME_STOCK_PIXMAP_BOOK_OPEN),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_STOCK(N_("Help"), N_("Help on Help"), help_onhelp, GNOME_STOCK_PIXMAP_HELP),
     GNOMEUIINFO_END
@@ -498,13 +500,16 @@ update_toolbar(HelpWindow w)
 		gtk_widget_set_sensitive(w->tb_forw, queue_isnext(w->queue));
 }
 
-static GtkWidget *
+static void
 makeEntryArea(HelpWindow w)
 {
-    GtkWidget *handleBox, *hbox, *label, *entry;
-    
-    handleBox = gtk_handle_box_new();
+    GtkWidget *dock, *hbox, *label, *entry;
 
+    dock = gnome_dock_item_new ("gnome-help-browser-toolbar1",
+				(GNOME_DOCK_ITEM_BEH_EXCLUSIVE
+				 | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL));
+    gtk_widget_show (dock);
+    
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
     gtk_widget_show(hbox);
@@ -514,16 +519,19 @@ makeEntryArea(HelpWindow w)
     
     entry = gnome_entry_new(NULL);
     gtk_widget_show(entry);
+
     gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry)->entry),
 		       "activate", (GtkSignalFunc)entryChanged, w);
 
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-    gtk_container_add(GTK_CONTAINER(handleBox), hbox);
 
+    gtk_container_add (GTK_CONTAINER (dock), hbox);
+    gnome_dock_add_item (GNOME_DOCK (GNOME_APP (w->app)->dock),
+			 GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
     w->entryBox = GTK_COMBO(entry)->entry;
     
-    return handleBox;
+    return dock;
 }
 
 
@@ -642,6 +650,8 @@ static void init_accel(HelpWindow win)
 {
     static gint page_up_signal = 0;
     static gint page_down_signal = 0;
+    static gint up_signal = 0;
+    static down_signal = 0;
     static gint grab_focus_signal = 0;
 
     GtkAccelGroup *accel_group = gtk_accel_group_get_default();
@@ -659,6 +669,18 @@ static void init_accel(HelpWindow win)
 	 GTK_RUN_FIRST,
 	 gtk_marshal_NONE__NONE,
 	 GTK_TYPE_NONE, 0);
+      up_signal = gtk_object_class_user_signal_new
+	(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb)->klass,
+	 "up",
+	 GTK_RUN_FIRST,
+	 gtk_marshal_NONE__NONE,
+	 GTK_TYPE_NONE, 0);
+      down_signal = gtk_object_class_user_signal_new
+	(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb)->klass,
+	 "down",
+	 GTK_RUN_FIRST,
+	 gtk_marshal_NONE__NONE,
+	 GTK_TYPE_NONE, 0);
     }
     gtk_signal_connect(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb), 
 		       "page_up", 
@@ -666,6 +688,12 @@ static void init_accel(HelpWindow win)
     gtk_signal_connect(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb), 
 		       "page_down", 
 		       GTK_SIGNAL_FUNC(pageDown), win);
+    gtk_signal_connect(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb), 
+		       "up", 
+		       GTK_SIGNAL_FUNC(spaceUp), win);
+    gtk_signal_connect(GTK_OBJECT(GTK_XMHTML (win->helpWidget)->html.vsb), 
+		       "down", 
+		       GTK_SIGNAL_FUNC(spaceDown), win);
     gtk_widget_add_accelerator(GTK_XMHTML (win->helpWidget)->html.vsb, 
 			       "page_up", accel_group, 
 			       'b', 0, 0);
@@ -679,6 +707,12 @@ static void init_accel(HelpWindow win)
     gtk_widget_add_accelerator(GTK_XMHTML (win->helpWidget)->html.vsb, 
 			       "page_down", accel_group, 
 			       GDK_Page_Down, 0, 0);
+    gtk_widget_add_accelerator(GTK_XMHTML (win->helpWidget)->html.vsb, 
+			       "up", accel_group, 
+			       GDK_Up, 0, 0);
+    gtk_widget_add_accelerator(GTK_XMHTML (win->helpWidget)->html.vsb, 
+			       "down", accel_group, 
+			       GDK_Down, 0, 0);
     gtk_widget_add_accelerator(win->entryBox, 
 			       "grab_focus", accel_group, 
 			       'g', 0, 0);
@@ -700,6 +734,22 @@ static void pageDown(GtkWidget *w, HelpWindow win)
     gtk_adjustment_set_value(adj, adj->value + (adj->page_size));
 }
 
+static void spaceUp(GtkWidget *w, HelpWindow win)
+{
+    GtkAdjustment *adj;
+    
+    adj = GTK_ADJUSTMENT(GTK_XMHTML(win->helpWidget)->vsba);
+    gtk_adjustment_set_value(adj, adj->value - (adj->step_increment));
+}
+
+static void spaceDown(GtkWidget *w, HelpWindow win)
+{
+    GtkAdjustment *adj;
+    
+    adj = GTK_ADJUSTMENT(GTK_XMHTML(win->helpWidget)->vsba);
+    gtk_adjustment_set_value(adj, adj->value + (adj->step_increment));
+}
+
 static void dndDrop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		    GtkSelectionData *data, guint info,
 		    guint time, HelpWindow win)
@@ -763,9 +813,7 @@ helpWindowNew(gchar *name,
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 	gtk_widget_show(vbox);
 
-	entryArea = makeEntryArea(w);
-	gtk_widget_show(entryArea);
-	gtk_box_pack_start(GTK_BOX(vbox), entryArea, FALSE, FALSE, 0);
+	makeEntryArea(w);
 
 	/* make the help window */
 	w->helpWidget = gnome_helpwin_new();
