@@ -14,6 +14,8 @@
 #include <gnome-panel.h>
 
 #define HAVE_SAVE_SESSION_SIGNAL 1
+#define HAVE_APPLET_BIND_EVENTS 1
+#define HAVE_PANEL_SIZE 1
 
 BEGIN_GNOME_DECLS
 
@@ -23,6 +25,12 @@ typedef GNOME_Panel_OrientType PanelOrientType;
 #define ORIENT_LEFT GNOME_Panel_ORIENT_LEFT
 #define ORIENT_RIGHT GNOME_Panel_ORIENT_RIGHT
 
+typedef GNOME_Panel_SizeType PanelSizeType;
+#define SIZE_TINY GNOME_Panel_SIZE_TINY
+#define SIZE_STANDARD GNOME_Panel_SIZE_STANDARD
+#define SIZE_LARGE GNOME_Panel_SIZE_LARGE
+#define SIZE_HUGE GNOME_Panel_SIZE_HUGE
+
 typedef GNOME_Panel_BackType PanelBackType;
 #define PANEL_BACK_NONE GNOME_Panel_BACK_NONE
 #define PANEL_BACK_COLOR GNOME_Panel_BACK_COLOR
@@ -42,10 +50,35 @@ struct _AppletWidget
 {
 	GtkPlug			window;
 	
+	/* this is the public data */
+	
 	char			*privcfgpath;
 	char			*globcfgpath;
+	
+	PanelOrientType		orient;			
+	PanelSizeType		size;			
+	
+	/* below this is private data */
 
         gpointer                corbadat; /* CORBA stuff */
+	
+	/*change freezing*/
+	int			frozen_level;
+
+	gboolean		frozen_got_orient;
+	PanelOrientType		frozen_orient;			
+
+	gboolean		frozen_got_size;
+	PanelSizeType		frozen_size;			
+	
+	gboolean		frozen_got_back;
+	PanelBackType		frozen_back_type;			
+	char 			*frozen_back_pixmap;			
+	GdkColor		frozen_back_color;			
+	
+	gboolean		frozen_got_position;
+	int			frozen_position_x;
+	int			frozen_position_y;
 };
 
 struct _AppletWidgetClass
@@ -58,6 +91,9 @@ struct _AppletWidgetClass
 	   that you can update your orientation properly */
 	void (* change_orient) (AppletWidget *applet,
 				GNOME_Panel_OrientType orient);
+	/* when the panel size changes, semantics are the same as above */
+	void (* change_size) (AppletWidget *applet,
+			      GNOME_Panel_SizeType size);
 	/* the panel background changes, the pixmap handeling is likely
 	   to change */
 	void (* back_change) (AppletWidget *applet,
@@ -77,6 +113,12 @@ struct _AppletWidgetClass
 	int (* save_session) (AppletWidget *applet,
 			      char *cfgpath,
 			      char *globcfgpath);
+	/*when the position changes and we selected to get this signal,
+	  it is sent so that you can move some external window along with
+	  the applet, it is not normally sent, so you need to enable it
+	  with the applet_widget_send_position*/
+	void (* change_position) (AppletWidget *applet,
+				  int x, int y);
 };
 
 typedef GtkWidget *(*AppletFactoryActivator)(const char *goad_id, const char **params, int nparams);
@@ -105,6 +147,18 @@ void		applet_widget_set_widget_tooltip(AppletWidget *applet,
    the second button, get the menu, etc ...*/
 void		applet_widget_add		(AppletWidget *applet,
 						 GtkWidget *widget);
+/* this function is the same as above, but you can select if the events
+   are actually bound, most applet writers can use the above, this is
+   just for very special cases*/
+void		applet_widget_add_full		(AppletWidget *applet,
+						 GtkWidget *widget,
+						 int bind_events);
+
+/* bind the events for button2 and button3 on a widget, this is useful
+   when you are added a new widget and want the right click menu and middle
+   button move events to work on it*/
+void		applet_widget_bind_events	(AppletWidget *applet,
+						 GtkWidget *widget);
 
 /* remove the plug from the panel, this will destroy the applet */
 void		applet_widget_remove		(AppletWidget *applet);
@@ -145,6 +199,11 @@ void		applet_widget_register_stock_callback_dir (AppletWidget *applet,
 void		applet_widget_unregister_callback_dir (AppletWidget *applet,
 						       char *name);
 
+/*enable/disable a menu entry*/
+void		applet_widget_callback_set_sensitive	(AppletWidget *applet,
+							 char *name,
+							 int sensitive);
+
 /*get thenumber of applets*/
 int		applet_widget_get_applet_count	(void);
 
@@ -154,9 +213,20 @@ int		applet_widget_get_applet_count	(void);
   session_save handler as it will result in a locked panel*/
 void		applet_widget_sync_config	(AppletWidget *applet);
 
-/* Get the oprientation the applet should use */
+/* Get the orientation the applet should use */
 GNOME_Panel_OrientType	applet_widget_get_panel_orient	(AppletWidget *applet);
 
+/* Get the size the applet should use */
+GNOME_Panel_SizeType	applet_widget_get_panel_size	(AppletWidget *applet);
+
+/* Get the free space for the applet if it's on an edge panel or 0
+   if on a packed panel or on error */
+int		applet_widget_get_free_space	(AppletWidget *applet);
+
+/* sets if the change_position signal is sent*/
+void		applet_widget_send_position	(AppletWidget *applet,
+						 int enable);
+
 /*use this instead of gnome init, if you want multi applet, you also
   have to specify a "start new applet" function which will launch a new
   applet*/
