@@ -15,6 +15,8 @@
 #include "panel-include.h"
 #include "gnome-panel.h"
 
+#define SMALL_ICON_SIZE 20
+
 #define APPLET_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
 			   GDK_BUTTON_RELEASE_MASK |		\
 			   GDK_POINTER_MOTION_MASK |		\
@@ -30,10 +32,10 @@ int applet_count = 0;
 extern int config_sync_timeout;
 extern int applets_to_sync;
 extern int panels_to_sync;
-extern int globals_to_sync;
 extern int need_complete_save;
 
 extern GlobalConfig global_config;
+extern PanelWidget *current_panel;
 
 static void
 move_applet_callback(GtkWidget *widget, AppletInfo *info)
@@ -54,8 +56,12 @@ panel_clean_applet(AppletInfo *info)
 {
 	g_return_if_fail(info != NULL);
 
-	if(info->widget)
+	if(info->widget) {
+		if(info->type == APPLET_STATUS) {
+			status_applet_put_offscreen(info->data);
+		}
 		gtk_widget_destroy(info->widget);
+	}
 }
 
 static void
@@ -109,6 +115,13 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 	case APPLET_MENU:
 		if(strcmp(menu->name,"properties")==0)
 			menu_properties(menu->info->data);
+		else if(strcmp(menu->name,"edit_menus")==0) {
+			char *tmp;
+			if((tmp = gnome_is_program_in_path("gmenu")))  {
+				gnome_execute_async(NULL, 1, &tmp);
+				g_free(tmp);
+			}
+		}
 		break;
 	default: break;
 	}
@@ -120,10 +133,8 @@ applet_menu_deactivate(GtkWidget *w, AppletInfo *info)
 	GtkWidget *panel = get_panel_parent(info->widget);
 	info->menu_age = 0;
 	
-	if(IS_SNAPPED_WIDGET(panel))
-		SNAPPED_WIDGET(panel)->autohide_inhibit = FALSE;
-	else if (IS_CORNER_WIDGET(panel))
-	        CORNER_WIDGET(panel)->autohide_inhibit = FALSE;
+	if(IS_BASEP_WIDGET(panel))
+		BASEP_WIDGET(panel)->autohide_inhibit = FALSE;
 }
 
 static AppletUserMenu *
@@ -149,10 +160,11 @@ applet_add_callback(AppletInfo *info,
 	g_return_if_fail(info != NULL);
 	
 	if((menu=applet_get_callback(info->user_menu,callback_name))==NULL) {
-		menu = g_new(AppletUserMenu,1);
+		menu = g_new0(AppletUserMenu,1);
 		menu->name = g_strdup(callback_name);
 		menu->stock_item = g_strdup(stock_item);
 		menu->text = g_strdup(menuitem_text);
+		menu->sensitive = TRUE;
 		menu->info = info;
 		menu->menuitem = NULL;
 		menu->submenu = NULL;
@@ -196,7 +208,34 @@ applet_remove_callback(AppletInfo *info, char *callback_name)
 		if(menu->text)
 			g_free(menu->text);
 		g_free(menu);
+	} else
+		return; /*it just isn't there*/
+
+	/*make sure the menu is rebuilt*/
+	if(info->menu) {
+		GList *list;
+		for(list=info->user_menu;list!=NULL;list=g_list_next(list)) {
+			AppletUserMenu *menu = list->data;
+			menu->menuitem=NULL;
+			menu->submenu=NULL;
+		}
+		gtk_widget_destroy(info->menu);
+		info->menu=NULL;
+		info->menu_age = 0;
 	}
+}
+
+void
+applet_callback_set_sensitive(AppletInfo *info, char *callback_name, int sensitive)
+{
+	AppletUserMenu *menu;
+
+	g_return_if_fail(info != NULL);
+	
+	if((menu=applet_get_callback(info->user_menu,callback_name))!=NULL)
+		menu->sensitive = sensitive;
+	else
+		return; /*it just isn't there*/
 
 	/*make sure the menu is rebuilt*/
 	if(info->menu) {
@@ -252,9 +291,11 @@ setup_an_item(AppletUserMenu *menu,
 		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu->menuitem),
 					  menu->submenu);
 		gtk_signal_connect (GTK_OBJECT (menu->submenu), "destroy",
-				    GTK_SIGNAL_FUNC (gtk_widget_destroyed),      
+				    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
 				    &menu->submenu);
 	}
+	
+	gtk_widget_set_sensitive(menu->menuitem,menu->sensitive);
 }
 
 static void
@@ -292,10 +333,11 @@ add_to_submenus(AppletInfo *info,
 	/*the user did not give us this sub menu, whoops, will create an empty
 	  one then*/
 	if(!s_menu) {
-		s_menu = g_new(AppletUserMenu,1);
+		s_menu = g_new0(AppletUserMenu,1);
 		s_menu->name = g_strdup(t);
-		s_menu->text = g_strdup(_("???"));
 		s_menu->stock_item = NULL;
+		s_menu->text = g_strdup(_("???"));
+		s_menu->sensitive = TRUE;
 		s_menu->info = info;
 		s_menu->menuitem = NULL;
 		s_menu->submenu = NULL;
@@ -325,15 +367,15 @@ add_to_submenus(AppletInfo *info,
 void
 create_applet_menu(AppletInfo *info)
 {
-	GtkWidget *menuitem;
+	GtkWidget *menuitem, *panel_menu;
 	GList *user_menu = info->user_menu;
+	gchar *pixmap;
 
 	info->menu = gtk_menu_new();
 
 	menuitem = gtk_menu_item_new();
 	setup_menuitem(menuitem,
-		       gnome_stock_pixmap_widget (info->menu,
-						  GNOME_STOCK_PIXMAP_REMOVE),
+		       gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REMOVE),
 		       _("Remove from panel"));
 	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
 			   (GtkSignalFunc) remove_applet_callback,
@@ -341,15 +383,35 @@ create_applet_menu(AppletInfo *info)
 	gtk_menu_append(GTK_MENU(info->menu), menuitem);
 	
 	menuitem = gtk_menu_item_new();
-	setup_menuitem(menuitem, NULL, _("Move applet"));
+	setup_menuitem(menuitem,NULL,_("Move applet"));
 	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
 			   (GtkSignalFunc) move_applet_callback,
 			   info);
 	gtk_menu_append(GTK_MENU(info->menu), menuitem);
-	
+
+	panel_menu = gtk_menu_new();
+	make_panel_submenu(panel_menu,TRUE);
+	menuitem = gtk_menu_item_new ();
+
+	pixmap = gnome_pixmap_file ("gnome-panel.png");
+	if (!pixmap) {
+		g_warning (_("Cannot find pixmap file %s"), "gnome-panel.png");
+		setup_menuitem (menuitem, NULL, _("Panel"));
+	} else {
+		setup_menuitem (menuitem, 
+				gnome_stock_pixmap_widget_at_size (NULL, pixmap,
+								   SMALL_ICON_SIZE,
+								   SMALL_ICON_SIZE),
+				_("Panel"));
+		g_free (pixmap);
+	}
+	gtk_menu_append (GTK_MENU (info->menu), menuitem);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),panel_menu);
+
 	if(user_menu) {
 		menuitem = gtk_menu_item_new();
 		gtk_menu_append(GTK_MENU(info->menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, FALSE);
 		gtk_widget_show(menuitem);
 	}
 
@@ -374,16 +436,14 @@ show_applet_menu(AppletInfo *info, GdkEventButton *event)
 	g_return_if_fail(info!=NULL);
 
 	panel = get_panel_parent(info->widget);
+	current_panel = PANEL_WIDGET (BASEP_WIDGET (panel)->panel);
 
 	if (!info->menu)
 		create_applet_menu(info);
 
-	if(IS_SNAPPED_WIDGET(panel)) {
-		SNAPPED_WIDGET(panel)->autohide_inhibit = TRUE;
-		snapped_widget_queue_pop_down(SNAPPED_WIDGET(panel));
-	} else if (IS_CORNER_WIDGET(panel)) {
-	        CORNER_WIDGET(panel)->autohide_inhibit = TRUE;
-		corner_widget_queue_pop_down(CORNER_WIDGET(panel));
+	if(IS_BASEP_WIDGET(panel)) {
+		BASEP_WIDGET(panel)->autohide_inhibit = TRUE;
+		basep_widget_queue_autohide(BASEP_WIDGET(panel));
 	}
 	info->menu_age = 0;
 	gtk_menu_popup(GTK_MENU(info->menu), NULL, NULL,
@@ -415,7 +475,6 @@ applet_button_press (GtkWidget *widget,GdkEventButton *event, AppletInfo *info)
 static void
 applet_destroy(GtkWidget *w, AppletInfo *info)
 {
-	AppletType type;
 	GList *li;
 	
 	g_return_if_fail(info != NULL);
@@ -474,7 +533,7 @@ register_toy(GtkWidget *applet,
 				      ~( GDK_POINTER_MOTION_MASK |
 					 GDK_POINTER_MOTION_HINT_MASK));
 
-	info = g_new(AppletInfo,1);
+	info = g_new0(AppletInfo,1);
 	info->applet_id = applet_count;
 	info->type = type;
 	info->widget = applet;
@@ -514,10 +573,12 @@ register_toy(GtkWidget *applet,
 	/*we will need to save this applet's config now*/
 	applets_to_sync = TRUE;
 
-	if(panel_widget_add_full(panel, applet, pos, TRUE)==-1) {
+	/*add at the beginning if pos == -1*/
+	if(panel_widget_add_full(panel, applet, pos>=0?pos:0, TRUE,
+				 pos>=0?FALSE:TRUE)==-1) {
 		GSList *list;
 		for(list = panels; list != NULL; list = g_slist_next(list))
-			if(panel_widget_add_full(panel, applet, pos, TRUE)!=-1)
+			if(panel_widget_add_full(panel, applet, 0,TRUE,FALSE)!=-1)
 				break;
 		if(!list) {
 			/*can't put it anywhere, clean up*/
@@ -551,6 +612,7 @@ register_toy(GtkWidget *applet,
 				 applet_drag_types, 1);*/
 
 	orientation_change(info,panel);
+	size_change(info,panel);
 	back_change(info,panel);
 
 	return TRUE;
