@@ -13,6 +13,8 @@ BEGIN_GNOME_DECLS
 #define SWALLOW_ID "Swallow"
 #define EXTERN_ID "Extern"
 #define LAUNCHER_ID "Launcher"
+#define LOCK_ID "Lock"
+#define STATUS_ID "Status"
 
 typedef enum {
 	APPLET_EXTERN,
@@ -23,7 +25,9 @@ typedef enum {
 	APPLET_LOGOUT,
 	APPLET_SWALLOW,
 	APPLET_LAUNCHER,
-	APPLET_EMPTY
+	APPLET_EMPTY,
+	APPLET_LOCK,
+	APPLET_STATUS
 } AppletType;
 
 typedef struct _AppletUserMenu AppletUserMenu;
@@ -33,6 +37,7 @@ struct _AppletUserMenu {
 	char *name;
 	char *stock_item;
 	char *text;
+	int sensitive;
 	AppletInfo *info;
 	GtkWidget *menuitem;
 	GtkWidget *submenu;
@@ -64,6 +69,9 @@ void applet_add_callback(AppletInfo *info,
 			 char *menuitem_text);
 void applet_remove_callback(AppletInfo *info,
 			    char *callback_name);
+void applet_callback_set_sensitive(AppletInfo *info,
+				   char *callback_name,
+				   int sensitive);
 void show_applet_menu(AppletInfo *info, GdkEventButton *event);
 
 END_GNOME_DECLS
