@@ -2,12 +2,16 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gnome.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libart_lgpl/art_affine.h>
+#include <libart_lgpl/art_rgb_pixbuf_affine.h>
 #include "button-widget.h"
 #include "panel-widget.h"
+#include "panel-types.h"
+#include "panel_config_global.h"
+#include "rgb-stuff.h"
 
-/*FIXME: make this global, and use it everywhere*/
-#define SMALL_ICON_SIZE 20
-#define BIG_ICON_SIZE   48
+extern GlobalConfig global_config;
 
 static void button_widget_class_init	(ButtonWidgetClass *klass);
 static void button_widget_init		(ButtonWidget      *button);
@@ -40,10 +44,11 @@ typedef void (*VoidSignal) (GtkObject * object,
 static GList *buttons=NULL;
 
 /*the tiles go here*/
-static GdkPixmap *tiles_up[LAST_TILE]={NULL,NULL,NULL,NULL};
-static GdkBitmap *tiles_up_mask[LAST_TILE]={NULL,NULL,NULL,NULL};
-static GdkPixmap *tiles_down[LAST_TILE]={NULL,NULL,NULL,NULL};
-static GdkBitmap *tiles_down_mask[LAST_TILE]={NULL,NULL,NULL,NULL};
+struct {
+	GdkPixbuf *tiles_up[LAST_TILE];
+	GdkPixbuf *tiles_down[LAST_TILE];
+} tiles = {{NULL}}; /*ansi C trick to make it all 0*/
+
 static int tile_border[LAST_TILE]={0,0,0,0};
 static int tile_depth[LAST_TILE]={0,0,0,0};
 
@@ -67,9 +72,8 @@ button_widget_get_type ()
 			sizeof (ButtonWidgetClass),
 			(GtkClassInitFunc) button_widget_class_init,
 			(GtkObjectInitFunc) button_widget_init,
-			NULL,
-			NULL,
-			(GtkClassInitFunc) NULL
+			(GtkArgSetFunc) NULL,
+			(GtkArgGetFunc) NULL,
 		};
 
 		button_widget_type = gtk_type_unique (GTK_TYPE_WIDGET,
@@ -86,7 +90,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint button_widget_signals[LAST_SIGNAL] = {0};
+static int button_widget_signals[LAST_SIGNAL] = {0};
 
 static void
 marshal_signal_void (GtkObject * object,
@@ -158,6 +162,22 @@ button_widget_class_init (ButtonWidgetClass *class)
 	
 }
 
+static void
+setup_no_alpha(ButtonWidget *button)
+{
+	button->no_alpha = FALSE;
+	if(!tiles_enabled[button->tile] ||
+	   global_config.tile_when_over)
+		return;
+	if(button->pressed) {
+		if(!tiles.tiles_down[button->tile]->art_pixbuf->has_alpha)
+			button->no_alpha = TRUE;
+	} else {
+		if(!tiles.tiles_up[button->tile]->art_pixbuf->has_alpha)
+			button->no_alpha = TRUE;
+	}
+}
+
 static void
 button_widget_realize(GtkWidget *widget)
 {
@@ -244,53 +264,77 @@ button_widget_destroy(GtkWidget *w, gpointer data)
 {
 	ButtonWidget *button = BUTTON_WIDGET(w);
 
-	if(button->pixmap)
-		gdk_pixmap_unref(button->pixmap);
-	button->pixmap = NULL;
-	if(button->mask)
-		gdk_bitmap_unref(button->mask);
-	button->mask = NULL;
+	if(button->pixbuf)
+		gdk_pixbuf_unref(button->pixbuf);
+	button->pixbuf = NULL;
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
 
 	g_free(button->text);
 
 	buttons = g_list_remove(buttons,button);
 }
 
+static GdkPixbuf *
+loadup_file(char *file)
+{
+	GdkPixbuf *pb = NULL;
+	
+	if(!file) {
+		return NULL;
+	}
+
+	if(*file!='/') {
+		char *f;
+		f = gnome_pixmap_file (file);
+		if(f) {
+			pb = gdk_pixbuf_new_from_file(f);
+			g_free(f);
+		}
+	} else
+		pb = gdk_pixbuf_new_from_file (file);
+	return pb;
+}
+
+
+#define SCALE(x) (((x)*size)/48.0)
+
 static void
-draw_arrow(GdkPoint *points, PanelOrientType orient)
+draw_arrow(GdkPoint *points, PanelOrientType orient, int size)
 {
 	switch(orient) {
 	case ORIENT_UP:
-		points[0].x = BIG_ICON_SIZE-12;
-		points[0].y = 10;
-		points[1].x = BIG_ICON_SIZE-4;
-		points[1].y = 10;
-		points[2].x = BIG_ICON_SIZE-8;
-		points[2].y = 3;
+		points[0].x = SCALE(48-12);
+		points[0].y = SCALE(10);
+		points[1].x = SCALE(48-4);
+		points[1].y = SCALE(10);
+		points[2].x = SCALE(48-8);
+		points[2].y = SCALE(3);
 		break;
 	case ORIENT_DOWN:
-		points[0].x = 4;
-		points[0].y = BIG_ICON_SIZE - 10;
-		points[1].x = 12;
-		points[1].y = BIG_ICON_SIZE - 10;
-		points[2].x = 8;
-		points[2].y = BIG_ICON_SIZE - 3;
+		points[0].x = SCALE(4);
+		points[0].y = SCALE(48 - 10);
+		points[1].x = SCALE(12);
+		points[1].y = SCALE(48 - 10);
+		points[2].x = SCALE(8);
+		points[2].y = SCALE(48 - 3);
 		break;
 	case ORIENT_LEFT:
-		points[0].x = 10;
-		points[0].y = 4;
-		points[1].x = 10;
-		points[1].y = 12;
-		points[2].x = 3;
-		points[2].y = 8;
+		points[0].x = SCALE(10);
+		points[0].y = SCALE(4);
+		points[1].x = SCALE(10);
+		points[1].y = SCALE(12);
+		points[2].x = SCALE(3);
+		points[2].y = SCALE(8);
 		break;
 	case ORIENT_RIGHT:
-		points[0].x = BIG_ICON_SIZE - 10;
-		points[0].y = BIG_ICON_SIZE - 12;
-		points[1].x = BIG_ICON_SIZE - 10;
-		points[1].y = BIG_ICON_SIZE - 4;
-		points[2].x = BIG_ICON_SIZE - 3;
-		points[2].y = BIG_ICON_SIZE - 8;
+		points[0].x = SCALE(48 - 10);
+		points[0].y = SCALE(48 - 12);
+		points[1].x = SCALE(48 - 10);
+		points[1].y = SCALE(48 - 4);
+		points[2].x = SCALE(48 - 3);
+		points[2].y = SCALE(48 - 8);
 		break;
 	}
 }
@@ -303,88 +347,94 @@ button_widget_real_draw(GtkWidget *widget, GdkRectangle *area)
 }
 
 void
-button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap, int offx, int offy)
+button_widget_draw(ButtonWidget *button, guchar *rgb, int rowstride)
 {
 	GtkWidget *widget = GTK_WIDGET(button);
 	GtkWidget *pwidget;
-	GdkPixmap *tile = NULL;
-	GdkBitmap *tile_mask = NULL;
-	GdkGC *gc;
+	PanelWidget *panel = PANEL_WIDGET(widget->parent);
+	int size = panel_widget_get_pixel_size(panel);
 	/*offset for pressed buttons*/
-	int off = button->in_button&&button->pressed?tile_depth[button->tile]:0;
+	int off = button->in_button&&button->pressed?SCALE(tile_depth[button->tile]):0;
 	/*border to not draw when drawing a tile*/
-	int border = tiles_enabled[button->tile]?tile_border[button->tile]:0;
-	int i;
+	int border = tiles_enabled[button->tile]?SCALE(tile_border[button->tile]):0;
 	 
-	if(!GTK_WIDGET_REALIZED(button))
-		return;
+	button->size = size;
 	
 	pwidget = widget->parent;
 	
-	gc = gdk_gc_new(pixmap);
-
 	if(tiles_enabled[button->tile]) {
 		if(button->pressed && button->in_button) {
-			tile = tiles_down[button->tile];
-			tile_mask = tiles_down_mask[button->tile];
-		} else {
-			tile = tiles_up[button->tile];
-			tile_mask = tiles_up_mask[button->tile];
-		}
-	}
-
-	if(tiles_enabled[button->tile] && tile) {
-		if (tile_mask) {
-			gdk_gc_set_clip_mask (gc, tile_mask);
-			gdk_gc_set_clip_origin (gc, widget->allocation.x-offx,
-						widget->allocation.y-offy);
-		}
-		gdk_draw_pixmap(pixmap, gc, tile, 0,0,
-				widget->allocation.x-offx,
-				widget->allocation.y-offy,
-				BIG_ICON_SIZE, BIG_ICON_SIZE);
-		if (tile_mask) {
-			gdk_gc_set_clip_mask (gc, NULL);
-			gdk_gc_set_clip_origin (gc, 0, 0);
+			if(tiles.tiles_down[button->tile]) {
+				double affine[6];
+				make_scale_affine(affine,
+						  tiles.tiles_down[button->tile]->art_pixbuf->width,
+						  tiles.tiles_down[button->tile]->art_pixbuf->height,
+						  size);
+				art_rgb_pixbuf_affine (rgb, 0, 0, size, size, rowstride,
+						       tiles.tiles_down[button->tile]->art_pixbuf,
+						       affine, ART_FILTER_NEAREST, NULL);
+			}
+		} else if (!global_config.tile_when_over || button->in_button) {
+			if(tiles.tiles_up[button->tile]) {
+				double affine[6];
+				make_scale_affine(affine,
+						  tiles.tiles_up[button->tile]->art_pixbuf->width,
+						  tiles.tiles_up[button->tile]->art_pixbuf->height,
+						  size);
+				art_rgb_pixbuf_affine (rgb, 0, 0, size, size, rowstride,
+						       tiles.tiles_up[button->tile]->art_pixbuf,
+						       affine, ART_FILTER_NEAREST, NULL);
+			}
 		}
 	}
 
 	if (pixmaps_enabled[button->tile]) {
-		if (button->mask) {
-			gdk_gc_set_clip_mask (gc, button->mask);
-			gdk_gc_set_clip_origin (gc,
-						widget->allocation.x+off-offx,
-						widget->allocation.y+off-offy);
-		}
+		if(button->pixbuf) {
+			double affine[6];
+			double transl[6];
+			make_scale_affine(affine,
+					  button->pixbuf->art_pixbuf->width,
+					  button->pixbuf->art_pixbuf->height,
+					  size);
+			art_affine_translate(transl,-border+off,-border+off);
+			art_affine_multiply(affine,affine,transl);
 
-
-		i = MAX(border-off,0);
-		if(button->pixmap) {
-			gdk_draw_pixmap (pixmap, gc, button->pixmap,
-					 i, i,
-					 widget->allocation.x+i+off-offx,
-					 widget->allocation.y+i+off-offy,
-					 BIG_ICON_SIZE-i-off-border,
-					 BIG_ICON_SIZE-i-off-border);
-		}
-
-		if (button->mask) {
-			gdk_gc_set_clip_mask (gc, NULL);
-			gdk_gc_set_clip_origin (gc, 0, 0);
+			art_rgb_pixbuf_affine((rgb+border*rowstride+border*3),
+					      0, 0,
+					      size-2*border, size-2*border,
+					      rowstride,
+					      button->pixbuf->art_pixbuf,
+					      affine, ART_FILTER_NEAREST, NULL);
 		}
 	}
+}
 
+/* draw the xlib part (arrow/text) */
+void
+button_widget_draw_xlib(ButtonWidget *button, GdkPixmap *pixmap)
+{
+	GtkWidget *widget = GTK_WIDGET(button);
+	GtkWidget *pwidget;
+	GdkGC *gc;
+	int size = button->size;
+	/*offset for pressed buttons*/
+	int off = button->in_button&&button->pressed?SCALE(tile_depth[button->tile]):0;
+	/*border to not draw when drawing a tile*/
+	 
+	gc = gdk_gc_new(pixmap);
+	
+	pwidget = widget->parent;
 	/*draw text*/
 	if (!pixmaps_enabled[button->tile] ||
-	  always_text[button->tile] ||
-	  !button->pixmap) {
+	    always_text[button->tile] ||
+	    !button->pixbuf) {
 		char *text = g_strdup(button->text);
 		int twidth,theight;
 		GdkFont *font;
 		GdkRectangle rect;
 	         
-	        rect.x = widget->allocation.x-offx;
-	        rect.y = widget->allocation.y-offy;
+	        rect.x = 0;
+	        rect.y = 0;
 	        rect.width = widget->allocation.width;
 	        rect.height = widget->allocation.height;
 
@@ -401,19 +451,19 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap, int offx, int offy)
 
 		twidth = gdk_string_width(font,text);
 		theight = gdk_string_height(font,text);
-		if(twidth>BIG_ICON_SIZE)
-			twidth = BIG_ICON_SIZE;
+		if(twidth>size)
+			twidth = size;
 		
 		gdk_gc_set_foreground(gc,&widget->style->black);
 		gdk_draw_rectangle(pixmap,gc,TRUE,
-				   widget->allocation.x+(widget->allocation.width/2)-(twidth/2)+off-1-offx,
-				   widget->allocation.y+(widget->allocation.height/2)-(theight/2)-1+off-offy,
+				   (widget->allocation.width/2)-(twidth/2)+off-1,
+				   (widget->allocation.height/2)-(theight/2)-1+off,
 				   twidth+2,
 				   theight+2);
 		gdk_gc_set_foreground(gc,&widget->style->white);
 		gdk_draw_string(pixmap,font,gc,
-				widget->allocation.x+(widget->allocation.width/2)-(twidth/2)+off-offx,
-				widget->allocation.y+(widget->allocation.height/2)+(theight/2)+off-offy,
+				(widget->allocation.width/2)-(twidth/2)+off,
+				(widget->allocation.height/2)+(theight/2)+off,
 				text);
 		gdk_gc_set_foreground(gc,&widget->style->black);
 		gdk_gc_set_clip_rectangle (gc, NULL);
@@ -426,10 +476,10 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap, int offx, int offy)
 	if(button->arrow) {
 		int i;
 		GdkPoint points[3];
-		draw_arrow(points,button->orient);
+		draw_arrow(points,button->orient,size);
 		for(i=0;i<3;i++) {
-			points[i].x+=widget->allocation.x+off-offx;
-			points[i].y+=widget->allocation.y+off-offy;
+			points[i].x+=off;
+			points[i].y+=off;
 		}
 		gdk_gc_set_foreground(gc,&pwidget->style->white);
 		gdk_draw_polygon(pixmap,gc,TRUE,points,3);
@@ -443,7 +493,9 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap, int offx, int offy)
 static void
 button_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 {
-	requisition->width = requisition->height = BIG_ICON_SIZE;
+	PanelWidget *panel = PANEL_WIDGET(widget->parent);
+	requisition->width = requisition->height = 
+		panel_widget_get_pixel_size(panel);
 }
 static void
 button_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
@@ -471,8 +523,7 @@ button_widget_init (ButtonWidget *button)
 
 	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 
-	button->pixmap = NULL;
-	button->mask = NULL;
+	button->pixbuf = NULL;
 	
 	button->tile = 0;
 	button->arrow = 0;
@@ -558,8 +609,13 @@ button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 	    (event->detail != GDK_NOTIFY_INFERIOR)) {
 		ButtonWidget *button = BUTTON_WIDGET (widget);
 		button->in_button = TRUE;
-		panel_widget_draw_icon(PANEL_WIDGET(widget->parent),
-				       button);
+		if(global_config.tile_when_over) {
+			if(button->cache)
+				gdk_pixmap_unref(button->cache);
+			button->cache = NULL;
+			panel_widget_draw_icon(PANEL_WIDGET(widget->parent),
+					       button);
+		}
 	}
 
 	return FALSE;
@@ -583,8 +639,13 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 	    (event->detail != GDK_NOTIFY_INFERIOR) &&
 	    (!button->ignore_leave)) {
 		button->in_button = FALSE;
-		panel_widget_draw_icon(PANEL_WIDGET(widget->parent),
-				       button);
+		if(global_config.tile_when_over) {
+			if(button->cache)
+				gdk_pixmap_unref(button->cache);
+			button->cache = NULL;
+			panel_widget_draw_icon(PANEL_WIDGET(widget->parent),
+					       button);
+		}
 	}
 
 	return FALSE;
@@ -602,6 +663,10 @@ static void
 button_widget_pressed(ButtonWidget *button)
 {
 	button->pressed = TRUE;
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
+	setup_no_alpha(button);
 	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 			       button);
 }
@@ -609,6 +674,10 @@ static void
 button_widget_unpressed(ButtonWidget *button)
 {
 	button->pressed = FALSE;
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
+	setup_no_alpha(button);
 	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 			       button);
 	if(button->in_button)
@@ -632,8 +701,8 @@ button_widget_up(ButtonWidget *button)
 
 
 GtkWidget*
-button_widget_new(GdkPixmap *pixmap,
-		  GdkBitmap *mask,
+button_widget_new(char *filename,
+		  int size,
 		  guint tile,
 		  guint arrow,
 		  PanelOrientType orient,
@@ -643,108 +712,37 @@ button_widget_new(GdkPixmap *pixmap,
 
 	button = BUTTON_WIDGET (gtk_type_new (button_widget_get_type ()));
 	
-	button->pixmap = pixmap;
-	button->mask = mask;
+	button->pixbuf = loadup_file(filename);
+	button->filename = g_strdup(filename);
+	button->size = size;
 	button->tile = tile;
 	button->arrow = arrow;
 	button->orient = orient;
 	button->text = text?g_strdup(text):NULL;
+
+	setup_no_alpha(button);
 	
 	return GTK_WIDGET(button);
 }
 
-static void
-loadup_file(GdkPixmap **pixmap, GdkBitmap **mask, char *file)
+int
+button_widget_set_pixmap(ButtonWidget *button, char *pixmap, int size)
 {
-	GdkImlibImage *im = NULL;
-	int w,h;
+	if(size<0)
+		size = panel_widget_get_pixel_size(PANEL_WIDGET(GTK_WIDGET(button)->parent));
 	
-	if(!file) {
-		*pixmap = NULL;
-		*mask = NULL;
-		return;
-	}
+	button->pixbuf = loadup_file(pixmap);
+	g_free(button->filename);
+	button->filename = g_strdup(pixmap);
+	button->size = size;
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
 
-	if(*file!='/') {
-		char *f;
-		f = gnome_pixmap_file (file);
-		if(f) {
-			im = gdk_imlib_load_image (f);
-			g_free(f);
-		}
-	} else
-		im = gdk_imlib_load_image (file);
-	if(!im) {
-		*pixmap = NULL;
-		*mask = NULL;
-		return;
-	}
-
-	w = im->rgb_width;
-	h = im->rgb_height;
-	if(w>h) {
-		if(w>BIG_ICON_SIZE) {
-			h = h*((double)BIG_ICON_SIZE/w);
-			w = BIG_ICON_SIZE;
-		}
-	} else {
-		if(h>BIG_ICON_SIZE) {
-			w = w*((double)BIG_ICON_SIZE/h);
-			h = BIG_ICON_SIZE;
-		}
-	}
-	w = w>0?w:1;
-	h = h>0?h:1;
-	gdk_imlib_render (im, w, h);
-
-	*pixmap = gdk_imlib_copy_image (im);
-	*mask = gdk_imlib_copy_mask (im);
-
-	gdk_imlib_destroy_image (im);
-}
-
-GtkWidget*
-button_widget_new_from_file(char *pixmap,
-			    guint tile,
-			    guint arrow,
-			    PanelOrientType orient,
-			    char *text)
-{
-	GdkPixmap *_pixmap;
-	GdkBitmap *mask;
-	
-	loadup_file(&_pixmap,&mask,pixmap);
-	
-	return button_widget_new(_pixmap,mask,tile,arrow,orient,text);
-}
-
-void
-button_widget_set_pixmap(ButtonWidget *button, GdkPixmap *pixmap, GdkBitmap *mask)
-{
-	if(button->pixmap)
-		gdk_pixmap_unref(button->pixmap);
-	button->pixmap = NULL;
-	if(button->mask)
-		gdk_bitmap_unref(button->mask);
-	button->mask = NULL;
-	
-	button->pixmap = pixmap;
-	button->mask = mask;
-	
 	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 			       button);
-}
 
-int
-button_widget_set_pixmap_from_file(ButtonWidget *button, char *pixmap)
-{
-	GdkPixmap *_pixmap;
-	GdkBitmap *mask;
-	
-	loadup_file(&_pixmap,&mask,pixmap);
-	button_widget_set_pixmap(button,_pixmap,mask);
-
-	if(!_pixmap)
+	if(!button->pixbuf)
 		return FALSE;
 	
 	return TRUE;
@@ -755,6 +753,10 @@ button_widget_set_text(ButtonWidget *button, char *text)
 {
 	g_free(button->text);
 	button->text = text?g_strdup(text):NULL;
+
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
 	
 	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 			       button);
@@ -769,6 +771,11 @@ button_widget_set_params(ButtonWidget *button,
 	button->tile = tile;
 	button->arrow = arrow;
 	button->orient = orient;
+
+	if(button->cache)
+		gdk_pixmap_unref(button->cache);
+	button->cache = NULL;
+	setup_no_alpha(button);
 	
 	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 			       button);
@@ -780,31 +787,27 @@ button_widget_load_tile(int tile, char *tile_up, char *tile_down,
 {
 	GList *list;
 
-	if(tiles_up[tile])
-		gdk_pixmap_unref(tiles_up[tile]);
-	tiles_up[tile] = NULL;
-	if(tiles_up_mask[tile])
-		gdk_bitmap_unref(tiles_up_mask[tile]);
-	tiles_up_mask[tile] = NULL;
+	if(tiles.tiles_up[tile])
+		gdk_pixbuf_unref(tiles.tiles_up[tile]);
+	tiles.tiles_up[tile] = loadup_file(tile_up);
 
-	if(tiles_down[tile])
-		gdk_pixmap_unref(tiles_down[tile]);
-	tiles_down[tile] = NULL;
-	if(tiles_down_mask[tile])
-		gdk_bitmap_unref(tiles_down_mask[tile]);
-	tiles_down_mask[tile] = NULL;
+	if(tiles.tiles_down[tile])
+		gdk_pixbuf_unref(tiles.tiles_down[tile]);
+	tiles.tiles_down[tile] = loadup_file(tile_down);
 
-	loadup_file(&tiles_up[tile],&tiles_up_mask[tile],tile_up);
-	loadup_file(&tiles_down[tile],&tiles_down_mask[tile],tile_down);
-	
 	tile_border[tile] = border;
 	tile_depth[tile] = depth;
-	
+
 	for(list = buttons;list!=NULL;list=g_list_next(list)) {
 		ButtonWidget *button = list->data;
-		if(button->tile == tile)
+		if(button->tile == tile) {
+			if(button->cache)
+				gdk_pixmap_unref(button->cache);
+			button->cache = NULL;
+			setup_no_alpha(button);
 			panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
 					       button);
+		}
 	}
 }
 
@@ -820,8 +823,14 @@ button_widget_set_flags(int type, int _tiles_enabled, int _pixmaps_enabled, int
 		pixmaps_enabled[type] = _pixmaps_enabled;
 		always_text[type] = _always_text;
 
-		for(list = buttons;list!=NULL;list=g_list_next(list))
+		for(list = buttons;list!=NULL;list=g_list_next(list)) {
+			ButtonWidget *button = list->data;
+			if(button->cache)
+				gdk_pixmap_unref(button->cache);
+			button->cache = NULL;
+			setup_no_alpha(button);
 			panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(list->data)->parent),
-					       list->data);
+					       button);
+		}
 	}
 }
