@@ -18,10 +18,17 @@ typedef struct _ButtonWidgetClass	ButtonWidgetClass;
 
 struct _ButtonWidget
 {
-	GtkWidget		_pixmap;
+	GtkWidget		pixmap;
 	
-	GdkPixmap		*pixmap; /*this is the one we start from*/
-	GdkBitmap		*mask;
+	GdkPixbuf		*pixbuf;
+
+	GdkPixmap		*cache;
+	gboolean		no_alpha; /*we don't have any alpha to speak of,
+					    so we don't have to dump the cache all
+					    the time*/
+	
+	char			*filename;
+	int			size;
 	
 	GdkWindow               *event_window;
 	
@@ -54,28 +61,23 @@ struct _ButtonWidgetClass
 
 guint		button_widget_get_type		(void);
 
-GtkWidget*	button_widget_new		(GdkPixmap *pixmap,
-						 GdkBitmap *mask,
-						 guint tile,
-						 guint arrow,
-						 PanelOrientType orient,
-						 char *text);
-GtkWidget*	button_widget_new_from_file	(char *pixmap,
+GtkWidget*	button_widget_new		(char *pixmap,
+						 int size,
 						 guint tile,
 						 guint arrow,
 						 PanelOrientType orient,
 						 char *text);
 
 void		button_widget_draw		(ButtonWidget *button,
-						 GdkPixmap *pixmap,
-						 int offx,
-						 int offy);
+						 guchar *rgb,
+						 int rowstride);
+/* draw the xlib part (arrow/text) */
+void		button_widget_draw_xlib		(ButtonWidget *button,
+						 GdkPixmap *pixmap);
 
-void		button_widget_set_pixmap	(ButtonWidget *button,
-						 GdkPixmap *pixmap,
-						 GdkBitmap *mask);
-int		button_widget_set_pixmap_from_file(ButtonWidget *button,
-						   char *pixmap);
+int		button_widget_set_pixmap	(ButtonWidget *button,
+						 char *pixmap,
+						 int size);
 
 void		button_widget_set_text		(ButtonWidget *button,
 						 char *text);
