@@ -29,7 +29,6 @@ extern int applet_count;
 extern int config_sync_timeout;
 extern int applets_to_sync;
 extern int panels_to_sync;
-extern int globals_to_sync;
 extern int need_complete_save;
 
 extern GtkTooltips *panel_tooltips;
@@ -39,6 +38,11 @@ extern GlobalConfig global_config;
 extern char *panel_cfg_path;
 extern char *old_panel_cfg_path;
 
+extern int ss_cur_applet;
+extern int ss_done_save;
+extern GtkWidget* ss_timeout_dlg;
+extern gushort ss_cookie;
+
 /********************* CORBA Stuff *******************/
 
 CORBA_ORB orb = NULL;
@@ -72,6 +76,14 @@ s_panel_quit(POA_GNOME_Panel *servant, CORBA_Environment *ev);
 static CORBA_boolean
 s_panel_get_in_drag(POA_GNOME_Panel *servant, CORBA_Environment *ev);
 
+static GNOME_StatusSpot
+s_panel_add_status(POA_GNOME_Panel *servant,
+		   CORBA_unsigned_long* wid,
+		   CORBA_Environment *ev);
+
+static void
+s_panel_notice_config_changes(POA_GNOME_Panel *servant,
+			      CORBA_Environment *ev);
 
 
 /*** PanelSpot stuff ***/
@@ -97,6 +109,22 @@ static GNOME_Panel_OrientType
 s_panelspot_get_parent_orient(POA_GNOME_PanelSpot *servant,
 			      CORBA_Environment *ev);
 
+static GNOME_Panel_SizeType
+s_panelspot_get_parent_size(POA_GNOME_PanelSpot *servant,
+			    CORBA_Environment *ev);
+
+static CORBA_short
+s_panelspot_get_free_space(POA_GNOME_PanelSpot *servant,
+			   CORBA_Environment *ev);
+
+static CORBA_boolean
+s_panelspot_get_send_position(POA_GNOME_PanelSpot *servant,
+			      CORBA_Environment *ev);
+static void
+s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
+			      CORBA_boolean,
+			      CORBA_Environment *ev);
+
 static void
 s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 		     CORBA_Environment *ev);
@@ -132,11 +160,31 @@ static void
 s_panelspot_remove_callback(POA_GNOME_PanelSpot *servant,
 			    CORBA_char *callback_name,
 			    CORBA_Environment *ev);
+static void
+s_panelspot_callback_set_sensitive(POA_GNOME_PanelSpot *servant,
+				   CORBA_char *callback_name,
+				   CORBA_boolean sensitive,
+				   CORBA_Environment *ev);
 
 static void
 s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
 			CORBA_Environment *ev);
 
+static void
+s_panelspot_done_session_save(POA_GNOME_PanelSpot *servant,
+			      CORBA_boolean ret,
+			      CORBA_unsigned_long cookie,
+			      CORBA_Environment *ev);
+
+/*** StatusSpot stuff ***/
+
+static void
+s_statusspot_remove(POA_GNOME_StatusSpot *servant,
+		    CORBA_Environment *ev);
+
+
+
+
 static PortableServer_ServantBase__epv panel_base_epv = {
   NULL, /* _private */
   NULL, /* finalize */
@@ -145,10 +193,12 @@ static PortableServer_ServantBase__epv panel_base_epv = {
 
 static POA_GNOME_Panel__epv panel_epv = {
   NULL, /* private data */
-  &s_panel_add_applet,
-  &s_panel_add_applet_full,
-  &s_panel_quit,
-  &s_panel_get_in_drag
+  (gpointer)&s_panel_add_applet,
+  (gpointer)&s_panel_add_applet_full,
+  (gpointer)&s_panel_quit,
+  (gpointer)&s_panel_get_in_drag,
+  (gpointer)&s_panel_add_status,
+  (gpointer)&s_panel_notice_config_changes
 };
 static POA_GNOME_Panel__vepv panel_vepv = { &panel_base_epv, &panel_epv };
 static POA_GNOME_Panel panel_servant = { NULL, &panel_vepv };
@@ -162,23 +212,41 @@ static PortableServer_ServantBase__epv panelspot_base_epv = {
 
 static POA_GNOME_PanelSpot__epv panelspot_epv = {
   NULL, /* private data */
-  &s_panelspot_get_tooltip,
-  &s_panelspot_set_tooltip,
-  &s_panelspot_get_parent_panel,
-  &s_panelspot_get_spot_pos,
-  &s_panelspot_get_parent_orient,
-  &s_panelspot_register_us,
-  &s_panelspot_unregister_us,
-  &s_panelspot_abort_load,
-  &s_panelspot_show_menu,
-  &s_panelspot_drag_start,
-  &s_panelspot_drag_stop,
-  &s_panelspot_add_callback,
-  &s_panelspot_remove_callback,
-  &s_panelspot_sync_config
+  (gpointer)&s_panelspot_get_tooltip,
+  (gpointer)&s_panelspot_set_tooltip,
+  (gpointer)&s_panelspot_get_parent_panel,
+  (gpointer)&s_panelspot_get_spot_pos,
+  (gpointer)&s_panelspot_get_parent_orient,
+  (gpointer)&s_panelspot_get_parent_size,
+  (gpointer)&s_panelspot_get_free_space,
+  (gpointer)&s_panelspot_get_send_position,
+  (gpointer)&s_panelspot_set_send_position,
+  (gpointer)&s_panelspot_register_us,
+  (gpointer)&s_panelspot_unregister_us,
+  (gpointer)&s_panelspot_abort_load,
+  (gpointer)&s_panelspot_show_menu,
+  (gpointer)&s_panelspot_drag_start,
+  (gpointer)&s_panelspot_drag_stop,
+  (gpointer)&s_panelspot_add_callback,
+  (gpointer)&s_panelspot_remove_callback,
+  (gpointer)&s_panelspot_callback_set_sensitive,
+  (gpointer)&s_panelspot_sync_config,
+  (gpointer)&s_panelspot_done_session_save
 };
 static POA_GNOME_PanelSpot__vepv panelspot_vepv = { &panelspot_base_epv, &panelspot_epv };
 
+static PortableServer_ServantBase__epv statusspot_base_epv = {
+  NULL, /* _private */
+  NULL, /* finalize */
+  NULL, /* use base default_POA function */
+};
+
+static POA_GNOME_StatusSpot__epv statusspot_epv = {
+  NULL, /* private data */
+  (gpointer)&s_statusspot_remove
+};
+static POA_GNOME_StatusSpot__vepv statusspot_vepv = { &statusspot_base_epv, &statusspot_epv };
+
 /********************* NON-CORBA Stuff *******************/
 
 static void
@@ -233,15 +301,54 @@ sal(GtkWidget *applet, GtkAllocation *alloc)
 	       applet->allocation.height);
 }*/
 
+
 static void
-test_size(GtkWidget *applet, GtkAllocation *alloc)
+send_position_change(Extern *ext)
 {
-	if(applet->allocation.width > applet->requisition.width ||
-	   applet->allocation.height > applet->requisition.height) {
-		gtk_widget_queue_resize(applet->parent);
+	/*ingore this until we get an ior*/
+	if(ext->applet) {
+		int x=0,y=0;
+		GtkWidget *wid=ext->ebox;
+		
+		CORBA_Environment ev;
+		CORBA_exception_init(&ev);
+		g_warning ("Crazy function!\n");
+		/*go the the toplevel panel widget*/
+		for(;;) {
+			if(!GTK_WIDGET_NO_WINDOW(wid)) {
+				x += wid->allocation.x;
+				y += wid->allocation.y;
+			}
+			if(wid->parent)
+				wid = wid->parent;
+			else
+				break;
+		}
+		GNOME_Applet_change_position(ext->applet,
+					     x,y,
+					     &ev);
+		if(ev._major)
+			panel_clean_applet(ext->info);
+		CORBA_exception_free(&ev);
 	}
 }
 
+static void
+ebox_size_allocate(GtkWidget *applet, GtkAllocation *alloc, Extern *ext)
+{
+	if(ext->send_position)
+		send_position_change(ext);
+}
+
+static void
+socket_size_allocate(GtkWidget *applet, GtkAllocation *alloc)
+{
+  if (applet->requisition.width > 0 && applet->requisition.height > 0 &&
+      (applet->allocation.width > applet->requisition.width ||
+       applet->allocation.height > applet->requisition.height))
+    gtk_widget_queue_resize(applet->parent);
+}
+
 /*note that type should be APPLET_EXTERN_RESERVED or APPLET_EXTERN_PENDING
   only*/
 static CORBA_unsigned_long
@@ -255,11 +362,15 @@ reserve_applet_spot (Extern *ext, PanelWidget *panel, int pos,
 					  APPLET_EVENT_MASK) &
 			      ~( GDK_POINTER_MOTION_MASK |
 				 GDK_POINTER_MOTION_HINT_MASK));
+	gtk_signal_connect_after(GTK_OBJECT(ext->ebox),"size_allocate",
+				 GTK_SIGNAL_FUNC(ebox_size_allocate),
+				 ext);
 
 	socket = gtk_socket_new();
 
 	gtk_signal_connect_after(GTK_OBJECT(socket),"size_allocate",
-				 GTK_SIGNAL_FUNC(test_size),NULL);
+				 GTK_SIGNAL_FUNC(socket_size_allocate),
+				 NULL);
 
 	/*gtk_signal_connect_after(GTK_OBJECT(socket),"size_allocate",
 				 GTK_SIGNAL_FUNC(sal),NULL);*/
@@ -297,14 +408,15 @@ load_extern_applet(char *goad_id, char *cfgpath, PanelWidget *panel, int pos, in
 	POA_GNOME_PanelSpot *panelspot_servant;
 
 	if(!cfgpath || !*cfgpath)
-		cfgpath = g_strconcat(old_panel_cfg_path,
-					 "Applet_Dummy/",NULL);
+		cfgpath = g_strconcat(PANEL_CONFIG_PATH,
+				      "Applet_Dummy/",NULL);
 	else
 		/*we will free this lateer*/
 		cfgpath = g_strdup(cfgpath);
 	
-	ext = g_new(Extern,1);
+	ext = g_new0(Extern,1);
 	ext->started = FALSE;
+	ext->send_position = FALSE;
 
 	panelspot_servant = (POA_GNOME_PanelSpot *)ext;
 	panelspot_servant->_private = NULL;
@@ -378,7 +490,6 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 			CORBA_Environment *ev)
 {
 	GSList *li;
-	int i;
 	Extern *ext;
 	char *p;
 	POA_GNOME_PanelSpot *panelspot_servant;
@@ -402,7 +513,7 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 				*cfgpath = CORBA_string_dup(ext->cfg);
 				g_free(ext->cfg);
 				ext->cfg = NULL;
-				*globcfgpath = CORBA_string_dup(old_panel_cfg_path);
+				*globcfgpath = CORBA_string_dup(PANEL_CONFIG_PATH);
 				info->type = APPLET_EXTERN_RESERVED;
 				*wid=GDK_WINDOW_XWINDOW(socket->window);
 #ifdef PANEL_DEBUG
@@ -422,7 +533,9 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 	
 	/*this is an applet that was started from outside, otherwise we would
 	  have already reserved a spot for it*/
-	ext = g_new(Extern,1);
+	ext = g_new0(Extern,1);
+	ext->started = FALSE;
+	ext->send_position = FALSE;
 
 	panelspot_servant = (POA_GNOME_PanelSpot *)ext;
 	panelspot_servant->_private = NULL;
@@ -460,10 +573,10 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 		*cfgpath = NULL;
 		return CORBA_OBJECT_NIL;
 	}
-	p = g_strconcat(old_panel_cfg_path,"Applet_Dummy/",NULL);
+	p = g_strconcat(PANEL_CONFIG_PATH,"Applet_Dummy/",NULL);
 	*cfgpath = CORBA_string_dup(p);
 	g_free(p);
-	*globcfgpath = CORBA_string_dup(old_panel_cfg_path);
+	*globcfgpath = CORBA_string_dup (PANEL_CONFIG_PATH);
 
 	return CORBA_Object_duplicate(acc, ev);
 }
@@ -480,6 +593,47 @@ s_panel_get_in_drag(POA_GNOME_Panel *servant, CORBA_Environment *ev)
 	return panel_applet_in_drag;
 }
 
+static GNOME_StatusSpot
+s_panel_add_status(POA_GNOME_Panel *servant,
+		   CORBA_unsigned_long *wid,
+		   CORBA_Environment *ev)
+{
+	POA_GNOME_StatusSpot *statusspot_servant;
+	GNOME_StatusSpot acc;
+	StatusSpot *ss;
+	
+	*wid = 0;
+	
+	ss = new_status_spot();
+	if(!ss)
+		return CORBA_OBJECT_NIL;
+	
+	statusspot_servant = (POA_GNOME_StatusSpot *)ss;
+	statusspot_servant->_private = NULL;
+	statusspot_servant->vepv = &statusspot_vepv;
+
+	POA_GNOME_StatusSpot__init(statusspot_servant, ev);
+	
+	CORBA_free(PortableServer_POA_activate_object(thepoa, statusspot_servant, ev));
+	g_return_val_if_fail(ev->_major == CORBA_NO_EXCEPTION,NULL);
+
+	acc = PortableServer_POA_servant_to_reference(thepoa, statusspot_servant, ev);
+	g_return_val_if_fail(ev->_major == CORBA_NO_EXCEPTION,NULL);
+
+	ss->sspot = CORBA_Object_duplicate(acc, ev);
+
+	g_return_val_if_fail(ev->_major == CORBA_NO_EXCEPTION,NULL);
+
+	*wid = ss->wid;
+	return CORBA_Object_duplicate(acc, ev);
+}
+
+static void
+s_panel_notice_config_changes(POA_GNOME_Panel *servant,
+			      CORBA_Environment *ev)
+{
+	load_up_globals();
+}
 
 
 /*** PanelSpot stuff ***/
@@ -533,7 +687,6 @@ s_panelspot_get_spot_pos(POA_GNOME_PanelSpot *servant,
 			 CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
-	PanelWidget *panel;
 	AppletData *ad;
 
 	g_assert(ext);
@@ -563,6 +716,61 @@ s_panelspot_get_parent_orient(POA_GNOME_PanelSpot *servant,
 	return get_applet_orient(panel);
 }
 
+static GNOME_Panel_SizeType
+s_panelspot_get_parent_size(POA_GNOME_PanelSpot *servant,
+			    CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+	PanelWidget *panel;
+
+	g_assert(ext);
+	g_assert(ext->info);
+
+	panel = PANEL_WIDGET(ext->info->widget->parent);
+
+	g_return_val_if_fail(panel != NULL,SIZE_STANDARD);
+
+	return panel->sz;
+}
+
+static CORBA_short
+s_panelspot_get_free_space(POA_GNOME_PanelSpot *servant,
+			   CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+	PanelWidget *panel;
+
+	g_assert(ext);
+	g_assert(ext->info);
+
+	panel = PANEL_WIDGET(ext->info->widget->parent);
+	
+	return panel_widget_get_free_space(panel,ext->info->widget);
+}
+
+static CORBA_boolean
+s_panelspot_get_send_position(POA_GNOME_PanelSpot *servant,
+			      CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+
+	g_assert(ext);
+	
+	return ext->send_position;
+}
+
+static void
+s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
+			      CORBA_boolean enable,
+			      CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+
+	g_assert(ext);
+	
+	ext->send_position = enable?TRUE:FALSE;
+}
+
 static void
 s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 			CORBA_Environment *ev)
@@ -585,7 +793,10 @@ s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 	ext->info->type = APPLET_EXTERN;
 
 	orientation_change(ext->info,panel);
+	size_change(ext->info,panel);
 	back_change(ext->info,panel);
+	if(ext->send_position)
+		send_position_change(ext);
 
 	GNOME_Applet_set_tooltips_state(ext->applet,
 					global_config.tooltips_enabled, ev);
@@ -639,13 +850,9 @@ s_panelspot_show_menu(POA_GNOME_PanelSpot *servant,
 		create_applet_menu(ext->info);
 
 	panel = get_panel_parent(ext->info->widget);
-	if(IS_SNAPPED_WIDGET(panel)) {
-		SNAPPED_WIDGET(panel)->autohide_inhibit = TRUE;
-		snapped_widget_queue_pop_down(SNAPPED_WIDGET(panel));
-	} else if (IS_CORNER_WIDGET(panel)) {
-	        CORNER_WIDGET(panel)->autohide_inhibit = TRUE;
-	        corner_widget_queue_pop_down(CORNER_WIDGET(panel));
-	}
+
+	BASEP_WIDGET(panel)->autohide_inhibit = TRUE;
+	basep_widget_queue_autohide(BASEP_WIDGET(panel));
 	
 	ext->info->menu_age = 0;
 	gtk_menu_popup(GTK_MENU(ext->info->menu), NULL, NULL,
@@ -721,6 +928,19 @@ s_panelspot_remove_callback(POA_GNOME_PanelSpot *servant,
 	applet_remove_callback(ext->info, callback_name);
 }
 
+static void
+s_panelspot_callback_set_sensitive(POA_GNOME_PanelSpot *servant,
+				   CORBA_char *callback_name,
+				   CORBA_boolean sensitive,
+				   CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+
+	g_return_if_fail(ext != NULL);
+	g_return_if_fail(ext->info != NULL);
+	applet_callback_set_sensitive(ext->info, callback_name, sensitive);
+}
+
 static void
 s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
 			CORBA_Environment *ev)
@@ -733,6 +953,122 @@ s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
 	panel_config_sync();
 }
 
+static int
+save_next_idle(gpointer data)
+{
+	save_next_applet();
+	return FALSE;
+}
+
+static void
+s_panelspot_done_session_save(POA_GNOME_PanelSpot *servant,
+			      CORBA_boolean ret,
+			      CORBA_unsigned_long cookie,
+			      CORBA_Environment *ev)
+{
+	GSList *cur;
+	AppletInfo *info;
+	char *buf;
+	PanelWidget *panel;
+	AppletData *ad;
+	Extern *ext;
+	int panel_num;
+	
+	/*ignore bad cookies*/
+	if(cookie != ss_cookie)
+		return;
+
+	/*increment cookie to kill the timeout warning*/
+	ss_cookie++;
+	
+	if(ss_timeout_dlg) {
+		gtk_widget_destroy(ss_timeout_dlg);
+		ss_timeout_dlg = NULL;
+	}
+
+	if(g_slist_length(applets)<=ss_cur_applet) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	cur = g_slist_nth(applets,ss_cur_applet);
+	
+	if(!cur) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	info = cur->data;
+
+	/*hmm, this came from a different applet?, we are
+	  getting seriously confused*/
+	if(info->type!=APPLET_EXTERN ||
+	   (gpointer)servant!=(gpointer)info->data) {
+		applets_to_sync = TRUE; /*we need to redo this yet again*/
+		/*save next applet, but from an idle handler, so that
+		  this call returns*/
+		gtk_idle_add(save_next_idle,NULL);
+		return;
+	}
+	
+	ext = info->data;
+	
+	buf = g_strdup_printf("%sApplet_Config/Applet_%d/", PANEL_CONFIG_PATH, info->applet_id+1);
+	gnome_config_push_prefix(buf);
+	g_free(buf);
+
+	panel = PANEL_WIDGET(info->widget->parent);
+	ad = gtk_object_get_data(GTK_OBJECT(info->widget),PANEL_APPLET_DATA);
+
+	if((panel_num = g_slist_index(panels,panel)) == -1) {
+		gnome_config_set_string("id", EMPTY_ID);
+		gnome_config_pop_prefix();
+		gnome_config_sync();
+		/*save next applet, but from an idle handler, so that
+		  this call returns*/
+		gtk_idle_add(save_next_idle,NULL);
+		return;
+	}
+		
+	/*have the applet do it's own session saving*/
+	if(ret) {
+		gnome_config_set_string("id", EXTERN_ID);
+		gnome_config_set_string("goad_id",
+					ext->goad_id);
+	} else {
+		gnome_config_set_string("id", EMPTY_ID);
+		gnome_config_pop_prefix();
+		gnome_config_sync();
+		/*save next applet, but from an idle handler, so that
+		  this call returns*/
+		gtk_idle_add(save_next_idle,NULL);
+		return;
+	}
+	
+	gnome_config_set_int("position", ad->pos);
+	gnome_config_set_int("panel", panel_num);
+	gnome_config_set_bool("right_stick",
+			      panel_widget_is_applet_stuck(panel,
+							   info->widget));
+	gnome_config_pop_prefix();
+	
+	gnome_config_sync();
+	/*save next applet, but from an idle handler, so that
+	  this call returns*/
+	gtk_idle_add(save_next_idle,NULL);
+}
+
+/*** StatusSpot stuff ***/
+
+static void
+s_statusspot_remove(POA_GNOME_StatusSpot *servant,
+		    CORBA_Environment *ev)
+{
+	StatusSpot *ss = (StatusSpot *)servant;
+	status_spot_remove(ss, TRUE);
+}
+
+
 void
 panel_corba_clean_up(void)
 {
@@ -744,9 +1080,7 @@ gint
 panel_corba_gtk_init(CORBA_ORB panel_orb)
 {
   GNOME_Panel acc;
-  char hostname [4096];
-  char *name;
-  CORBA_Object ns, old_server;
+  CORBA_Object old_server;
   gint status;
 
   CORBA_exception_init(&ev);
