@@ -10,6 +10,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -32,16 +33,22 @@ extern GlobalConfig global_config;
 
 
 static void
-launch (GtkWidget *widget, void *data)
+launch (Launcher *launcher, int argc, char *argv[])
 {
-	GnomeDesktopEntry *item = data;
+	GnomeDesktopEntry *item;
+
+	g_return_if_fail(launcher != NULL);
+	g_return_if_fail(launcher->dentry != NULL);
+
+	item = launcher->dentry;
 
 	if(!item->exec) {
 		GtkWidget *dlg;
 		dlg = gnome_message_box_new(_("This launch icon does not "
 					      "specify a program to run"),
 					    GNOME_MESSAGE_BOX_ERROR,
-					    _("Close"), NULL);
+					    GNOME_STOCK_BUTTON_CLOSE,
+					    NULL);
 		gtk_window_set_wmclass(GTK_WINDOW(dlg),
 				       "no_exec_dialog","Panel");
 		gtk_widget_show_all(dlg);
@@ -53,27 +60,60 @@ launch (GtkWidget *widget, void *data)
 	    && strstr(item->exec[0], "://"))
 		gnome_url_show(item->exec[0]);
 	else
-		gnome_desktop_entry_launch (item);
+		gnome_desktop_entry_launch_with_args (item,argc,argv);
 	
 	if(global_config.drawer_auto_close) {
 		GtkWidget *parent =
-			gtk_object_get_data(GTK_OBJECT(widget->parent),PANEL_PARENT);
+			PANEL_WIDGET(launcher->button->parent)->panel_parent;
 		g_return_if_fail(parent!=NULL);
 		if(IS_DRAWER_WIDGET(parent)) {
 			BasePWidget *basep = BASEP_WIDGET(parent);
 			GtkWidget *grandparent = PANEL_WIDGET(basep->panel)->master_widget->parent;
-			GtkWidget *grandparentw = gtk_object_get_data(GTK_OBJECT(grandparent),
-								      PANEL_PARENT);
-
-			drawer_widget_close_drawer(DRAWER_WIDGET(parent));
-			if(IS_SNAPPED_WIDGET(grandparentw))
-				SNAPPED_WIDGET(grandparentw)->drawers_open--;
-			else if(IS_CORNER_WIDGET(grandparentw))
-			        CORNER_WIDGET(grandparentw)->drawers_open--;
+			GtkWidget *grandparentw =
+				PANEL_WIDGET(grandparent)->panel_parent;
+			drawer_widget_close_drawer (DRAWER_WIDGET (parent),
+						    BASEP_WIDGET (grandparentw));
 		}
 	}
 }
 
+static void
+launch_cb (GtkWidget *widget, gpointer data)
+{
+	launch(data,0,NULL);
+}
+
+static void
+drag_data_received_cb (GtkWidget        *widget,
+		       GdkDragContext   *context,
+		       gint              x,
+		       gint              y,
+		       GtkSelectionData *selection_data,
+		       guint             info,
+		       guint             time,
+		       Launcher         *launcher)
+{
+
+	GList *li, *files;
+	int argc;
+	char **argv;
+	int i;
+
+	g_return_if_fail(launcher!=NULL);
+
+	files = gnome_uri_list_extract_filenames(selection_data->data);
+	argc = g_list_length(files);
+	argv = g_new(char *,argc+1);
+	argv[argc] = NULL;
+
+	for(i=0,li = files; li; i++,li = g_list_next(li))
+		argv[i]=li->data;
+	
+	launch(launcher,argc,argv);
+	gnome_uri_list_free_strings (files);
+	g_free(argv);
+}
+
 static void
 destroy_launcher(GtkWidget *widget, gpointer data)
 {
@@ -112,7 +152,7 @@ create_launcher (char *parameters, GnomeDesktopEntry *dentry)
 {
 	char *icon;
 	Launcher *launcher;
-        static GtkTargetEntry drag_targets[] = {
+        static GtkTargetEntry dnd_targets[] = {
 		{ "text/uri-list", 0, 0 }
 	};
 
@@ -157,18 +197,19 @@ create_launcher (char *parameters, GnomeDesktopEntry *dentry)
 			dentry->icon = gnome_pixmap_file (icon);
 			g_free (icon);
 		}
-		launcher->button = button_widget_new_from_file(dentry->icon,
-							       LAUNCHER_TILE,
-							       FALSE,
-							       ORIENT_UP,
-							       dentry->name);
+		launcher->button = button_widget_new(dentry->icon,
+						     -1,
+						     LAUNCHER_TILE,
+						     FALSE,
+						     ORIENT_UP,
+						     dentry->name);
 	}
 	if (!launcher->button) {
 		launcher->button =
-			button_widget_new_from_file(default_app_pixmap,
-						    LAUNCHER_TILE,
-						    FALSE,ORIENT_UP,
-						    _("App"));
+			button_widget_new(default_app_pixmap,-1,
+					  LAUNCHER_TILE,
+					  FALSE,ORIENT_UP,
+					  _("App"));
 	}
 	gtk_widget_show (launcher->button);
 
@@ -180,17 +221,25 @@ create_launcher (char *parameters, GnomeDesktopEntry *dentry)
 	GTK_WIDGET_UNSET_FLAGS(launcher->button,GTK_NO_WINDOW);
 	gtk_drag_source_set(launcher->button,
 			    GDK_BUTTON1_MASK,
-			    drag_targets, 1,
+			    dnd_targets, 1,
 			    GDK_ACTION_COPY);
 	GTK_WIDGET_SET_FLAGS(launcher->button,GTK_NO_WINDOW);
+	
+	gtk_drag_dest_set (GTK_WIDGET (launcher->button),
+			   GTK_DEST_DEFAULT_MOTION |
+			   GTK_DEST_DEFAULT_HIGHLIGHT |
+			   GTK_DEST_DEFAULT_DROP,
+			   dnd_targets, 1,
+			   GDK_ACTION_COPY);
 
 	gtk_signal_connect(GTK_OBJECT(launcher->button), "drag_data_get",
 			   drag_data_get_cb, launcher);
+	gtk_signal_connect(GTK_OBJECT(launcher->button), "drag_data_received",
+			   drag_data_received_cb, launcher);
 
-	gtk_signal_connect (GTK_OBJECT(launcher->button),
-			    "clicked",
-			    (GtkSignalFunc) launch,
-			    dentry);
+	gtk_signal_connect (GTK_OBJECT(launcher->button), "clicked",
+			    (GtkSignalFunc) launch_cb,
+			    launcher);
 	
 	gtk_signal_connect (GTK_OBJECT(launcher->button), "destroy",
 			    GTK_SIGNAL_FUNC(destroy_launcher),
@@ -214,10 +263,6 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 	
 	if (page != -1)
 		return;
-	
-	/* remove the old launcher callback */
-	gtk_signal_disconnect_by_func(GTK_OBJECT(launcher->button),
-			(GtkSignalFunc) launch, launcher->dentry);
 
 	gnome_desktop_entry_free(launcher->dentry);
 	launcher->dentry =
@@ -227,12 +272,6 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 		launcher->dentry->name=g_strdup("???");
 	}
 
-	/* and install the new one with the right dentry pointer */
-	gtk_signal_connect (GTK_OBJECT(launcher->button),
-			    "clicked",
-			    (GtkSignalFunc) launch,
-			    launcher->dentry);
-
 	gtk_tooltips_set_tip (panel_tooltips,launcher->button,
 			      launcher->dentry->comment,NULL);
 	
@@ -246,14 +285,17 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 			launcher->dentry->icon = gnome_pixmap_file (icon);
 			g_free (icon);
 		}
-		if(!button_widget_set_pixmap_from_file (BUTTON_WIDGET(launcher->button),
-							launcher->dentry->icon))
-			button_widget_set_pixmap_from_file (BUTTON_WIDGET(launcher->button),
-							    default_app_pixmap);
+		if(!button_widget_set_pixmap (BUTTON_WIDGET(launcher->button),
+					      launcher->dentry->icon,
+					      -1))
+			button_widget_set_pixmap (BUTTON_WIDGET(launcher->button),
+						  default_app_pixmap,
+						  -1);
 	} else {
-		button_widget_set_pixmap_from_file (BUTTON_WIDGET(launcher->button),
-						    default_app_pixmap);
+		button_widget_set_pixmap(BUTTON_WIDGET(launcher->button),
+					 default_app_pixmap, -1);
 	}
+	panel_config_sync();
 }
 
 static void
@@ -334,9 +376,9 @@ launcher_properties(Launcher *launcher)
 	gtk_widget_show_all (dialog);
 }
 
-static void
-_load_launcher_applet(char *params, GnomeDesktopEntry *dentry,
-		      PanelWidget *panel, int pos)
+void
+load_launcher_applet_full (char *params, GnomeDesktopEntry *dentry,
+			   PanelWidget *panel, int pos)
 {
 	Launcher *launcher;
 
@@ -366,11 +408,12 @@ really_add_launcher(GtkWidget *d,int button, gpointer data)
 	
 	if(button == 0) {
 		dentry = gnome_dentry_get_dentry(dedit);
+
 		if(!dentry->name || !(*(dentry->name))) {
 			g_free(dentry->name);
 			dentry->name=g_strdup("???");
 		}
-		_load_launcher_applet(NULL, dentry, panel, pos);
+		load_launcher_applet_full (NULL, dentry, panel, pos);
 		panel_config_sync();
 	}
 	gtk_widget_destroy(d);
@@ -431,12 +474,12 @@ load_launcher_applet_from_info(char *name, char *comment,
 	else
 		dentry->icon = g_strdup(icon);
 
-	_load_launcher_applet(NULL,dentry,panel, pos);
+	load_launcher_applet_full (NULL,dentry,panel, pos);
 	panel_config_sync();
 }
 
 void
 load_launcher_applet(char *params, PanelWidget *panel, int pos)
 {
-	_load_launcher_applet(params,NULL,panel, pos);
+	load_launcher_applet_full (params,NULL,panel, pos);
 }
