@@ -15,20 +15,15 @@ static void
 logout(GtkWidget *widget)
 {
 	if(global_config.drawer_auto_close) {
-		GtkWidget *parent =
-			gtk_object_get_data(GTK_OBJECT(widget->parent),PANEL_PARENT);
+		GtkWidget *parent = PANEL_WIDGET(widget->parent)->panel_parent;
 		g_return_if_fail(parent!=NULL);
 		if(IS_DRAWER_WIDGET(parent)) {
 			BasePWidget *basep = BASEP_WIDGET(parent);
 			GtkWidget *grandparent = PANEL_WIDGET(basep->panel)->master_widget->parent;
-			GtkWidget *grandparentw = gtk_object_get_data(GTK_OBJECT(grandparent),
-								      PANEL_PARENT);
-
-			drawer_widget_close_drawer(DRAWER_WIDGET(parent));
-			if(IS_SNAPPED_WIDGET(grandparentw))
-				SNAPPED_WIDGET(grandparentw)->drawers_open--;
-			else if(IS_CORNER_WIDGET(grandparentw))
-		                CORNER_WIDGET(grandparentw)->drawers_open--;
+			GtkWidget *grandparentw =
+				PANEL_WIDGET(grandparent)->panel_parent;
+			drawer_widget_close_drawer (DRAWER_WIDGET (parent),
+						    BASEP_WIDGET (grandparentw));
 		}
 	}
 
@@ -39,16 +34,15 @@ static GtkWidget *
 create_logout_widget(void)
 {
 	GtkWidget *button;
-	GtkWidget *pixmap;
 	char *pixmap_name;
 
 	pixmap_name = gnome_pixmap_file("gnome-term-night.png");
 
-	button = button_widget_new_from_file(pixmap_name,
-					     LOGOUT_TILE,
-					     FALSE,
-					     ORIENT_UP,
-					     _("Log out"));
+	button = button_widget_new(pixmap_name,-1,
+				   MISC_TILE,
+				   FALSE,
+				   ORIENT_UP,
+				   _("Log out"));
 	g_free(pixmap_name);
 	gtk_tooltips_set_tip (panel_tooltips,button,_("Log out of GNOME"),NULL);
 
@@ -69,3 +63,36 @@ load_logout_applet(PanelWidget *panel, int pos)
 		register_toy(logout, NULL, panel, pos, APPLET_LOGOUT);
 }
 
+static GtkWidget *
+create_lock_widget(void)
+{
+	GtkWidget *button;
+	char *pixmap_name;
+
+	pixmap_name = gnome_pixmap_file("gnome-lockscreen.png");
+
+	button = button_widget_new(pixmap_name,-1,
+				   MISC_TILE,
+				   FALSE,
+				   ORIENT_UP,
+				   _("Lock screen"));
+	g_free(pixmap_name);
+	gtk_tooltips_set_tip (panel_tooltips,button,_("Lock screen"),NULL);
+
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(panel_lock), NULL);
+
+	return button;
+}
+
+void
+load_lock_applet(PanelWidget *panel, int pos)
+{
+	GtkWidget *lock;
+
+	lock = create_lock_widget();
+
+	if(lock)
+		register_toy(lock, NULL, panel, pos, APPLET_LOCK);
+}
+
