@@ -12,13 +12,14 @@
 #include <sys/wait.h>
 #include <gnome.h>
 #include <libgnorba/gnorba.h>
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
 
 #include "panel-include.h"
 
 extern int config_sync_timeout;
 extern int applets_to_sync;
 extern int panels_to_sync;
-extern int globals_to_sync;
 extern int need_complete_save;
 
 extern GSList *panels;
@@ -41,6 +42,8 @@ GnomeClient *client = NULL;
 /*a list of started extern applet child processes*/
 extern GList * children;
 
+GList *check_swallows = NULL;
+
 static int
 menu_age_timeout(gpointer data)
 {
@@ -90,6 +93,88 @@ try_config_sync(gpointer data)
 	return TRUE;
 }
 
+static int
+ignore_x_error(Display* d, XErrorEvent* e)
+{
+	return 0;
+}
+
+static int
+get_window_id(Window win, char *title, guint32 *wid)
+{
+	Window root_return;
+	Window parent_return;
+	Window *children = NULL;
+	unsigned int nchildren;
+	unsigned int i;
+	char *tit;
+	int ret = FALSE;
+	
+	if(XFetchName(GDK_DISPLAY(), win, &tit) && tit) {
+		if(strstr(tit,title)!=NULL) {
+			if(wid) *wid = win;
+			ret = TRUE;
+		}
+		XFree(tit);
+	}
+	
+	if(ret) return TRUE;
+
+	XQueryTree(GDK_DISPLAY(),
+		   win,
+		   &root_return,
+		   &parent_return,
+		   &children,
+		   &nchildren);
+	
+	/*otherwise we got a problem*/
+	if(children) {
+		for(i=0;!ret && i<nchildren;i++)
+			ret=get_window_id(children[i],title,wid);
+		XFree(children);
+	}
+	return ret;
+}
+
+static GdkFilterReturn
+event_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
+{
+	XEvent *xevent;
+
+	xevent = (XEvent *)gdk_xevent;
+	if (xevent->type == MapNotify && check_swallows) {
+		int (*oldErrorHandler)(Display*, XErrorEvent*);
+		GList *li;
+		int remove;
+
+		oldErrorHandler = XSetErrorHandler(ignore_x_error);
+
+		remove = 0;
+		for(li = check_swallows; li; li = g_list_next(li)) {
+			Swallow *swallow = li->data;
+			if(get_window_id(xevent->xmap.window,swallow->title,
+					 &(swallow->wid))) {
+				gtk_socket_steal(GTK_SOCKET(swallow->socket),swallow->wid);
+				li->data = NULL;
+				remove++;
+			}
+		}
+		while(remove--)
+			check_swallows = g_list_remove(check_swallows,NULL);
+		
+		if(!check_swallows)
+			XSelectInput(GDK_DISPLAY(), GDK_ROOT_WINDOW(), 0);
+		
+		XSetErrorHandler(oldErrorHandler);
+	}
+	if ((event->any.window) &&
+	    (gdk_window_get_type(event->any.window) == GDK_WINDOW_FOREIGN))
+		return GDK_FILTER_REMOVE;
+	else
+		return GDK_FILTER_CONTINUE;
+}
+
+
 int
 main(int argc, char **argv)
 {
@@ -107,44 +192,39 @@ main(int argc, char **argv)
 			       GNORBA_INIT_SERVER_FUNC, &ev);
 	CORBA_exception_free(&ev);
 
-	{
-		CORBA_Object exist;
-		exist = goad_server_activate_with_repo_id(NULL, "IDL:GNOME/Panel:1.0", GOAD_ACTIVATE_EXISTING_ONLY, NULL);
-		if(!CORBA_Object_is_nil(exist, &ev))
-			duplicate = TRUE;
-		else
-			CORBA_Object_release(exist, &ev);
+	switch (panel_corba_gtk_init (orb)) {
+	case 0: 
+		duplicate = 0;
+		break; /* success */
+	case -4: {
+		GtkWidget* box = gnome_question_dialog
+			(_("I've detected a panel already running.\n"
+			   "Start another panel as well?\n" 
+			   "(The new panel will not be restarted.)"), NULL, NULL);
+		if (gnome_dialog_run_and_close (GNOME_DIALOG (box)))
+			return 0;
+		duplicate = 1;
+		break;
+	}
+	default: {
+		GtkWidget *box = gnome_error_dialog
+			(_("There was a problem registering the panel "
+			   "with the GOAD server.\n"
+			   "The panel will now exit."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (box));
+		return 0;
+		break;
+	}
 	}
-
-	duplicate = (panel_corba_gtk_init(orb) == -4);
 
 	client = gnome_master_client ();
 
-#if 0
-	if (! duplicate && 
-	    (gnome_client_get_flags(client) & GNOME_CLIENT_IS_CONNECTED)) {
-	}
-#endif
+	gnome_client_set_restart_style (client, duplicate 
+					? GNOME_RESTART_NEVER 
+					: GNOME_RESTART_IMMEDIATELY);
 
-	gnome_client_set_restart_style (client, duplicate ? GNOME_RESTART_NEVER : GNOME_RESTART_IMMEDIATELY);
-
-	if (duplicate) {
-		GtkWidget* box = gnome_question_dialog
-		  (_("I've detected a panel already running.\n"
-		     "Start another panel as well?\n" 
-		     "(The new panel will not be restarted.)"), NULL, NULL);
-		if (gnome_dialog_run_and_close (GNOME_DIALOG (box))) {
-			gnome_client_flush (client);
-			return 0;
-		}
-	}
 	gnome_client_set_priority(client,40);
 
-	/* Since we are implementing only a single session, that
-	 * either can be saved by the user on exit, or not saved,
-	 * it is most natural if we make the saved state global,
-	 * not per-session
-	 */
 
 	if (gnome_client_get_flags(client) & GNOME_CLIENT_RESTORED)
 		old_panel_cfg_path = g_strdup (gnome_client_get_config_prefix (client));
@@ -160,7 +240,8 @@ main(int argc, char **argv)
 	g_free (real_global_path);
 #endif /* !PER_SESSION_CONFIGURATION */
 
-	gnome_client_set_global_config_prefix (client, "/panel.d/Session/");
+	gnome_client_set_global_config_prefix (client, PANEL_CONFIG_PATH);
+	
 	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
 			    GTK_SIGNAL_FUNC (panel_session_save), NULL);
 	gtk_signal_connect (GTK_OBJECT (client), "die",
@@ -168,6 +249,11 @@ main(int argc, char **argv)
 
 	panel_tooltips = gtk_tooltips_new();
 
+	gnome_win_hints_init ();
+
+	/* read, convert and remove old config */
+	convert_old_config();
+
 	/*set the globals*/
 	load_up_globals();
 	
@@ -184,7 +270,7 @@ main(int argc, char **argv)
 
 	/*this will make the drawers be hidden for closed panels etc ...*/
 	send_state_change();
-	
+
 	/*attempt to sync the config every 10 seconds, only if a change was
 	  indicated though*/
 	config_sync_timeout = gtk_timeout_add(10*1000,try_config_sync,NULL);
@@ -194,6 +280,10 @@ main(int argc, char **argv)
 	/*load these as the last thing to prevent some races any races from
 	  starting multiple goad_id's at once are libgnorba's problem*/
 	load_queued_externs();
+
+	/* set up a filter on the root window to get map requests */
+	/* we will select the events later when we actually need them */
+	gdk_window_add_filter(GDK_ROOT_PARENT(), event_filter, NULL);
 	
 	/* I use the glue code to avoid making this a C++ file */
 	gtk_main ();
