@@ -1,6 +1,7 @@
 #include <config.h>
 #include <string.h>
 #include <glib.h>
+#include <sys/types.h>
 #include <fcntl.h>
 #include <dirent.h>
 
@@ -10,7 +11,7 @@ int
 string_is_in_list(GSList *list,char *text)
 {
 	for(;list!=NULL;list=g_slist_next(list))
-		if(strcmp(text,list->data)==0)
+		if(strcmp(text,(char *)list->data)==0)
 			return TRUE;
 	return FALSE;
 }
@@ -47,9 +48,11 @@ create_text_entry(GtkWidget *table,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
-					       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
-					       GTK_OBJECT(w));
+	if(w) {
+		gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
+						       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
+						       GTK_OBJECT(w));
+	}
 	return entry;
 }
 
@@ -86,16 +89,18 @@ create_pixmap_entry(GtkWidget *table,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
-					       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
-					       GTK_OBJECT(w));
+	if(w) {
+		gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
+						       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
+						       GTK_OBJECT(w));
+	}
 	return entry;
 }
 
 GtkWidget *
 create_icon_entry(GtkWidget *table,
 		  char *history_id,
-		  int row,
+		  int cols, int cole,
 		  char *label,
 		  char *text,
 		  GtkWidget *w)
@@ -107,9 +112,9 @@ create_icon_entry(GtkWidget *table,
 	wlabel = gtk_label_new(label);
 	gtk_misc_set_alignment(GTK_MISC(wlabel), 0.0, 0.5);
 	gtk_table_attach(GTK_TABLE(table), wlabel,
-			 0, 1, row, row + 1,
+			 cols, cole, 2, 3,
+			 GTK_SHRINK,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_widget_show(wlabel);
 
@@ -119,14 +124,16 @@ create_icon_entry(GtkWidget *table,
 
 	t = gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY (entry));
 	gtk_table_attach(GTK_TABLE(table), entry,
-			 1, 2, row, row + 1,
+			 cols, cole, 1, 2,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
-					       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
-					       GTK_OBJECT(w));
+	if(w) {
+		gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
+						       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
+						       GTK_OBJECT(w));
+	}
 	return entry;
 }
 
@@ -161,9 +168,11 @@ create_file_entry(GtkWidget *table,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
-					       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
-					       GTK_OBJECT(w));
+	if(w) {
+		gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
+						       GTK_SIGNAL_FUNC(gnome_property_box_changed), 
+						       GTK_OBJECT(w));
+	}
 	return entry;
 }
 
