@@ -5,10 +5,6 @@
 
 BEGIN_GNOME_DECLS
 
-#ifndef __PANEL_WIDGET_H__
-typedef struct _PanelWidget            PanelWidget;
-#endif
-
 int string_is_in_list(GSList *list,char *text);
 GtkWidget * create_text_entry(GtkWidget *table,
 			      char *history_id,
@@ -31,7 +27,8 @@ GtkWidget * create_pixmap_entry(GtkWidget *table,
 				int pw, int ph /*preview size*/);
 GtkWidget * create_icon_entry(GtkWidget *table,
 			      char *history_id,
-			      int row,
+			      int cols,
+			      int cole,
 			      char *label,
 			      char *text,
 			      GtkWidget *w);
