@@ -11,6 +11,13 @@
 #include "panel-widget.h"
 #include "button-widget.h"
 #include "panel-util.h"
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libart_lgpl/art_alphagamma.h>
+#include <libart_lgpl/art_filterlevel.h>
+#include <libart_lgpl/art_pixbuf.h>
+#include <libart_lgpl/art_rgb_pixbuf_affine.h>
+#include <libart_lgpl/art_affine.h>
+#include "rgb-stuff.h"
 
 GSList *panels=NULL; /*other panels we might want to move the applet to*/
 
@@ -40,6 +47,8 @@ static int  panel_widget_expose		(GtkWidget        *widget,
 					 GdkEventExpose   *event);
 static void panel_widget_draw		(GtkWidget        *widget,
 					 GdkRectangle     *area);
+static void applet_move			(PanelWidget      *panel,
+					 GtkWidget        *applet);
 
 
 /*global settings*/
@@ -57,20 +66,12 @@ int pw_applet_padding = 3;
 			   GDK_POINTER_MOTION_MASK |		\
 			   GDK_POINTER_MOTION_HINT_MASK)
 
-typedef void (*OrientSignal) (GtkObject * object,
-			      PanelOrientation orient,
-			      gpointer data);
-
 typedef void (*BackSignal) (GtkObject * object,
 			    PanelBackType type,
 			    char *pixmap,
 			    GdkColor *color,
 			    gpointer data);
 
-typedef void (*AppletSignal) (GtkObject * object,
-			      GtkWidget * applet,
-			      gpointer data);
-
 /************************
  debugging
  ************************/
@@ -111,8 +112,8 @@ panel_widget_get_type ()
 			sizeof (PanelWidgetClass),
 			(GtkClassInitFunc) panel_widget_class_init,
 			(GtkObjectInitFunc) panel_widget_init,
-			NULL, NULL,
-			(GtkClassInitFunc) NULL
+			(GtkArgSetFunc) NULL,
+			(GtkArgGetFunc) NULL,
 		};
 
 		panel_widget_type =
@@ -125,6 +126,7 @@ panel_widget_get_type ()
 
 enum {
 	ORIENT_CHANGE_SIGNAL,
+	SIZE_CHANGE_SIGNAL,
 	APPLET_MOVE_SIGNAL,
 	APPLET_ADDED_SIGNAL,
 	APPLET_REMOVED_SIGNAL,
@@ -132,38 +134,10 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint panel_widget_signals[LAST_SIGNAL] = {0,0,0,0,0};
+static int panel_widget_signals[LAST_SIGNAL] = {0,0,0,0,0};
 static GtkFixedClass *parent_class = NULL;
 
 
-static void
-marshal_signal_orient (GtkObject * object,
-		       GtkSignalFunc func,
-		       gpointer func_data,
-		       GtkArg * args)
-{
-	OrientSignal rfunc;
-
-	rfunc = (OrientSignal) func;
-
-	(*rfunc) (object, GTK_VALUE_ENUM (args[0]),
-		  func_data);
-}
-
-static void
-marshal_signal_applet (GtkObject * object,
-		       GtkSignalFunc func,
-		       gpointer func_data,
-		       GtkArg * args)
-{
-	AppletSignal rfunc;
-
-	rfunc = (AppletSignal) func;
-
-	(*rfunc) (object, GTK_VALUE_POINTER (args[0]),
-		  func_data);
-}
-
 static void
 marshal_signal_back (GtkObject * object,
 		     GtkSignalFunc func,
@@ -195,7 +169,17 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
 			       			 orient_change),
-			       marshal_signal_orient,
+			       gtk_marshal_NONE__ENUM,
+			       GTK_TYPE_NONE,
+			       1,
+			       GTK_TYPE_ENUM);
+	panel_widget_signals[SIZE_CHANGE_SIGNAL] =
+		gtk_signal_new("size_change",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
+			       			 size_change),
+			       gtk_marshal_NONE__ENUM,
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_ENUM);
@@ -205,7 +189,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
 			       			 applet_move),
-			       marshal_signal_applet,
+			       gtk_marshal_NONE__POINTER,
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_POINTER);
@@ -215,7 +199,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
 			       			 applet_added),
-			       marshal_signal_applet,
+			       gtk_marshal_NONE__POINTER,
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_POINTER);
@@ -225,7 +209,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
 			       			 applet_removed),
-			       marshal_signal_applet,
+			       gtk_marshal_NONE__POINTER,
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_POINTER);
@@ -245,7 +229,8 @@ panel_widget_class_init (PanelWidgetClass *class)
 				     LAST_SIGNAL);
 
 	class->orient_change = NULL;
-	class->applet_move = NULL;
+	class->size_change = NULL;
+	class->applet_move = applet_move;
 	class->applet_added = NULL;
 	class->applet_removed = NULL;
 	class->back_change = NULL;
@@ -259,6 +244,22 @@ panel_widget_class_init (PanelWidgetClass *class)
 	container_class->remove = panel_widget_cremove;
 }
 
+static void
+applet_move(PanelWidget *panel, GtkWidget *applet)
+{
+	if(panel->back_type == PANEL_BACK_COLOR ||
+	   (panel->back_type == PANEL_BACK_NONE &&
+	    !GTK_WIDGET(panel)->style->bg_pixmap[GTK_WIDGET_STATE(panel)]))
+		return;
+	if(IS_BUTTON_WIDGET(applet)) {
+		ButtonWidget *button = BUTTON_WIDGET(applet);
+		if(!button->no_alpha && button->cache) {
+			gdk_pixmap_unref(button->cache);
+			button->cache = NULL;
+		}
+	}
+}
+
 static void
 remove_panel_from_forbidden(PanelWidget *panel, PanelWidget *r)
 {
@@ -507,8 +508,7 @@ static void
 panel_widget_switch_applet_right(PanelWidget *panel, GList *list)
 {
 	AppletData *ad;
-	AppletData *nad;
-	GList *nlist;
+	AppletData *nad = NULL;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
@@ -516,17 +516,17 @@ panel_widget_switch_applet_right(PanelWidget *panel, GList *list)
 	
 	ad = list->data;
 
-	nlist = g_list_next(list);
-	if(nlist)
-		nad = nlist->data;
-	if(!nlist || nad->pos > ad->pos+ad->cells) {
+	if(list->next)
+		nad = list->next->data;
+	if(!nad || nad->pos > ad->pos+ad->cells) {
 		ad->pos++;
 		panel_widget_queue_applet_for_resize(ad);
 		return;
 	}
+
 	nad->pos = ad->pos;
 	ad->pos = nad->pos+nad->cells;
-	panel->applet_list = my_g_list_swap_prev(panel->applet_list,nlist);
+	panel->applet_list = my_g_list_swap_next(panel->applet_list,list);
 
 	panel_widget_queue_applet_for_resize(ad);
 	panel_widget_queue_applet_for_resize(nad);
@@ -536,8 +536,7 @@ static void
 panel_widget_switch_applet_left(PanelWidget *panel, GList *list)
 {
 	AppletData *ad;
-	AppletData *pad;
-	GList *nlist;
+	AppletData *pad = NULL;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
@@ -545,17 +544,16 @@ panel_widget_switch_applet_left(PanelWidget *panel, GList *list)
 	
 	ad = list->data;
 
-	nlist = g_list_previous(list);
-	if(nlist)
-		pad = nlist->data;
-	if(!nlist || pad->pos+pad->cells < ad->pos) {
+	if(list->prev)
+		pad = list->prev->data;
+	if(!pad || pad->pos+pad->cells < ad->pos) {
 		ad->pos--;
 		panel_widget_queue_applet_for_resize(ad);
 		return;
 	}
 	ad->pos = pad->pos;
 	pad->pos = ad->pos+ad->cells;
-	panel->applet_list = my_g_list_swap_next(panel->applet_list,nlist);
+	panel->applet_list = my_g_list_swap_prev(panel->applet_list,list);
 
 	panel_widget_queue_applet_for_resize(ad);
 	panel_widget_queue_applet_for_resize(pad);
@@ -565,7 +563,7 @@ static int
 panel_widget_get_right_switch_pos(PanelWidget *panel, GList *list)
 {
 	AppletData *ad;
-	AppletData *nad;
+	AppletData *nad = NULL;
 
 	g_return_val_if_fail(panel!=NULL,-1);
 	g_return_val_if_fail(IS_PANEL_WIDGET(panel),-1);
@@ -573,10 +571,9 @@ panel_widget_get_right_switch_pos(PanelWidget *panel, GList *list)
 	
 	ad = list->data;
 
-	list = g_list_next(list);
-	if(list)
-		nad = list->data;
-	if(!list || nad->pos > ad->pos+ad->cells)
+	if(list->next)
+		nad = list->next->data;
+	if(!nad || nad->pos > ad->pos+ad->cells)
 		return ad->pos+1;
 	return nad->pos+nad->cells-ad->cells;
 }
@@ -585,7 +582,7 @@ static int
 panel_widget_get_left_switch_pos(PanelWidget *panel, GList *list)
 {
 	AppletData *ad;
-	AppletData *pad;
+	AppletData *pad = NULL;
 
 	g_return_val_if_fail(panel!=NULL,-1);
 	g_return_val_if_fail(IS_PANEL_WIDGET(panel),-1);
@@ -593,10 +590,9 @@ panel_widget_get_left_switch_pos(PanelWidget *panel, GList *list)
 	
 	ad = list->data;
 
-	list = g_list_previous(list);
-	if(list)
-		pad = list->data;
-	if(!list || pad->pos+pad->cells < ad->pos)
+	if(list->prev)
+		pad = list->prev->data;
+	if(!pad || pad->pos+pad->cells < ad->pos)
 		return ad->pos-1;
 	return pad->pos;
 }
@@ -638,8 +634,109 @@ panel_widget_switch_move(PanelWidget *panel, AppletData *ad, int moveby)
 			return;
 		panel_widget_switch_applet_left(panel,list);
 	}
+}
 
-	return;
+static int
+push_applet_right(PanelWidget *panel, GList *list)
+{
+	AppletData *ad = NULL;
+	AppletData *nad = NULL;
+
+	g_return_val_if_fail(panel!=NULL,FALSE);
+	g_return_val_if_fail(IS_PANEL_WIDGET(panel),FALSE);
+	
+	if (list) {
+		ad = list->data;
+	
+		if(ad->pos + ad->cells >= panel->size)
+			return FALSE;
+
+		if(list->next)
+			nad = list->next->data;
+	}
+
+	if(!nad || nad->pos > ad->pos+ad->cells) {
+		ad->pos++;
+		panel_widget_queue_applet_for_resize(ad);
+		return TRUE;
+	}
+
+	g_return_val_if_fail(list!=NULL,FALSE);
+	
+	if(!push_applet_right(panel,list->next)) {
+		return FALSE;
+	}
+	ad->pos++;
+	panel_widget_queue_applet_for_resize(ad);
+	return TRUE;
+}
+
+static int
+push_applet_left(PanelWidget *panel, GList *list)
+{
+	AppletData *ad = NULL;
+	AppletData *pad = NULL;
+
+	g_return_val_if_fail(panel!=NULL,FALSE);
+	g_return_val_if_fail(IS_PANEL_WIDGET(panel),FALSE);
+
+	if (list) {
+		ad = list->data;
+
+		if(ad->pos <= pw_applet_padding)
+			return FALSE;
+
+		if(list->prev)
+			pad = list->prev->data;
+	}
+
+	if(!list || !pad || pad->pos+pad->cells < ad->pos) {
+		ad->pos--;
+		panel_widget_queue_applet_for_resize(ad);
+		return TRUE;
+	}
+
+	g_return_val_if_fail(list!=NULL,FALSE);
+	
+	if(!push_applet_left(panel,list->prev)) {
+		return FALSE;
+	}
+	ad->pos--;
+	panel_widget_queue_applet_for_resize(ad);
+	return TRUE;
+}
+
+static void
+panel_widget_push_move(PanelWidget *panel, AppletData *ad, int moveby)
+{
+	int finalpos;
+	GList *list;
+
+	g_return_if_fail(ad!=NULL);
+	g_return_if_fail(panel!=NULL);
+	g_return_if_fail(IS_PANEL_WIDGET(panel));
+
+	if(moveby==0)
+		return;
+
+	list = g_list_find(panel->applet_list,ad);
+	g_return_if_fail(list!=NULL);
+	
+	finalpos = ad->pos+moveby;
+
+	if(finalpos >= panel->size)
+		finalpos = panel->size-1;
+	else if(finalpos < pw_applet_padding)
+		finalpos = pw_applet_padding;
+
+	while((ad->pos+ad->cells-1)<finalpos) {
+		if(!push_applet_right(panel,list))
+			return;
+	}
+	while(ad->pos>finalpos) {
+		if(!push_applet_left(panel,list))
+			return;
+	}
 }
 
 /*this is a special function and may fail if called improperly, it works
@@ -699,10 +796,10 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
 	if(panel->orient == PANEL_HORIZONTAL) {
 		requisition->width = pw_applet_padding;
-		requisition->height = PANEL_MINIMUM_WIDTH;
+		requisition->height = panel_widget_get_pixel_size(panel);
 	} else {
 		requisition->height = pw_applet_padding;
-		requisition->width = PANEL_MINIMUM_WIDTH;
+		requisition->width = panel_widget_get_pixel_size(panel);
 	}
 
 	for(list = panel->applet_list; list!=NULL; list = g_list_next(list)) {
@@ -732,6 +829,68 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 	}
 }
 
+static void
+make_background(PanelWidget *panel, guchar *rgb_buf,
+		int x, int y, int w, int h,
+		GdkPixbuf *pb, int scale_w, int scale_h)
+{
+	if(pb) {
+		if(scale_w == 0 || scale_h == 0) {
+			tile_rgb(rgb_buf,w,h,x,y,w*3,
+				 pb->art_pixbuf->pixels,
+				 pb->art_pixbuf->width,
+				 pb->art_pixbuf->height,
+				 pb->art_pixbuf->rowstride,
+				 pb->art_pixbuf->has_alpha);
+		} else {
+			tile_rgb_pixbuf(rgb_buf, w, h, x, y, w*3,
+					pb->art_pixbuf, scale_w, scale_h);
+		}
+	} else {
+		int i;
+		int size;
+		int r,g,b;
+		guchar *p;
+		if(panel->back_type != PANEL_BACK_COLOR) {
+			GtkWidget *widget = GTK_WIDGET(panel);
+			/* convert to 8 bit per channel */
+			r = widget->style->bg[GTK_WIDGET_STATE(widget)].red>>8;
+			g = widget->style->bg[GTK_WIDGET_STATE(widget)].green>>8;
+			b = widget->style->bg[GTK_WIDGET_STATE(widget)].blue>>8;
+		} else {
+			/* convert to 8 bit per channel */
+			r = panel->back_color.red>>8;
+			g = panel->back_color.green>>8;
+			b = panel->back_color.blue>>8;
+		}
+		size = w*h;
+		p = rgb_buf;
+		for(i=0;i<size;i++) {
+			(*p++) = r;
+			(*p++) = g;
+			(*p++) = b;
+		}
+	}
+}
+
+static void
+kill_cache_on_all_buttons(PanelWidget *panel, int even_no_alpha)
+{
+	GList *li;
+	for(li = panel->applet_list; li != NULL;
+	    li = g_list_next(li)) {
+		AppletData *ad = li->data;
+		if(IS_BUTTON_WIDGET(ad->applet)) {
+			ButtonWidget *button = BUTTON_WIDGET(ad->applet);
+			if((even_no_alpha || !button->no_alpha)
+			   && button->cache) {
+				gdk_pixmap_unref(button->cache);
+				button->cache = NULL;
+			}
+		}
+	}
+}
+
 void
 panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 {
@@ -739,7 +898,11 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 	GtkWidget *widget;
 	GdkGC *gc;
 	GdkPixmap *pixmap;
+	GdkPixmap *bg_pixmap;
 	GdkRectangle da;
+	GdkPixbuf *pb = NULL;
+	int size;
+	int scale_w = 0,scale_h = 0;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
@@ -750,7 +913,7 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 	   widget->allocation.width <= 0 ||
 	   widget->allocation.height <= 0)
 		return;
-	
+
 	if(area) {
 		if(area->width==0 || area->height==0)
 			return;
@@ -762,6 +925,29 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 		da.height = widget->allocation.height;
 	}
 	
+	bg_pixmap = widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)];
+
+	if(panel->back_type == PANEL_BACK_NONE) {
+		if(bg_pixmap && !panel->backpix) {
+			if(panel->backpix)
+				gdk_pixbuf_unref(panel->backpix);
+			panel->backpix = my_gdk_pixbuf_rgb_from_drawable(bg_pixmap);
+			kill_cache_on_all_buttons(panel, FALSE);
+		} else if(!bg_pixmap && panel->backpix) {
+			if(panel->backpix)
+				gdk_pixbuf_unref(panel->backpix);
+			panel->backpix = NULL;
+			kill_cache_on_all_buttons(panel, FALSE);
+		}
+		pb = panel->backpix;
+	} else if(panel->back_type == PANEL_BACK_PIXMAP) {
+		if(panel->fit_pixmap_bg) {
+			scale_w = panel->scale_w;
+			scale_h = panel->scale_h;
+		}
+		pb = panel->backpix;
+	} 
+	
 	pixmap = gdk_pixmap_new(widget->window,
 				da.width,da.height,
 				gtk_widget_get_visual(widget)->depth);
@@ -769,24 +955,68 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 	gc = gdk_gc_new(pixmap);
 	gdk_gc_copy(gc,widget->style->bg_gc[GTK_WIDGET_STATE(widget)]);
 
-	if(widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]) {
-		gdk_gc_set_fill(gc, GDK_TILED);
-		gdk_gc_set_tile(gc, widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]);
-		gdk_gc_set_ts_origin(gc,-da.x,-da.y);
+	if(panel->back_type == PANEL_BACK_NONE) {
+		if(widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]) {
+			gdk_gc_set_fill(gc, GDK_TILED);
+			gdk_gc_set_tile(gc,
+					widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]);
+			gdk_gc_set_ts_origin(gc,-da.x,-da.y);
+		}
+	} else if(panel->back_type == PANEL_BACK_PIXMAP) {
+		if(panel->backpixmap) {
+			gdk_gc_set_fill(gc, GDK_TILED);
+			gdk_gc_set_tile(gc, panel->backpixmap);
+			gdk_gc_set_ts_origin(gc,-da.x,-da.y);
+		}
+	} else /*COLOR*/ {
+		gdk_gc_set_foreground(gc, &panel->back_color);
 	}
-				
+
+
 	gdk_draw_rectangle(pixmap,gc, TRUE, 0,0,-1,-1);
 
-	gdk_gc_unref(gc);
+	/* get pixel size of an icon */
+	size = panel_widget_get_pixel_size(panel);
 
 	for(li = panel->applet_list; li != NULL;
 	    li = g_list_next(li)) {
 		AppletData *ad = li->data;
 		if(IS_BUTTON_WIDGET(ad->applet) &&
-		   (!area || gtk_widget_intersect(ad->applet, area, NULL)))
-			button_widget_draw(BUTTON_WIDGET(ad->applet),
-					   pixmap,da.x,da.y);
+		   ad->applet->allocation.x>=0 &&
+		   (!area || gtk_widget_intersect(ad->applet, area, NULL))) {
+			ButtonWidget *button = BUTTON_WIDGET(ad->applet);
+			if(!button->cache) {
+				guchar *rgb_buf;
+
+				button->cache =
+					gdk_pixmap_new(widget->window, size,size,
+						       gtk_widget_get_visual(widget)->depth);
+				rgb_buf = g_new0(guchar, size*size*3);
+				/*if the icon doesn't have an opaque tile,
+				  draw us a background*/
+				if(!button->no_alpha)
+					make_background(panel, rgb_buf,
+							ad->applet->allocation.x,
+							ad->applet->allocation.y,
+							size,size, pb, scale_w, scale_h);
+				button_widget_draw(button, rgb_buf, size*3);
+				gdk_draw_rgb_image(button->cache,gc,
+						   0,0, size, size,
+						   GDK_RGB_DITHER_NORMAL,
+						   rgb_buf, size*3);
+				g_free(rgb_buf);
+				button_widget_draw_xlib(button, button->cache);
+			}
+			gdk_draw_pixmap(pixmap,gc,
+					button->cache,
+					0,0,
+					ad->applet->allocation.x - da.x,
+					ad->applet->allocation.y - da.y,
+					size, size);
+		}
 	}
+	gdk_gc_unref(gc);
+
 	gdk_draw_pixmap(widget->window,
 			widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
 			pixmap,
@@ -924,6 +1154,13 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 	old_size = panel->size;
 	old_thick = panel->thick;
+
+	if((widget->allocation.width != allocation->width ||
+	    widget->allocation.height != allocation->height) &&
+	   panel->back_type != PANEL_BACK_COLOR &&
+	   (panel->back_type != PANEL_BACK_NONE ||
+	    GTK_WIDGET(panel)->style->bg_pixmap[GTK_WIDGET_STATE(panel)]))
+		kill_cache_on_all_buttons(panel, FALSE);
 	
 	widget->allocation = *allocation;
 	if (GTK_WIDGET_REALIZED (widget))
@@ -1096,23 +1333,24 @@ panel_widget_set_back_pixmap (PanelWidget *panel, char *file)
 			g_free (panel->back_pixmap);
 		panel->back_pixmap = g_strdup (file);
 		panel->back_type = PANEL_BACK_PIXMAP;
+		panel_widget_draw_all(panel,NULL);
+
 		gtk_signal_emit(GTK_OBJECT(panel),
 				panel_widget_signals[BACK_CHANGE_SIGNAL],
 				panel->back_type,
 				panel->back_pixmap,
 				&panel->back_color);
-	}
+	} else
+		panel_widget_draw_all(panel,NULL);
 }
 
 static void
 panel_try_to_set_default_back(PanelWidget *panel)
 {
-	GtkStyle *ns;
-
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 
-	gtk_widget_set_rc_style(GTK_WIDGET(panel));
+	kill_cache_on_all_buttons(panel, FALSE);
 
 	panel_widget_draw_all(panel,NULL);
 }
@@ -1120,29 +1358,21 @@ panel_try_to_set_default_back(PanelWidget *panel)
 static void
 panel_try_to_set_back_color(PanelWidget *panel, GdkColor *color)
 {
-	GtkStyle *ns;
+	GdkColormap *cmap;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 	g_return_if_fail(color!=NULL);
+	
+	kill_cache_on_all_buttons(panel, FALSE);
 
-	ns = gtk_style_copy(GTK_WIDGET(panel)->style);
-	gtk_style_ref(ns);
+	cmap = gtk_widget_get_colormap(GTK_WIDGET(panel));
 
-	ns->bg[GTK_STATE_NORMAL] = panel->back_color = *color;
-	ns->bg[GTK_STATE_NORMAL].pixel =
-		panel->back_color.pixel = 1; /* bogus */
+	if(gdk_colormap_alloc_color(cmap,color,FALSE,TRUE)) {
+		panel->back_color = *color;
 
-	if(ns->bg_pixmap[GTK_STATE_NORMAL]) {
-		gdk_pixmap_unref(ns->bg_pixmap[GTK_STATE_NORMAL]);
-		ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;
+		panel_widget_draw_all(panel,NULL);
 	}
-
-	gtk_widget_set_style(GTK_WIDGET(panel), ns);
-
-	gtk_style_unref(ns);
-
-	panel_widget_draw_all(panel,NULL);
 }
 
 void
@@ -1152,9 +1382,9 @@ panel_widget_set_back_color(PanelWidget *panel, GdkColor *color)
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 	g_return_if_fail(color!=NULL);
 
+	panel->back_type = PANEL_BACK_COLOR;
 	panel_try_to_set_back_color(panel, color);
 
-	panel->back_type = PANEL_BACK_COLOR;
 	gtk_signal_emit(GTK_OBJECT(panel),
 			panel_widget_signals[BACK_CHANGE_SIGNAL],
 			panel->back_type,
@@ -1162,48 +1392,76 @@ panel_widget_set_back_color(PanelWidget *panel, GdkColor *color)
 			&panel->back_color);
 }
 
+static GdkPixmap *
+get_pixmap_from_pixbuf(GtkWidget *w, GdkPixbuf *pb, int scale_w, int scale_h)
+{
+	GdkGC *gc;
+	GdkPixmap *p;
+	gdouble affine[6];
+	guchar *rgb;
+	p = gdk_pixmap_new(w->window, scale_w,scale_h,
+			   gtk_widget_get_visual(GTK_WIDGET(w))->depth);
+	gc = gdk_gc_new(p);
+	
+	affine[1] = affine[2] = affine[4] = affine[5] = 0;
+
+	affine[0] = scale_w / (double)(pb->art_pixbuf->width);
+	affine[3] = scale_h / (double)(pb->art_pixbuf->height);
+	
+	rgb = g_new0(guchar,scale_w*scale_h*3);
+	art_rgb_pixbuf_affine(rgb,
+			      0,0,scale_w,scale_h,scale_w*3,
+			      pb->art_pixbuf,affine,
+			      ART_FILTER_NEAREST,NULL);
+	gdk_draw_rgb_image(p,gc,0,0,
+			   scale_w,scale_h,
+			   GDK_RGB_DITHER_NORMAL,
+			   rgb, scale_w*3);
+	g_free(rgb);
+
+	gdk_gc_destroy(gc);
+	return p;
+}
+
+
 static void
 panel_resize_pixmap(PanelWidget *panel)
 {
-	GdkImlibImage *im;
-	GdkPixmap *p;
-	GtkStyle *ns;
 	int w, h;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
+	
+	if(!panel->backpix) return;
+	
+	if(panel->backpixmap)
+		gdk_pixmap_unref(panel->backpixmap);
+	panel->backpixmap = NULL;
 
-	im = gtk_object_get_data(GTK_OBJECT(panel),"gdk_image");
-	if(!im) return;
-
-	w = im->rgb_width;
-	h = im->rgb_height;
+	panel->scale_w = w = panel->backpix->art_pixbuf->width;
+	panel->scale_w = h = panel->backpix->art_pixbuf->height;
 
 	switch (panel->orient) {
 	case PANEL_HORIZONTAL:
-		gdk_imlib_render (im, w * panel->thick / h, panel->thick);
+		panel->scale_w = w * panel->thick / h;
+		panel->scale_h = panel->thick;
 		break;
 
 	case PANEL_VERTICAL:
-		gdk_imlib_render (im, panel->thick, h * panel->thick / w);
+		panel->scale_w = panel->thick;
+		panel->scale_h = h * panel->thick / w;
 		break;
 
 	default:
 		g_assert_not_reached ();
 	}
 
-	p = gdk_imlib_copy_image (im);
-
-	ns = gtk_style_copy(GTK_WIDGET(panel)->style);
-	gtk_style_ref(ns);
-
-	if(ns->bg_pixmap[GTK_STATE_NORMAL])
-		gdk_pixmap_unref (ns->bg_pixmap[GTK_STATE_NORMAL]);
-	ns->bg_pixmap[GTK_STATE_NORMAL] = p;
-
-	gtk_widget_set_style(GTK_WIDGET(panel), ns);
+	panel->backpixmap = get_pixmap_from_pixbuf(GTK_WIDGET(panel),
+						   panel->backpix,
+						   panel->scale_w,
+						   panel->scale_h);
 	
-	gtk_style_unref(ns);
+	kill_cache_on_all_buttons(panel, FALSE);
 
 	panel_widget_draw_all(panel,NULL);
 }
@@ -1211,77 +1469,63 @@ panel_resize_pixmap(PanelWidget *panel)
 static int
 panel_try_to_set_pixmap (PanelWidget *panel, char *pixmap)
 {
-	GdkImlibImage *im;
-	GdkImlibImage *im2;
-	GdkPixmap *p;
-	GtkStyle *ns;
-
 	g_return_val_if_fail(panel!=NULL,FALSE);
 	g_return_val_if_fail(IS_PANEL_WIDGET(panel),FALSE);
 
+	kill_cache_on_all_buttons(panel, FALSE);
+
+	if(panel->backpix)
+		gdk_pixbuf_unref(panel->backpix);
+	panel->backpix = NULL;
+	if(panel->backpixmap)
+		gdk_pixmap_unref(panel->backpixmap);
+	panel->backpixmap = NULL;
+
 	if(!pixmap || strcmp(pixmap,"")==0) {
-		ns = gtk_style_copy(GTK_WIDGET(panel)->style);
-		gtk_style_ref(ns);
-
-		p = ns->bg_pixmap[GTK_STATE_NORMAL];
-		if(p)
-			gdk_pixmap_unref (p);
-		ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;
-
-		gtk_widget_set_style(GTK_WIDGET(panel), ns);
-	
-		gtk_style_unref(ns);
-
-		panel_widget_draw_all(panel,NULL);
 		return TRUE;
 	}
 
 	if (!g_file_exists (pixmap))
 		return FALSE;
 	
-	im = gdk_imlib_load_image (pixmap);
-	if (!im)
+	panel->backpix = gdk_pixbuf_new_from_file (pixmap);
+	if (!panel->backpix)
 		return FALSE;
-
-	im2 = gtk_object_get_data(GTK_OBJECT(panel),"gdk_image");
-	if(im2) gdk_imlib_destroy_image (im2);
 	
 	if (panel->fit_pixmap_bg) {
 		int w, h;
 
-		w = im->rgb_width;
-		h = im->rgb_height;
+		panel->scale_w = w = panel->backpix->art_pixbuf->width;
+		panel->scale_h = h = panel->backpix->art_pixbuf->height;
 
 		switch (panel->orient) {
 		case PANEL_HORIZONTAL:
-			gdk_imlib_render (im, w * panel->thick / h, panel->thick);
+			panel->scale_w = w * panel->thick / h;
+			panel->scale_h = panel->thick;
 			break;
 
 		case PANEL_VERTICAL:
-			gdk_imlib_render (im, panel->thick, h * panel->thick / w);
+			panel->scale_w = panel->thick;
+			panel->scale_h = h * panel->thick / w;
 			break;
 
 		default:
 			g_assert_not_reached ();
 		}
-	} else
-		gdk_imlib_render (im, im->rgb_width, im->rgb_height);
-
-	p = gdk_imlib_copy_image (im);
-
-	ns = gtk_style_copy(GTK_WIDGET(panel)->style);
-	gtk_style_ref(ns);
-
-	if(ns->bg_pixmap[GTK_STATE_NORMAL])
-		gdk_pixmap_unref (ns->bg_pixmap[GTK_STATE_NORMAL]);
-	ns->bg_pixmap[GTK_STATE_NORMAL] = p;
-
-	gtk_widget_set_style(GTK_WIDGET(panel), ns);
 	
-	gtk_style_unref(ns);
-	gtk_object_set_data(GTK_OBJECT(panel),"gdk_image",im);
-	
-	panel_widget_draw_all(panel,NULL);
+		panel->backpixmap =
+			get_pixmap_from_pixbuf(GTK_WIDGET(panel), 
+					       panel->backpix,
+					       panel->scale_w,
+					       panel->scale_h);
+	} else {
+		panel->backpixmap =
+			get_pixmap_from_pixbuf(GTK_WIDGET(panel),
+					       panel->backpix,
+					       panel->backpix->art_pixbuf->width,
+					       panel->backpix->art_pixbuf->height);
+	}
+
 	return TRUE;
 }
 
@@ -1298,6 +1542,7 @@ panel_widget_realize(GtkWidget *w, gpointer data)
 	if(panel->back_type == PANEL_BACK_PIXMAP) {
 		if (!panel_try_to_set_pixmap (panel, panel->back_pixmap))
 			panel->back_type = PANEL_BACK_NONE;
+		panel_widget_draw_all(panel,NULL);
 	} else if(panel->back_type == PANEL_BACK_COLOR) {
 		panel_try_to_set_back_color(panel, &panel->back_color);
 	}
@@ -1308,15 +1553,14 @@ panel_widget_realize(GtkWidget *w, gpointer data)
 static void
 panel_widget_destroy(GtkWidget *w, gpointer data)
 {
-	GdkImlibImage *im;
 	PanelWidget *panel = PANEL_WIDGET(w);
 
 	g_return_if_fail(w!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(w));
 
-	im = gtk_object_get_data(GTK_OBJECT(w),"gdk_image");
-	if(im) gdk_imlib_destroy_image (im);
-	gtk_object_set_data(GTK_OBJECT(w),"gdk_image",NULL);
+	if(panel->backpix)
+		gdk_pixbuf_unref(panel->backpix);
+	panel->backpix = NULL;
 	
 	/*remove from panels list*/
 	panels = g_slist_remove(panels,w);
@@ -1327,6 +1571,17 @@ panel_widget_destroy(GtkWidget *w, gpointer data)
 		gtk_timeout_remove(panel->pixmap_resize_timeout_top);
 }
 
+static void
+panel_widget_style_set(PanelWidget *panel)
+{
+	if(panel->back_type == PANEL_BACK_NONE) {
+		if(panel->backpix)
+			gdk_pixbuf_unref(panel->backpix);
+		panel->backpix = NULL;
+		kill_cache_on_all_buttons(panel, TRUE);
+	}
+}
+
 static int panel_widget_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 
 static int
@@ -1403,11 +1658,17 @@ panel_widget_init (PanelWidget *panel)
 	panel->no_window_applet_list = NULL;
 	panel->master_widget = NULL;
 	panel->drop_widget = GTK_WIDGET(panel);
+	panel->backpix = NULL;
+	panel->backpixmap = NULL;
 
 	gtk_signal_connect(GTK_OBJECT(panel),
 			   "destroy",
 			   GTK_SIGNAL_FUNC(panel_widget_destroy),
 			   NULL);
+	gtk_signal_connect(GTK_OBJECT(panel),
+			   "style_set",
+			   GTK_SIGNAL_FUNC(panel_widget_style_set),
+			   NULL);
 	gtk_signal_connect(GTK_OBJECT(panel),
 			   "event",
 			   GTK_SIGNAL_FUNC(panel_widget_event),
@@ -1419,6 +1680,7 @@ panel_widget_init (PanelWidget *panel)
 GtkWidget *
 panel_widget_new (int packed,
 		  PanelOrientation orient,
+		  PanelSizeType sz,
 		  PanelBackType back_type,
 		  char *back_pixmap,
 		  int fit_pixmap_bg,
@@ -1426,8 +1688,8 @@ panel_widget_new (int packed,
 {
 	PanelWidget *panel;
 
-	gtk_widget_push_visual (gdk_imlib_get_visual ());
-	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
+	gtk_widget_push_visual (gdk_rgb_get_visual ());
+	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
 	panel = gtk_type_new(panel_widget_get_type());
 
@@ -1449,6 +1711,7 @@ panel_widget_new (int packed,
 	}	
 
 	panel->orient = orient;
+	panel->sz = sz;
 
 	panel->packed = packed;
 	if(packed)
@@ -1468,8 +1731,8 @@ static void
 _panel_widget_applet_drag_start_no_grab(PanelWidget *panel, GtkWidget *applet)
 {
 #ifdef PANEL_DEBUG
-  g_message("Starting drag on a %s at %p\n",
-	    gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
+	g_message("Starting drag on a %s at %p\n",
+		  gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
 #endif
 	panel->currently_dragged_applet =
 		gtk_object_get_data(GTK_OBJECT(applet), PANEL_APPLET_DATA);
@@ -1511,8 +1774,8 @@ static void
 _panel_widget_applet_drag_start(PanelWidget *panel, GtkWidget *applet)
 {
 #ifdef PANEL_DEBUG
-  g_message("Starting drag [grabbed] on a %s at %p\n",
-	    gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
+	g_message("Starting drag [grabbed] on a %s at %p\n",
+		  gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
 #endif
 	_panel_widget_applet_drag_start_no_grab(panel,applet);
 
@@ -1577,6 +1840,48 @@ panel_widget_get_cursorloc(PanelWidget *panel)
 		return y;
 }
 
+/*get amount of free space around the applet (including the applet size),
+  or return 0 on error or if the panel is packed*/
+int
+panel_widget_get_free_space(PanelWidget *panel, GtkWidget *applet)
+{
+	int right,left;
+	GList *li;
+	AppletData *ad;
+
+	g_return_val_if_fail(panel!=NULL,0);
+	g_return_val_if_fail(IS_PANEL_WIDGET(panel),0);
+	g_return_val_if_fail(applet!=NULL,0);
+	g_return_val_if_fail(GTK_IS_WIDGET(applet),0);
+	
+	/*this function doesn't make sense on packed panels*/
+	if(panel->packed)
+		return 0;
+	
+	right = pw_applet_padding;
+	left = panel->size-pw_applet_padding;
+	
+	for(li = panel->applet_list; li; li = g_list_next(li)) {
+		ad = li->data;
+		if(ad->applet == applet)
+			break;
+	}
+	/*the applet is not on this panel*/
+	if(!li)
+		return 0;
+	
+	if(li->prev) {
+		AppletData *pad = li->prev->data;
+		left = pad->pos+pad->cells+pw_applet_padding;
+	}
+	if(li->next) {
+		AppletData *nad = li->next->data;
+		right = nad->pos-pw_applet_padding;
+	}
+	
+	return right-left;
+}
+
 /*calculates the value to move the applet by*/
 static int
 panel_widget_get_moveby(PanelWidget *panel, int pos)
@@ -1741,14 +2046,16 @@ panel_widget_applet_move_to_cursor(PanelWidget *panel)
 	if (panel->currently_dragged_applet) {
 		int moveby;
 		int pos = panel->currently_dragged_applet->pos;
+		int movement;
 		GtkWidget *applet;
 		GSList *forb;
+		GdkModifierType mods;
 
 		applet = panel->currently_dragged_applet->applet;
 		g_assert(GTK_IS_WIDGET(applet));
 		forb = gtk_object_get_data(GTK_OBJECT(applet),
 					   PANEL_APPLET_FORBIDDEN_PANELS);
-
+		
 		if(!panel_widget_is_cursor(panel,10)) {
 			GSList *list;
 			for(list=panels;
@@ -1781,14 +2088,31 @@ panel_widget_applet_move_to_cursor(PanelWidget *panel)
 			}
 		}
 
-		if(pw_movement_type == PANEL_SWITCH_MOVE ||
-		   panel->packed) {
+		gdk_window_get_pointer(GTK_WIDGET(panel)->window,
+				       NULL,NULL,&mods);
+		
+		movement = pw_movement_type;
+		
+		if(panel->packed)
+			movement = PANEL_SWITCH_MOVE;
+		else {
+			if(mods & GDK_CONTROL_MASK)
+				movement = PANEL_SWITCH_MOVE;
+			else if(mods & GDK_SHIFT_MASK)
+				movement = PANEL_PUSH_MOVE;
+			else if(mods & GDK_MOD1_MASK)
+				movement = PANEL_FREE_MOVE;
+		}
+
+		switch(movement) {
+		case PANEL_SWITCH_MOVE:
 			moveby = panel_widget_get_moveby(panel,pos);
 			if(moveby != 0)
 				panel_widget_switch_move(panel,
 					panel->currently_dragged_applet,
 					moveby);
-		} else {
+			break;
+		case PANEL_FREE_MOVE:
 			pos = panel_widget_get_free_spot(panel,
 					panel->currently_dragged_applet);
 
@@ -1796,6 +2120,14 @@ panel_widget_applet_move_to_cursor(PanelWidget *panel)
 				panel_widget_nice_move(panel,
 					panel->currently_dragged_applet,
 					pos);
+			break;
+		case PANEL_PUSH_MOVE:
+			moveby = panel_widget_get_moveby(panel,pos);
+			if(moveby != 0)
+				panel_widget_push_move(panel,
+					panel->currently_dragged_applet,
+					moveby);
+			break;
 		}
 		return TRUE;
 	}
@@ -1821,8 +2153,6 @@ move_timeout_handler(gpointer data)
 	return FALSE;
 }
 
-
-
 static int
 panel_widget_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -2045,7 +2375,7 @@ panel_widget_add_forbidden(PanelWidget *panel)
 }
 
 int
-panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_lower_events)
+panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_lower_events, int insert_at_pos)
 {
 	AppletData *ad = NULL;
 
@@ -2062,15 +2392,18 @@ panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_
 
 	if(pos<pw_applet_padding)
 		pos = pw_applet_padding;
-
-	if(pw_movement_type == PANEL_SWITCH_MOVE ||
-	   panel->packed) {
-		if(get_applet_list_pos(panel,pos)) 
-			/*this is a slight hack so that this applet is
-			  inserted AFTER an applet with this pos number*/
-			pos++;
-	} else
-		pos = panel_widget_find_empty_pos(panel,pos);
+	
+	if(!insert_at_pos) {
+		if(pw_movement_type != PANEL_FREE_MOVE ||
+		   panel->packed) {
+			if(get_applet_list_pos(panel,pos)) 
+				/*this is a slight hack so that this applet
+				  is inserted AFTER an applet with this pos
+				  number*/
+				pos++;
+		} else
+			pos = panel_widget_find_empty_pos(panel,pos);
+	}
 
 	if(pos==-1) return -1;
 
@@ -2141,6 +2474,10 @@ panel_widget_reparent (PanelWidget *old_panel,
 
 	/*reparent applet*/
 	if (IS_BUTTON_WIDGET (applet)) {
+		ButtonWidget *button = BUTTON_WIDGET(applet);
+		if(button->cache)
+			gdk_pixmap_unref(button->cache);
+		button->cache = NULL;
 		gtk_widget_ref (applet);
 		gtk_container_remove (GTK_CONTAINER (applet->parent), applet);
 		gtk_container_add (GTK_CONTAINER (new_panel), applet);
@@ -2218,12 +2555,14 @@ panel_widget_get_pos(PanelWidget *panel, GtkWidget *applet)
 void
 panel_widget_change_params(PanelWidget *panel,
 			   PanelOrientation orient,
+			   PanelSizeType sz,
 			   PanelBackType back_type,
 			   char *pixmap,
 			   int fit_pixmap_bg,
 			   GdkColor *back_color)
 {
 	PanelOrientation oldorient;
+	PanelSizeType oldsz;
 	int change_back = FALSE;
 
 	g_return_if_fail(panel!=NULL);
@@ -2231,14 +2570,23 @@ panel_widget_change_params(PanelWidget *panel,
 	g_return_if_fail(GTK_WIDGET_REALIZED(panel));
 
 	oldorient = panel->orient;
-
 	panel->orient = orient;
 
+	oldsz = panel->sz;
+	panel->sz = sz;
+
+	kill_cache_on_all_buttons(panel, TRUE);
+
 	if(oldorient != panel->orient) {
 	   	gtk_signal_emit(GTK_OBJECT(panel),
 	   			panel_widget_signals[ORIENT_CHANGE_SIGNAL],
 	   			panel->orient);
 	}
+	if(oldsz != panel->sz) {
+	   	gtk_signal_emit(GTK_OBJECT(panel),
+	   			panel_widget_signals[SIZE_CHANGE_SIGNAL],
+	   			panel->sz);
+	}
 	if(back_color) {
 		/*this will allways trigger, but so what*/
 		if(back_type == PANEL_BACK_COLOR)
@@ -2266,6 +2614,7 @@ panel_widget_change_params(PanelWidget *panel,
 	panel->fit_pixmap_bg = fit_pixmap_bg;
 	if(back_type == PANEL_BACK_PIXMAP) {
 		panel_try_to_set_pixmap (panel, panel->back_pixmap);
+		panel_widget_draw_all(panel,NULL);
 	} else if(back_type == PANEL_BACK_COLOR) {
 		panel_try_to_set_back_color(panel, &panel->back_color);
 	} else {
@@ -2289,6 +2638,7 @@ panel_widget_change_orient(PanelWidget *panel,
 {
 	panel_widget_change_params(panel,
 				   orient,
+				   panel->sz,
 				   panel->back_type,
 				   panel->back_pixmap,
 				   panel->fit_pixmap_bg,
@@ -2328,3 +2678,18 @@ panel_widget_change_global(int explicit_step,
 			gtk_widget_queue_resize(li->data);
 	}
 }
+
+int
+panel_widget_get_pixel_size(PanelWidget *panel)
+{
+	g_return_val_if_fail(panel!=NULL,48);
+	g_return_val_if_fail(IS_PANEL_WIDGET(panel),48);
+
+	switch(panel->sz) {
+	case SIZE_TINY: return 24;
+	case SIZE_STANDARD: return 48;
+	case SIZE_LARGE: return 64;
+	case SIZE_HUGE: return 80;
+	default: return 48;
+	}
+}
