@@ -7,14 +7,6 @@
 
 BEGIN_GNOME_DECLS
 
-typedef enum {
-	SNAPPED_PANEL,
-	DRAWER_PANEL,
-	CORNER_PANEL,
-	FREE_PANEL, /*not yet implemented*/
-	TABBED_PANEL /*not yet implemented*/
-} PanelType;
-
 typedef struct _PanelData PanelData;
 struct _PanelData {
 	PanelType type;
@@ -24,17 +16,23 @@ struct _PanelData {
 };
 
 void orientation_change(AppletInfo *info, PanelWidget *panel);
+void size_change(AppletInfo *info, PanelWidget *panel);
 void back_change(AppletInfo *info, PanelWidget *panel);
 
 PanelOrientType get_applet_orient(PanelWidget *panel);
 
 void panel_setup(GtkWidget *panel);
+void basep_pos_connect_signals (BasePWidget *basep);
 
 /*send state change to all the panels*/
 void send_state_change(void);
 
+void panel_freeze_changes(PanelWidget *panel);
+
+void panel_thaw_changes(PanelWidget *panel);
+
 #define get_panel_parent(appletw) \
-	(gtk_object_get_data(GTK_OBJECT(appletw->parent), PANEL_PARENT))
+	 (PANEL_WIDGET(GTK_WIDGET(appletw)->parent)->panel_parent)
 
 
 END_GNOME_DECLS
