@@ -28,8 +28,8 @@ struct _GlobalConfig {
 	char *tile_down[LAST_TILE];
 	int tile_border[LAST_TILE];
 	int tile_depth[LAST_TILE];
+	int hide_panel_frame;
+	int tile_when_over;
 };
 
-void panel_config_global(void);
-
-#endif /* PANEL_CONFIG_H */
+#endif /* PANEL_CONFIG_GLOBAL_H */
