@@ -15,10 +15,11 @@
 
 #include "panel-include.h"
 
+/*#define PANEL_DEBUG 1*/
+
 int config_sync_timeout = 0;
 int applets_to_sync = FALSE;
 int panels_to_sync = FALSE;
-int globals_to_sync = FALSE;
 int need_complete_save = FALSE;
 
 extern GSList *panels;
@@ -38,8 +39,12 @@ char *old_panel_cfg_path=NULL;
 /*list of all panel widgets created*/
 extern GSList *panel_list;
 
-/*add applets to this queue on startup to avoid races*/
-static GSList *applets_to_start = NULL;
+int ss_cur_applet = 0;
+int ss_done_save = FALSE;
+gushort ss_cookie = 0;
+GtkWidget *ss_timeout_dlg = NULL;
+static int ss_interactive = FALSE;
+static int ss_timeout = 500;
 
 /*send the tooltips state to all external applets*/
 static void
@@ -77,7 +82,6 @@ apply_global_config(void)
 	static int keep_bottom_old = -1;
 	static int autohide_size_old = -1;
 	GSList *li;
-
 	panel_widget_change_global(global_config.explicit_hide_step_size,
 				   global_config.auto_hide_step_size,
 				   global_config.drawer_step_size,
@@ -86,6 +90,7 @@ apply_global_config(void)
 				   global_config.movement_type,
 				   global_config.disable_animations,
 				   global_config.applet_padding);
+
 	if(global_config.tooltips_enabled)
 		gtk_tooltips_enable(panel_tooltips);
 	else
@@ -95,6 +100,7 @@ apply_global_config(void)
 	  so that he doesn't have to reread his menus all the time:)*/
 	if(dot_buttons_old != global_config.show_dot_buttons ||
 	   small_icons_old != global_config.show_small_icons) {
+		GSList *li;
 		for(li=applets;li!=NULL;li=g_slist_next(li)) {
 			AppletInfo *info = li->data;
 			if(info->menu) {
@@ -126,33 +132,12 @@ apply_global_config(void)
 
 	if(keep_bottom_old == -1 ||
 	   keep_bottom_old != global_config.keep_bottom) {
-		GSList *li;
 		for(li = panel_list; li != NULL; li = g_slist_next(li)) {
 			PanelData *pd = li->data;
 			if(!GTK_WIDGET_REALIZED(pd->panel))
 				continue;
-			if((IS_SNAPPED_WIDGET(pd->panel) &&
-			    SNAPPED_WIDGET(pd->panel)->mode != SNAPPED_AUTO_HIDE &&
-			    SNAPPED_WIDGET(pd->panel)->state == SNAPPED_SHOWN) ||
-			   (IS_CORNER_WIDGET(pd->panel) &&
-			    CORNER_WIDGET(pd->panel)->state == SNAPPED_SHOWN)) {
-				if(global_config.keep_bottom)
-					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_BELOW);
-				else
-					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_DOCK);
-			} else if(IS_DRAWER_WIDGET(pd->panel)) {
-				if(global_config.keep_bottom)
-					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_BELOW);
-				else
-					gnome_win_hints_set_layer(pd->panel,
-								  WIN_LAYER_ABOVE_DOCK);
-			} else {
-				if(global_config.keep_bottom)
-					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_ONTOP);
-				else
-					gnome_win_hints_set_layer(pd->panel,
-								  WIN_LAYER_ABOVE_DOCK);
-			}
+			if (IS_BASEP_WIDGET (pd->panel))
+				basep_widget_update_winhints (BASEP_WIDGET (pd->panel));
 		}
 	}
 	keep_bottom_old = global_config.keep_bottom;
@@ -164,43 +149,99 @@ apply_global_config(void)
 					global_config.tile_down[i],
 					global_config.tile_border[i],
 					global_config.tile_depth[i]);
-	}
+	}	
 
-	for(li = panel_list; li != NULL; li = g_slist_next (li)) {
+	for(li = panel_list; li != NULL; li = g_slist_next(li)) {
 		PanelData *pd = li->data;
-		if ((autohide_size_old != global_config.minimize_delay) &&			
-		    (IS_CORNER_WIDGET (pd->panel) &&
-		     (CORNER_WIDGET (pd->panel)->state == CORNER_AUTO_HIDDEN)) ||
-		    (IS_SNAPPED_WIDGET (pd->panel) &&
-		     (SNAPPED_WIDGET (pd->panel)->state == SNAPPED_HIDDEN)))
-			gtk_widget_queue_resize (GTK_WIDGET (pd->panel));
+		if (IS_BASEP_WIDGET (pd->panel)) {
+			if ((autohide_size_old != global_config.minimized_size) &&
+			    (BASEP_WIDGET (pd->panel)->state == 
+			     BASEP_AUTO_HIDDEN)) {
+				gtk_widget_queue_resize (GTK_WIDGET (pd->panel));
+			}
+			basep_update_frame (BASEP_WIDGET (pd->panel));
+		}
 	}
 	autohide_size_old = global_config.minimized_size;
 }
 
 static int
+session_save_timeout(gpointer data)
+{
+	int cookie = GPOINTER_TO_INT(data);
+	if(cookie != ss_cookie)
+		return FALSE;
+
+#ifdef PANEL_DEBUG	
+	printf("SAVE TIMEOUT (%u)\n",ss_cookie);
+#endif
+	if (!ss_interactive)
+		return FALSE;
+
+	ss_timeout_dlg =
+		gnome_message_box_new(_("An applet is not "
+					"responding to a "
+					"save request.\n"
+					"Remove the applet "
+					"or continue waiting?"),
+				      GNOME_MESSAGE_BOX_WARNING,
+				      NULL);
+	gtk_signal_connect(GTK_OBJECT(ss_timeout_dlg),"destroy",
+			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+			   &ss_timeout_dlg);
+	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (ss_timeout_dlg),
+						_("Remove applet"),
+						GNOME_STOCK_PIXMAP_TRASH);
+	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (ss_timeout_dlg),
+						_("Continue waiting"),
+						GNOME_STOCK_PIXMAP_TIMER);
+	
+	if (0 == gnome_dialog_run_and_close (GNOME_DIALOG (ss_timeout_dlg))) {
+		ss_cookie++;
+		g_warning(_("Timed out on sending session save to an applet"));
+		save_next_applet();
+		return FALSE;
+	}
+	return TRUE;
+}
+
+static void
 send_applet_session_save (AppletInfo *info,
 			  CORBA_Object obj,
 			  const char *cfgpath,
 			  const char *globcfgpath)
 {
-  CORBA_short retval;
-  CORBA_Environment ev;
+	CORBA_Environment ev;
+	
+	/*new unique cookie*/
+	ss_cookie++;
+	
+#ifdef PANEL_DEBUG	
+	printf("SENDING_SESSION_SAVE (%u)\n",ss_cookie);
+#endif
 
-  CORBA_exception_init(&ev);
-  retval = GNOME_Applet_session_save(obj,
-				     (CORBA_char *)cfgpath,
-				     (CORBA_char *)globcfgpath, &ev);
-  if(ev._major)
-    panel_clean_applet(info);
-  CORBA_exception_free(&ev);
+	gtk_timeout_add(ss_timeout,session_save_timeout,GINT_TO_POINTER((int)ss_cookie));
 
-  return retval;
+	CORBA_exception_init(&ev);
+	GNOME_Applet_session_save(obj,
+				  (CORBA_char *)cfgpath,
+				  (CORBA_char *)globcfgpath,
+				  ss_cookie, &ev);
+	if(ev._major) {
+		panel_clean_applet(info);
+		CORBA_exception_free(&ev);
+		save_next_applet();
+		return;
+	}
+
+	CORBA_exception_free(&ev);
 }
 
 
 
-static void
+/*returns TRUE if the save was completed, FALSE if we need to wait
+  for the applet to respond*/
+static int
 save_applet_configuration(AppletInfo *info)
 {
 	GString       *buf;
@@ -208,10 +249,16 @@ save_applet_configuration(AppletInfo *info)
 	PanelWidget   *panel;
 	AppletData    *ad;
 	
-	g_return_if_fail(info!=NULL);
+	g_return_val_if_fail(info!=NULL,TRUE);
 
 	buf = g_string_new(NULL);
-	g_string_sprintf(buf, "%sApplet_Config/Applet_%d/", panel_cfg_path, info->applet_id+1);
+
+	gnome_config_push_prefix("");
+	g_string_sprintf(buf, "%sApplet_Config/Applet_%d", PANEL_CONFIG_PATH, info->applet_id+1);
+	gnome_config_clean_section(buf->str);
+	gnome_config_pop_prefix();
+
+	g_string_sprintf(buf, "%sApplet_Config/Applet_%d/", PANEL_CONFIG_PATH, info->applet_id+1);
 	gnome_config_push_prefix(buf->str);
 
 	/*obviously no need for saving*/
@@ -221,7 +268,7 @@ save_applet_configuration(AppletInfo *info)
 		gnome_config_set_string("id", EMPTY_ID);
 		gnome_config_pop_prefix();
 		g_string_free(buf,TRUE);
-		return;
+		return TRUE;
 	}
 
 	panel = PANEL_WIDGET(info->widget->parent);
@@ -231,7 +278,7 @@ save_applet_configuration(AppletInfo *info)
 		gnome_config_set_string("id", EMPTY_ID);
 		gnome_config_pop_prefix();
 		g_string_free(buf,TRUE);
-		return;
+		return TRUE;
 	}
 
 	switch(info->type) {
@@ -239,40 +286,24 @@ save_applet_configuration(AppletInfo *info)
 		{
 			char *globalcfg;
 			Extern *ext = info->data;
-/*this should no longer be needed on our side, we don't write anything
-  to the applet's file and it doesn't write anything to ours*/
-#if 0
-			/*sync before the applet does it's stuff*/
-			gnome_config_sync();
-			/*I think this should be done at sync and also that
-			  there should be some flocking ... but this works
-			  for now*/
-			gnome_config_drop_all();
-#endif
 
-			globalcfg = g_concat_dir_and_file(panel_cfg_path,
+			globalcfg = g_concat_dir_and_file(PANEL_CONFIG_PATH,
 							  "Applet_All_Extern/");
 
 			/*this is the file path we pass to the applet for it's
 			  own config, this is a separate file, so that we */
 			g_string_sprintf(buf, "%sApplet_%d_Extern/",
-					 panel_cfg_path, info->applet_id+1);
+					 PANEL_CONFIG_PATH, info->applet_id+1);
+			gnome_config_clean_file(buf->str);
+			/*just in case the applet times out*/
+			gnome_config_set_string("id", EMPTY_ID);
+			gnome_config_pop_prefix();
+			gnome_config_sync();
 			/*have the applet do it's own session saving*/
-			if(send_applet_session_save(info,ext->applet,
-						    buf->str, globalcfg)) {
-
-				gnome_config_set_string("id", EXTERN_ID);
-				gnome_config_set_string("goad_id",
-							ext->goad_id);
-			} else {
-				g_free(globalcfg);
-				gnome_config_set_string("id", EMPTY_ID);
-				gnome_config_pop_prefix();
-				g_string_free(buf,TRUE);
-				return;
-			}
+			send_applet_session_save(info,ext->applet,
+						 buf->str, globalcfg);
 			g_free(globalcfg);
-			break;
+			return FALSE; /*here we'll wait for done_session_save*/
 		}
 	case APPLET_DRAWER: 
 		{
@@ -321,7 +352,7 @@ save_applet_configuration(AppletInfo *info)
 			/*we set the .desktop to be in the panel config
 			  dir*/
 			g_string_sprintf(buf, "%s/%sApplet_%d.desktop",
-					 gnome_user_dir,panel_cfg_path,
+					 gnome_user_dir,PANEL_CONFIG_PATH,
 					 info->applet_id+1);
 			g_free(launcher->dentry->location);
 			launcher->dentry->location = g_strdup(buf->str);
@@ -334,6 +365,12 @@ save_applet_configuration(AppletInfo *info)
 	case APPLET_LOGOUT:
 		gnome_config_set_string("id", LOGOUT_ID);
 		break;
+	case APPLET_LOCK:
+		gnome_config_set_string("id", LOCK_ID);
+		break;
+	case APPLET_STATUS:
+		gnome_config_set_string("id", STATUS_ID);
+		break;
 	default:
 		g_assert_not_reached();
 	}
@@ -344,6 +381,8 @@ save_applet_configuration(AppletInfo *info)
 							   info->widget));
 	g_string_free(buf,TRUE);
 	gnome_config_pop_prefix();
+	
+	return TRUE;
 }
 
 static void
@@ -357,8 +396,14 @@ save_panel_configuration(gpointer data, gpointer user_data)
 	
 	buf = g_string_new(NULL);
 
-	g_string_sprintf(buf, "%spanel/Panel_%d/", panel_cfg_path, (*num)++);
+	gnome_config_push_prefix ("");
+	g_string_sprintf(buf, "%spanel/Panel_%d", PANEL_CONFIG_PATH, *num);
 	gnome_config_clean_section(buf->str);
+	gnome_config_pop_prefix ();
+
+	g_string_sprintf(buf, "%spanel/Panel_%d/", PANEL_CONFIG_PATH, *num);
+
+	(*num)++;
 
 	gnome_config_push_prefix (buf->str);
 
@@ -369,34 +414,12 @@ save_panel_configuration(gpointer data, gpointer user_data)
 	gnome_config_set_bool("hidebutton_pixmaps_enabled",
 			      basep->hidebutton_pixmaps_enabled);
 	
-	switch(pd->type) {
-	case SNAPPED_PANEL:
-		{
-		SnappedWidget *snapped = SNAPPED_WIDGET(pd->panel);
-		gnome_config_set_int("pos", snapped->pos);
-		gnome_config_set_int("mode", snapped->mode);
-		gnome_config_set_int("state", snapped->state);
-		break;
-		}
-	case CORNER_PANEL:
-		{
-		CornerWidget *corner = CORNER_WIDGET(pd->panel);
-		gnome_config_set_int("pos", corner->pos);
-		gnome_config_set_int("orient",panel->orient);
-		gnome_config_set_int("mode", corner->mode);
-		gnome_config_set_int("state", corner->state);
-		break;
-		}
-	case DRAWER_PANEL:
-		{
-		DrawerWidget *drawer = DRAWER_WIDGET(pd->panel);
-		gnome_config_set_int("orient",drawer->orient);
-		gnome_config_set_int("state", drawer->state);
-		break;
-		}
-	default:
-		g_assert_not_reached();
-	}
+
+	gnome_config_set_int ("mode", basep->mode);
+	gnome_config_set_int ("state", basep->state);
+
+	gnome_config_set_int("sz", panel->sz);
+
 	gnome_config_set_bool("fit_pixmap_bg", panel->fit_pixmap_bg);
 
 	gnome_config_set_string("backpixmap",
@@ -412,38 +435,81 @@ save_panel_configuration(gpointer data, gpointer user_data)
 	
 	g_string_free(buf,TRUE);
 
+	/* now do different types */
+	if (IS_BORDER_WIDGET(basep))
+		gnome_config_set_int("edge", BORDER_POS(basep->pos)->edge);
+
+	switch (pd->type) {
+	case ALIGNED_PANEL:
+		gnome_config_set_int ("align", ALIGNED_POS (basep->pos)->align);
+		break;
+	case SLIDING_PANEL:
+		gnome_config_set_int ("offset", SLIDING_POS (basep->pos)->offset);
+		gnome_config_set_int ("anchor", SLIDING_POS (basep->pos)->anchor);
+		break;
+	case FLOATING_PANEL:
+		gnome_config_set_int ("orient", PANEL_WIDGET (basep->panel)->orient);
+		gnome_config_set_int ("x", FLOATING_POS (basep->pos)->x);
+		gnome_config_set_int ("y", FLOATING_POS (basep->pos)->y);
+		break;
+	case DRAWER_PANEL:
+		gnome_config_set_int ("orient", DRAWER_POS (basep->pos)->orient);
+		/*gnome_config_set_int ("temp_hidden", DRAWER_POS (basep->pos)->temp_state);*/
+		break;
+	default:
+		break;
+	}
+
 	gnome_config_pop_prefix ();
 }
 
+void
+save_next_applet(void)
+{
+	GSList *cur;
+
+	ss_cur_applet++;
+	
+	if(g_slist_length(applets)<=ss_cur_applet) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	cur = g_slist_nth(applets,ss_cur_applet);
+	
+	if(!cur) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	if(save_applet_configuration(cur->data))
+		save_next_applet();
+
+	gnome_config_sync();
+	gnome_config_drop_all();
+}
+
 static void
 do_session_save(GnomeClient *client,
 		int complete_sync,
 		int sync_applets,
-		int sync_panels,
-		int sync_globals)
+		int sync_panels)
 {
 	int num;
 	char *s;
-	char *session_id;
-	int i;
-#ifdef PER_SESSION_CONFIGURATION
+#if PER_SESSION_CONFIGURATION
 	gchar *new_args[] = { "rm", "-r", NULL };
 #endif /* PER_SESSION_CONFIGURATION */
 
 	if (panel_cfg_path)
 		g_free(panel_cfg_path);
 
-	/* Since we are implementing only a single session, that
-	 * either can be saved by the user on exit, or not saved,
-	 * it is most natural if we make the saved state global,
-	 * not per-session
-	 */
 #ifdef PER_SESSION_CONFIGURATION
 	if (gnome_client_get_flags(client) & GNOME_CLIENT_IS_CONNECTED &&
 	    GNOME_CLIENT (client)->restart_style != GNOME_RESTART_NEVER)
 		panel_cfg_path = g_strdup (gnome_client_get_config_prefix (client));
 	else
-#endif /* PER_SESSION_CONFIGURATION */
+#endif /* PER_SESSION_CONFIGURATION */		
 		panel_cfg_path = g_strdup ("/panel.d/default/");
 
 #ifdef PER_SESSION_CONFIGURATION
@@ -452,19 +518,14 @@ do_session_save(GnomeClient *client,
 #endif /* PER_SESSION_CONFIGURATION */
 
 #ifdef PANEL_DEBUG	
-	printf("Saving to [%s]\n",panel_cfg_path);
+	printf("Saving to [%s]\n",PANEL_CONFIG_PATH);
 
 	printf("Saving session: 1"); fflush(stdout);
-#endif
-	if(complete_sync || sync_applets) {
-		GSList *li;
-		for(li=applets;li!=NULL;li=g_slist_next(li))
-			save_applet_configuration(li->data);
-	}
-#ifdef PANEL_DEBUG
+
 	printf(" 2"); fflush(stdout);
 #endif
-	s = g_concat_dir_and_file(panel_cfg_path,"panel/Config/");
+
+	s = g_concat_dir_and_file(PANEL_CONFIG_PATH,"panel/Config/");
 	gnome_config_push_prefix (s);
 	g_free(s);
 
@@ -479,69 +540,19 @@ do_session_save(GnomeClient *client,
 		gnome_config_set_int("panel_count",num-1);
 	}
 #ifdef PANEL_DEBUG
-	printf(" 4"); fflush(stdout);
+	printf(" 4\n"); fflush(stdout);
 #endif
-	if(complete_sync || sync_globals) {
-		GString *buf;
-		/*global options*/
-		gnome_config_set_int("auto_hide_step_size",
-				     global_config.auto_hide_step_size);
-		gnome_config_set_int("explicit_hide_step_size",
-				     global_config.explicit_hide_step_size);
-		gnome_config_set_int("drawer_step_size",
-				     global_config.drawer_step_size);
-		gnome_config_set_int("minimized_size",
-				     global_config.minimized_size);
-		gnome_config_set_int("minimize_delay",
-				     global_config.minimize_delay);
-		gnome_config_set_int("movement_type",
-				     (int)global_config.movement_type);
-		gnome_config_set_bool("tooltips_enabled",
-				      global_config.tooltips_enabled);
-		gnome_config_set_bool("show_small_icons",
-				      global_config.show_small_icons);
-		gnome_config_set_bool("show_dot_buttons",
-				      global_config.show_dot_buttons);
-		gnome_config_set_bool("hungry_menus",
-				      global_config.hungry_menus);
-		gnome_config_set_bool("off_panel_popups",
-				      global_config.off_panel_popups);
-		gnome_config_set_bool("disable_animations",
-				      global_config.disable_animations);
-		gnome_config_set_int("applet_padding",
-				     global_config.applet_padding);
-		gnome_config_set_bool("autoraise",
-				      global_config.autoraise);
-		gnome_config_set_bool("keep_bottom",
-				      global_config.keep_bottom);
-		gnome_config_set_bool("drawer_auto_close",
-				      global_config.drawer_auto_close);
-		gnome_config_set_bool("simple_movement",
-				      global_config.simple_movement);
-		buf = g_string_new(NULL);
-		for(i=0;i<LAST_TILE;i++) {
-			g_string_sprintf(buf,"tiles_enabled_%d",i);
-			gnome_config_set_bool(buf->str,
-					      global_config.tiles_enabled[i]);
-			g_string_sprintf(buf,"tile_up_%d",i);
-			gnome_config_set_string(buf->str,
-						global_config.tile_up[i]);
-			g_string_sprintf(buf,"tile_down_%d",i);
-			gnome_config_set_string(buf->str,
-						global_config.tile_down[i]);
-			g_string_sprintf(buf,"tile_border_%d",i);
-			gnome_config_set_int(buf->str,
-					     global_config.tile_border[i]);
-			g_string_sprintf(buf,"tile_depth_%d",i);
-			gnome_config_set_int(buf->str,
-					     global_config.tile_depth[i]);
-		}
-		g_string_free(buf,TRUE);
-	}
 
 	gnome_config_pop_prefix ();
 	gnome_config_sync();
-#ifdef PANEL_DEBUG
+	
+	if(complete_sync || sync_applets) {
+		ss_cur_applet = -1;
+		ss_done_save = FALSE;
+		save_next_applet();
+	}
+
+#if 0 /*PANEL_DEBUG*/
 	puts("");
 #endif
 }
@@ -549,26 +560,21 @@ do_session_save(GnomeClient *client,
 void
 panel_config_sync(void)
 {
+	int ncs = need_complete_save;
+	int ats = applets_to_sync;
+	int pts = panels_to_sync;
+
 	if(need_complete_save ||
 	   applets_to_sync ||
-	   panels_to_sync ||
-	   globals_to_sync) {
-/* For now, since we aren't saving state per-session, just
- * write the stuff out without going through the SM. This
- * saves triggering a problem where the SM times out on us
- * if our menu is up
- */
-#ifdef PER_SESSION_CONFIGURATION 
+	   panels_to_sync) {
+#ifdef PER_SESSION_CONFIGURATION
 		if (!(gnome_client_get_flags(client) & 
 		      GNOME_CLIENT_IS_CONNECTED)) {
-#endif /* PER_SESSION_CONFIGURATION */
-			do_session_save(client,need_complete_save,
-					applets_to_sync,panels_to_sync,
-					globals_to_sync);
+#endif
 			need_complete_save = FALSE;
 			applets_to_sync = FALSE;
 			panels_to_sync = FALSE;
-			globals_to_sync = FALSE;
+			do_session_save(client,ncs,ats,pts);
 #ifdef PER_SESSION_CONFIGURATION
 		} else {
 			/*prevent possible races by doing this before requesting
@@ -576,7 +582,6 @@ panel_config_sync(void)
 			need_complete_save = FALSE;
 			applets_to_sync = FALSE;
 			panels_to_sync = FALSE;
-			globals_to_sync = FALSE;
 			gnome_client_request_save (client, GNOME_SAVE_LOCAL, FALSE,
 						   GNOME_INTERACT_NONE, FALSE, FALSE);
 		}
@@ -600,7 +605,22 @@ panel_session_save (GnomeClient *client,
 		    int is_fast,
 		    gpointer client_data)
 {
-	do_session_save(client,TRUE,FALSE,FALSE,FALSE);
+	if(is_shutdown) {
+		ss_timeout = 1500;
+		ss_interactive = TRUE;
+	}
+	do_session_save(client,TRUE,FALSE,FALSE);
+	if(is_shutdown) {
+		GSList *li;
+		for(li=panel_list;li;li=g_slist_next(li)) {
+			PanelData *pd = li->data;
+			gtk_widget_hide(pd->panel);
+			if (pd->menu)
+				gtk_widget_hide(pd->menu);
+		}
+		while(!ss_done_save)
+			gtk_main_iteration_do(TRUE);
+	}
 	/* Always successful.  */
 	return TRUE;
 }
@@ -616,6 +636,9 @@ panel_session_die (GnomeClient *client,
 	/*don't catch these any more*/
 	signal(SIGCHLD, SIG_DFL);
 	
+	status_inhibit = TRUE;
+	status_spot_remove_all();
+	
 	for(li=applets; li!=NULL; li=g_slist_next(li)) {
 		AppletInfo *info = li->data;
 		if(info->type == APPLET_EXTERN) {
@@ -639,50 +662,75 @@ panel_session_die (GnomeClient *client,
 void
 panel_quit(void)
 {
-  if(GNOME_CLIENT_CONNECTED(client))
-    gnome_client_request_save (client, GNOME_SAVE_BOTH, 1,
-			       GNOME_INTERACT_ANY, 0, 1);
-  else
-    gtk_main_quit(); /* If we're not session-managed, just exit right now */
+	if(GNOME_CLIENT_CONNECTED(client))
+		gnome_client_request_save (client, GNOME_SAVE_BOTH, 1,
+					   GNOME_INTERACT_ANY, 0, 1);
+	else
+		gtk_main_quit();
 }
 
 static void
-load_default_applets(void)
+load_default_applets1(PanelWidget *panel)
 {
 	char *def_launchers[] =
 	      { "gnome/apps/gnome-help.desktop",
+		"gnome/apps/Utilities/gnome-terminal.desktop",
 		"gnome/apps/Settings/gnomecc.desktop",
 		"gnome/apps/Applications/Netscape.desktop",
-		"gnome/apps/Utilities/gnome-terminal.desktop",
 		NULL };
 	int i;
-	int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER;
+	int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER|
+		MAIN_MENU_SYSTEM_SUB|MAIN_MENU_USER_SUB|
+		MAIN_MENU_APPLETS|MAIN_MENU_APPLETS_SUB;
+	char *p;
 
 	/*guess redhat menus*/
-	if(g_file_exists("/etc/X11/wmconfig"))
+	if(g_file_exists(REDHAT_MENUDIR))
 		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
-	/*guess KDE menus */
+
+	/*guess KDE menus*/
 	if(g_file_exists(KDE_MENUDIR))
 		flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
+
 	/*guess debian menus*/
-	if (g_file_exists("/etc/menu-methods/gnome"))
+	if (g_file_exists(DEBIAN_MENUDIR))
 		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
+	
 	load_menu_applet(NULL,flags, panels->data, 0);
 
-	for(i=0;def_launchers[i]!=NULL;i++) {
-		char *p = gnome_datadir_file (def_launchers[i]);
-		int center = gdk_screen_width()/2;
-		if(p) {
-			load_launcher_applet(p,panels->data,center+i);
-			g_free(p);
+	/*load a Programs menu*/
+	p = gnome_datadir_file ("gnome/apps");
+	/*it really should exist*/
+	if(g_file_exists(p))
+		load_menu_applet(p,flags, panel, 1);
+	g_free(p);
+	
+	if(gdk_screen_width()>800) {
+		/*load up some launchers, but only if screen larger then 800*/
+		for(i=0;def_launchers[i]!=NULL;i++) {
+			p = gnome_datadir_file (def_launchers[i]);
+			/*int center = gdk_screen_width()/2;*/
+			if(p) {
+				load_launcher_applet(p,panel,48*3+i*48);
+				g_free(p);
+			}
 		}
 	}
 
+	load_extern_applet("tasklist_applet",NULL,
+			   panel,INT_MAX/2/*flush right*/,TRUE);
+	load_status_applet(panel,INT_MAX/2 + 1000/*flush right*/);
 	load_extern_applet("gen_util_clock",NULL,
-			   panels->data,INT_MAX/2/*right flush*/,TRUE);
-	/*we laoded default applets, so we didn't find the config or
-	  something else was wrong, so do complete save when next syncing*/
-	need_complete_save = TRUE;
+			   panel,INT_MAX/2 + 2000/*flush right*/,TRUE);
+	load_extern_applet("gen_util_mailcheck",NULL,
+			   panel,INT_MAX/2 + 3000/*flush right*/,TRUE);
+}
+
+static void
+load_default_applets2(PanelWidget *panel)
+{
+	load_extern_applet("deskguide_applet",NULL,
+			   panel,0,TRUE);
 }
 
 void
@@ -693,7 +741,7 @@ init_user_applets(void)
 
 	buf = g_string_new(NULL);
 	g_string_sprintf(buf,"%spanel/Config/applet_count=0",
-			 old_panel_cfg_path);
+			 PANEL_CONFIG_PATH);
 	count=gnome_config_get_int(buf->str);
 	for(num=1;num<=count;num++) {
 		char *applet_name;
@@ -701,7 +749,7 @@ init_user_applets(void)
 		PanelWidget *panel;
 
 		g_string_sprintf(buf,"%sApplet_Config/Applet_%d/",
-				 old_panel_cfg_path, num);
+				 PANEL_CONFIG_PATH, num);
 		gnome_config_push_prefix(buf->str);
 		applet_name = gnome_config_get_string("id=Unknown");
 		
@@ -739,7 +787,7 @@ init_user_applets(void)
 				/*this is the config path to be passed to the
 				  applet when it loads*/
 				g_string_sprintf(buf,"%sApplet_%d_Extern/",
-						 old_panel_cfg_path,num);
+						 PANEL_CONFIG_PATH,num);
 				load_extern_applet(goad_id,buf->str,panel,pos,TRUE);
 			}
 			g_free(goad_id);
@@ -749,6 +797,10 @@ init_user_applets(void)
 			g_free(params);
 		} else if(strcmp(applet_name,LOGOUT_ID) == 0) { 
 			load_logout_applet(panel,pos);
+		} else if(strcmp(applet_name,LOCK_ID) == 0) {
+			load_lock_applet(panel,pos);
+		} else if(strcmp(applet_name,STATUS_ID) == 0) {
+			load_status_applet(panel,pos);
 		} else if(strcmp(applet_name,SWALLOW_ID) == 0) {
 			char *path = gnome_config_get_string("execpath=");
 			char *params = gnome_config_get_string("parameters=");
@@ -806,11 +858,11 @@ init_user_panels(void)
 {
 	GString *buf;
 	int   count,num;	
-	GtkWidget *panel;
+	GtkWidget *panel=NULL;
 
 	buf = g_string_new(NULL);
 	g_string_sprintf(buf,"%spanel/Config/panel_count=0",
-			 old_panel_cfg_path);
+			 PANEL_CONFIG_PATH);
 	count=gnome_config_get_int(buf->str);
 
 	/*load a default snapped panel on the bottom of the screen,
@@ -818,9 +870,27 @@ init_user_panels(void)
 	  to work, so this is the way we find out if there was no
 	  config from last time*/
 	if(count<=0)  {
-		panel = snapped_widget_new(SNAPPED_BOTTOM,
-					   SNAPPED_EXPLICIT_HIDE,
-					   SNAPPED_SHOWN,
+		panel = edge_widget_new(BORDER_BOTTOM,
+					BASEP_EXPLICIT_HIDE,
+					BASEP_SHOWN,
+					SIZE_STANDARD,
+					TRUE,
+					TRUE,
+					PANEL_BACK_NONE,
+					NULL,
+					TRUE,
+					NULL);
+		panel_setup(panel);
+		gtk_widget_show(panel);
+
+		/*load up default applets on the default panel*/
+		load_default_applets1(PANEL_WIDGET(BASEP_WIDGET(panel)->panel));
+
+		panel = aligned_widget_new(ALIGNED_LEFT,
+					   BORDER_RIGHT,
+					   BASEP_EXPLICIT_HIDE,
+					   BASEP_SHOWN,
+					   SIZE_STANDARD,
 					   TRUE,
 					   TRUE,
 					   PANEL_BACK_NONE,
@@ -830,8 +900,13 @@ init_user_panels(void)
 		panel_setup(panel);
 		gtk_widget_show(panel);
 
-		/*load up default applets on the default panel*/
-		load_default_applets();
+		/*load up default applets on the second default panel*/
+		load_default_applets2(PANEL_WIDGET(BASEP_WIDGET(panel)->panel));
+
+		/*we laoded default applets, so we didn't find the config
+		  or something else was wrong, so do complete save when
+		  next syncing*/
+		need_complete_save = TRUE;
 
 		g_string_free(buf,TRUE);
 		return;
@@ -840,14 +915,18 @@ init_user_panels(void)
 	for(num=1;num<=count;num++) {
 		PanelType type;
 		PanelBackType back_type;
+		PanelSizeType sz;
+		BasePState state;
+		BasePMode mode;
+		BorderEdge edge;
 		char *back_pixmap, *color;
 		GdkColor back_color = {0,0,0,1};
 		int fit_pixmap_bg;
 		int hidebuttons_enabled;
 		int hidebutton_pixmaps_enabled;
-
+		
 		g_string_sprintf(buf,"%spanel/Panel_%d/",
-				 old_panel_cfg_path, num);
+				 PANEL_CONFIG_PATH, num);
 		gnome_config_push_prefix (buf->str);
 		
 		back_pixmap = gnome_config_get_string ("backpixmap=");
@@ -864,9 +943,12 @@ init_user_panels(void)
 		back_type=gnome_config_get_int(buf->str);
 		fit_pixmap_bg = gnome_config_get_bool ("fit_pixmap_bg=TRUE");
 
+		g_string_sprintf(buf,"sz=%d", SIZE_STANDARD);
+		sz=gnome_config_get_int(buf->str);
+		
 		/*now for type specific config*/
 
-		g_string_sprintf(buf,"type=%d", SNAPPED_PANEL);
+		g_string_sprintf(buf,"type=%d", EDGE_PANEL);
 		type = gnome_config_get_int(buf->str);
 
 		hidebuttons_enabled =
@@ -874,89 +956,101 @@ init_user_panels(void)
 		hidebutton_pixmaps_enabled =
 			gnome_config_get_bool("hidebutton_pixmaps_enabled=TRUE");
 
-		switch(type) {
-		case SNAPPED_PANEL:
-			{
-				SnappedPos pos;
-				SnappedMode mode;
-				SnappedState state;
+		state = gnome_config_get_int("state=0");
+		mode = gnome_config_get_int("mode=0");
+#if 0 /* i guess we can't easily do this for now */
+		pos = basep_widget_load_pos_settings();
+#endif
+		switch (type) {
+			
+		case EDGE_PANEL:
+			g_string_sprintf (buf, "edge=%d", BORDER_BOTTOM);
+			edge = gnome_config_get_int (buf->str);
+			panel = edge_widget_new (edge, 
+						 mode, state, sz,
+						 hidebuttons_enabled,
+						 hidebutton_pixmaps_enabled,
+						 back_type, back_pixmap,
+						 fit_pixmap_bg, &back_color);
+			break;
+		case ALIGNED_PANEL: {
+			AlignedAlignment align;
+			g_string_sprintf (buf, "edge=%d", BORDER_BOTTOM);
+			edge = gnome_config_get_int (buf->str);
+			
+			g_string_sprintf (buf, "align=%d", ALIGNED_LEFT);
+			align = gnome_config_get_int (buf->str);
 
-				g_string_sprintf(buf,"pos=%d", SNAPPED_BOTTOM);
-				pos=gnome_config_get_int(buf->str);
+			panel = aligned_widget_new (align, edge,
+						    mode, state, sz,
+						    hidebuttons_enabled,
+						    hidebutton_pixmaps_enabled,
+						    back_type, back_pixmap,
+						    fit_pixmap_bg, &back_color);
+			break;
+		}
+		case SLIDING_PANEL: {
+			gint16 offset;
+			SlidingAnchor anchor;
+			g_string_sprintf (buf, "edge=%d", BORDER_BOTTOM);
+			edge = gnome_config_get_int (buf->str);
+			
+			g_string_sprintf (buf, "anchor=%d", SLIDING_ANCHOR_LEFT);
+			anchor = gnome_config_get_int (buf->str);
 
-				g_string_sprintf(buf,"mode=%d",
-						 SNAPPED_EXPLICIT_HIDE);
-				mode=gnome_config_get_int(buf->str);
+			offset = gnome_config_get_int ("offset=0");
 
-				g_string_sprintf(buf,"state=%d", SNAPPED_SHOWN);
-				state=gnome_config_get_int(buf->str);
+			panel = sliding_widget_new (anchor, offset, edge,
+						    mode, state, sz,
+						    hidebuttons_enabled,
+						    hidebutton_pixmaps_enabled,
+						    back_type, back_pixmap,
+						    fit_pixmap_bg, &back_color);
+			break;
+		}
+		case DRAWER_PANEL: {
+			PanelOrientType orient;
+			/*BasePState temp_state;*/
 
-				panel = snapped_widget_new(pos,
-							   mode,
-							   state,
-							   hidebuttons_enabled,
-							   hidebutton_pixmaps_enabled,
-							   back_type,
-							   back_pixmap,
-							   fit_pixmap_bg,
-							   &back_color);
-				break;
-			}
-		case DRAWER_PANEL:
-			{
-				DrawerState state;
-				PanelOrientType orient;
+			g_string_sprintf (buf, "orient=%d", ORIENT_UP);
+			orient = gnome_config_get_int (buf->str);
 
-				g_string_sprintf(buf,"state=%d", DRAWER_SHOWN);
-				state=gnome_config_get_int(buf->str);
+#warning FIXME: there are some issues with auto hiding drawers
+			panel = drawer_widget_new (orient,
+						   BASEP_EXPLICIT_HIDE, 
+						   state, sz,
+						   hidebuttons_enabled,
+						   hidebutton_pixmaps_enabled,
+						   back_type, back_pixmap,
+						   fit_pixmap_bg, &back_color);
+#if 0
+			g_string_sprintf (buf, "temp_state=%d", BASEP_SHOWN);
+			temp_state = gnome_config_get_int (buf->str);
+			DRAWER_POS (BASEP_WIDGET (panel)->pos)->temp_state = temp_state;
+#endif
+			break;
+		}
+		case FLOATING_PANEL: {
+			PanelOrientation orient;
+			int x, y;
+			
+			g_string_sprintf (buf, "orient=%d", PANEL_HORIZONTAL);
+			orient = gnome_config_get_int (buf->str);
 
-				g_string_sprintf(buf,"orient=%d", ORIENT_UP);
-				orient=gnome_config_get_int(buf->str);
+			x = gnome_config_get_int ("x=0");
+			y = gnome_config_get_int ("y=0");
 
-				panel = drawer_widget_new(orient,
-							  state,
-							  back_type,
-							  back_pixmap,
-							  fit_pixmap_bg,
-							  &back_color,
-							  hidebutton_pixmaps_enabled,
-							  hidebuttons_enabled);
-				break;
-			}
-		case CORNER_PANEL:
-			{
-				CornerPos pos;
-				PanelOrientation orient;
-				CornerState state;
-				CornerMode mode;
-				
-				g_string_sprintf(buf,"pos=%d", CORNER_NE);
-				pos=gnome_config_get_int(buf->str);
-
-				g_string_sprintf(buf,"orient=%d",
-						 PANEL_HORIZONTAL);
-				orient=gnome_config_get_int(buf->str);
-
-				g_string_sprintf(buf,"state=%d", CORNER_SHOWN);
-				state=gnome_config_get_int(buf->str);
-
-				g_string_sprintf(buf,"mode=%d",
-						 CORNER_EXPLICIT_HIDE);
-				mode=gnome_config_get_int(buf->str);
-
-				panel = corner_widget_new(pos,
-							  orient,
-							  mode,
-							  state,
-							  hidebuttons_enabled,
-							  hidebutton_pixmaps_enabled,
-							  back_type,
-							  back_pixmap,
-							  fit_pixmap_bg,
-							  &back_color);
-				break;
-			}
-		default: panel=NULL; break; /*fix warning*/
+			panel = floating_widget_new (x, y, orient,
+						     mode, state, sz,
+						     hidebuttons_enabled,
+						     hidebutton_pixmaps_enabled,
+						     back_type, back_pixmap,
+						     fit_pixmap_bg, &back_color);
+			break;
+		}
+		default:
+			g_assert_not_reached ();
+			break;
 		}
 
 		gnome_config_pop_prefix ();
@@ -964,9 +1058,10 @@ init_user_panels(void)
 		g_free(color);
 		g_free(back_pixmap);
 
-		panel_setup(panel);
-
-		gtk_widget_show(panel);
+		if (panel) {
+			panel_setup(panel);
+			gtk_widget_show(panel);
+		}
 	}
 	g_string_free(buf,TRUE);
 }
@@ -981,9 +1076,7 @@ load_up_globals(void)
 	buf = g_string_new(NULL);
 
 	/*set up global options*/
-	
-	g_string_sprintf(buf,"%spanel/Config/",old_panel_cfg_path);
-	gnome_config_push_prefix(buf->str);
+	gnome_config_push_prefix("/panel/Config/");
 
 	global_config.tooltips_enabled =
 		gnome_config_get_bool("tooltips_enabled=TRUE");
@@ -1032,7 +1125,8 @@ load_up_globals(void)
 
 	global_config.drawer_auto_close = gnome_config_get_bool("drawer_auto_close=FALSE");
 	global_config.simple_movement = gnome_config_get_bool("simple_movement=FALSE");
-
+	global_config.hide_panel_frame = gnome_config_get_bool("hide_panel_frame=FALSE");
+	global_config.tile_when_over = gnome_config_get_bool("tile_when_over=FALSE");
 	for(i=0;i<LAST_TILE;i++) {
 		g_string_sprintf(buf,"tiles_enabled_%d=TRUE",i);
 		global_config.tiles_enabled[i] =
@@ -1053,9 +1147,208 @@ load_up_globals(void)
 		g_string_sprintf(buf,"tile_depth_%d=2",i);
 		global_config.tile_depth[i] = gnome_config_get_int(buf->str);
 	}
-	g_string_free(buf,TRUE);
-		
 	gnome_config_pop_prefix();
 
+	g_string_free(buf,TRUE);
+
 	apply_global_config();
 }
+
+/* used for conversion to new config */
+static void
+convert_write_config(void)
+{
+	int i;
+	int is_def;
+	GString *buf;
+	gnome_config_push_prefix("/panel/Config/");
+	
+	/* is there any new config written here */
+	gnome_config_get_int_with_default("auto_hide_step_size", &is_def);
+	if(!is_def) {
+		/* we don't want to overwrite a good config */
+		gnome_config_pop_prefix();
+		return;
+	}
+
+	gnome_config_set_int("auto_hide_step_size",
+			     global_config.auto_hide_step_size);
+	gnome_config_set_int("explicit_hide_step_size",
+			     global_config.explicit_hide_step_size);
+	gnome_config_set_int("drawer_step_size",
+			     global_config.drawer_step_size);
+	gnome_config_set_int("minimized_size",
+			     global_config.minimized_size);
+	gnome_config_set_int("minimize_delay",
+			     global_config.minimize_delay);
+	gnome_config_set_int("movement_type",
+			     (int)global_config.movement_type);
+	gnome_config_set_bool("tooltips_enabled",
+			      global_config.tooltips_enabled);
+	gnome_config_set_bool("show_small_icons",
+			      global_config.show_small_icons);
+	gnome_config_set_bool("show_dot_buttons",
+			      global_config.show_dot_buttons);
+	gnome_config_set_bool("hungry_menus",
+			      global_config.hungry_menus);
+	gnome_config_set_bool("off_panel_popups",
+			      global_config.off_panel_popups);
+	gnome_config_set_bool("disable_animations",
+			      global_config.disable_animations);
+	gnome_config_set_int("applet_padding",
+			     global_config.applet_padding);
+	gnome_config_set_bool("autoraise",
+			      global_config.autoraise);
+	gnome_config_set_bool("keep_bottom",
+			      global_config.keep_bottom);
+	gnome_config_set_bool("drawer_auto_close",
+			      global_config.drawer_auto_close);
+	gnome_config_set_bool("simple_movement",
+			      global_config.simple_movement);
+	gnome_config_set_bool("hide_panel_frame",
+			      global_config.hide_panel_frame);
+	gnome_config_set_bool("tile_when_over",
+			      global_config.tile_when_over);
+	buf = g_string_new(NULL);
+	for(i=0;i<LAST_TILE;i++) {
+		g_string_sprintf(buf,"tiles_enabled_%d",i);
+		gnome_config_set_bool(buf->str,
+				      global_config.tiles_enabled[i]);
+		g_string_sprintf(buf,"tile_up_%d",i);
+		gnome_config_set_string(buf->str,
+					global_config.tile_up[i]);
+		g_string_sprintf(buf,"tile_down_%d",i);
+		gnome_config_set_string(buf->str,
+					global_config.tile_down[i]);
+		g_string_sprintf(buf,"tile_border_%d",i);
+		gnome_config_set_int(buf->str,
+				     global_config.tile_border[i]);
+		g_string_sprintf(buf,"tile_depth_%d",i);
+		gnome_config_set_int(buf->str,
+				     global_config.tile_depth[i]);
+	}
+	g_string_free(buf,TRUE);
+	gnome_config_pop_prefix();
+	gnome_config_sync();
+}
+
+static gboolean
+convert_read_old_config(void)
+{
+	char *tile_def[]={"normal","purple","green","blue"};
+	GString *buf;
+	int i,is_def;
+	int applet_count; /*store this so that we can clean*/
+	int panel_count; /*store this so that we can clean*/
+
+	buf = g_string_new(NULL);
+	g_string_sprintf(buf,"%spanel/Config/",PANEL_CONFIG_PATH);
+	gnome_config_push_prefix(buf->str);
+
+	gnome_config_get_bool_with_default("tooltips_enabled=TRUE",&is_def);
+	if(is_def) {
+		gnome_config_pop_prefix();
+		g_string_free(buf,TRUE);
+		return FALSE;
+	}
+	
+	global_config.tooltips_enabled =
+		gnome_config_get_bool("tooltips_enabled=TRUE");
+
+	global_config.show_small_icons =
+		gnome_config_get_bool("show_small_icons=TRUE");
+
+	global_config.show_dot_buttons =
+		gnome_config_get_bool("show_dot_buttons=FALSE");
+
+	global_config.hungry_menus =
+		gnome_config_get_bool("hungry_menus=TRUE");
+
+	global_config.off_panel_popups =
+		gnome_config_get_bool("off_panel_popups=TRUE");
+
+	global_config.disable_animations =
+		gnome_config_get_bool("disable_animations=FALSE");
+
+	g_string_sprintf(buf,"auto_hide_step_size=%d",
+			 DEFAULT_AUTO_HIDE_STEP_SIZE);
+	global_config.auto_hide_step_size=gnome_config_get_int(buf->str);
+
+	g_string_sprintf(buf,"explicit_hide_step_size=%d",
+			 DEFAULT_EXPLICIT_HIDE_STEP_SIZE);
+	global_config.explicit_hide_step_size=gnome_config_get_int(buf->str);
+
+	g_string_sprintf(buf,"drawer_step_size=%d",
+			 DEFAULT_DRAWER_STEP_SIZE);
+	global_config.drawer_step_size=gnome_config_get_int(buf->str);
+
+	g_string_sprintf(buf,"minimize_delay=%d", DEFAULT_MINIMIZE_DELAY);
+	global_config.minimize_delay=gnome_config_get_int(buf->str);
+
+	g_string_sprintf(buf,"minimized_size=%d", DEFAULT_MINIMIZED_SIZE);
+	global_config.minimized_size=gnome_config_get_int(buf->str);
+
+	g_string_sprintf(buf,"movement_type=%d", PANEL_SWITCH_MOVE);
+	global_config.movement_type=gnome_config_get_int(buf->str);
+
+	global_config.applet_padding=gnome_config_get_int("applet_padding=3");
+
+	global_config.autoraise = gnome_config_get_bool("autoraise=TRUE");
+
+	global_config.keep_bottom = gnome_config_get_bool("keep_bottom=FALSE");
+
+	global_config.drawer_auto_close = gnome_config_get_bool("drawer_auto_close=FALSE");
+	global_config.simple_movement = gnome_config_get_bool("simple_movement=FALSE");
+	global_config.hide_panel_frame = gnome_config_get_bool("hide_panel_frame=FALSE");
+	global_config.tile_when_over = gnome_config_get_bool("tile_when_over=FALSE");
+	for(i=0;i<LAST_TILE;i++) {
+		g_string_sprintf(buf,"tiles_enabled_%d=TRUE",i);
+		global_config.tiles_enabled[i] =
+			gnome_config_get_bool(buf->str);
+
+		g_free(global_config.tile_up[i]);
+		g_string_sprintf(buf,"tile_up_%d=tiles/tile-%s-up.png",
+				 i, tile_def[i]);
+		global_config.tile_up[i] = gnome_config_get_string(buf->str);
+
+		g_free(global_config.tile_down[i]);
+		g_string_sprintf(buf,"tile_down_%d=tiles/tile-%s-down.png",
+				 i,tile_def[i]);
+		global_config.tile_down[i] = gnome_config_get_string(buf->str);
+
+		g_string_sprintf(buf,"tile_border_%d=2",i);
+		global_config.tile_border[i] = gnome_config_get_int(buf->str);
+		g_string_sprintf(buf,"tile_depth_%d=2",i);
+		global_config.tile_depth[i] = gnome_config_get_int(buf->str);
+	}
+
+	/* preserve applet count */
+	applet_count = gnome_config_get_int("applet_count=0");
+	panel_count = gnome_config_get_int("panel_count=0");
+
+	gnome_config_pop_prefix();
+
+	gnome_config_push_prefix("");
+
+	g_string_sprintf(buf,"%spanel/Config",PANEL_CONFIG_PATH);
+
+	gnome_config_clean_section(buf->str);
+
+	g_string_sprintf(buf,"%spanel/Config/applet_count",PANEL_CONFIG_PATH);
+	gnome_config_set_int(buf->str,applet_count);
+	g_string_sprintf(buf,"%spanel/Config/panel_count",PANEL_CONFIG_PATH);
+	gnome_config_set_int(buf->str,panel_count);
+
+	gnome_config_pop_prefix();
+
+	gnome_config_sync();
+	g_string_free(buf,TRUE);
+	return TRUE;
+}
+
+void
+convert_old_config(void)
+{
+	if(convert_read_old_config())
+		convert_write_config();
+}
