@@ -7,6 +7,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
@@ -22,6 +23,8 @@
 
 extern PanelWidget *current_panel;
 
+extern GList *check_swallows;
+
 static int
 ignore_x_error(Display* d, XErrorEvent* e)
 {
@@ -75,17 +78,6 @@ get_window_id(Window win, char *title, guint32 *wid)
 	return ret;
 }
 
-/*we should really do this differently but for now this is good enough*/
-static int
-socket_getwindow_timeout(Swallow *swallow)
-{
-	if(!get_window_id(GDK_ROOT_WINDOW(),swallow->title, &(swallow->wid)))
-		return TRUE;
-	swallow->timeout = -1;
-	gtk_socket_steal(GTK_SOCKET(swallow->socket),swallow->wid);
-	return FALSE;
-}
-
 static int
 socket_realized(GtkWidget *w, gpointer data)
 {
@@ -93,12 +85,11 @@ socket_realized(GtkWidget *w, gpointer data)
 
 	g_return_val_if_fail(swallow->title!=NULL,FALSE);
 
-	if(!get_window_id(GDK_ROOT_WINDOW(),swallow->title, &swallow->wid))
-		swallow->timeout = 
-			gtk_timeout_add(500,
-					(GtkFunction)socket_getwindow_timeout,
-					swallow);
-	else
+	if(!get_window_id(GDK_ROOT_WINDOW(),swallow->title, &swallow->wid)) {
+		check_swallows = g_list_prepend(check_swallows,swallow);
+		XSelectInput(GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+			     SubstructureNotifyMask);
+	} else
 		gtk_socket_steal(GTK_SOCKET(swallow->socket),swallow->wid);
 
 	return FALSE;
@@ -109,56 +100,15 @@ socket_destroyed(GtkWidget *w, gpointer data)
 {
 	Swallow *swallow = data;
 	
-	if(swallow->timeout!=-1) {
-		gtk_timeout_remove(swallow->timeout);
-		swallow->timeout = -1;
-	}
-	
 	gtk_widget_destroy(swallow->ebox);
 
+	check_swallows = g_list_remove(check_swallows,swallow);
+
 	g_free(swallow->title);
 	g_free(swallow->path);
 	g_free(swallow);
 }
 
-static int
-swallow_detached (GtkWidget *handle_box, gpointer data)
-{
-	Swallow *swallow  = gtk_object_get_data(GTK_OBJECT(handle_box), "swallow_obj");
-	
-	if(GPOINTER_TO_INT(gtk_object_get_user_data
-			   (GTK_OBJECT(handle_box)))==SWALLOW_VERTICAL) {
-		GTK_HANDLE_BOX(handle_box)->handle_position = GTK_POS_TOP;
-		gtk_widget_set_usize( swallow->handle_box, swallow->width,
-				      DRAG_HANDLE_SIZE);
-	} else {
-		GTK_HANDLE_BOX(handle_box)->handle_position = GTK_POS_LEFT; 
-		gtk_widget_set_usize( handle_box, DRAG_HANDLE_SIZE,
-				      swallow->height );
-	} 
-	gtk_widget_queue_resize ( GTK_WIDGET(swallow->ebox) );
-	return FALSE;
-}
-
-static int
-swallow_attached (GtkWidget *handle_box, gpointer data)
-{
-	Swallow *swallow = gtk_object_get_data(GTK_OBJECT(handle_box), "swallow_obj");
-	
-	if(GPOINTER_TO_INT(gtk_object_get_user_data
-			   (GTK_OBJECT(handle_box)))== SWALLOW_VERTICAL) {
-		GTK_HANDLE_BOX(handle_box)->handle_position = GTK_POS_TOP;
-		gtk_widget_set_usize( handle_box, swallow->width,
-				      swallow->height + DRAG_HANDLE_SIZE);
-	} else {
-		GTK_HANDLE_BOX(handle_box)->handle_position = GTK_POS_LEFT;
-		gtk_widget_set_usize( handle_box,
-				      swallow->width + DRAG_HANDLE_SIZE,
-				      swallow->height);
-	} 
-	gtk_widget_queue_resize ( GTK_WIDGET(swallow->ebox) );
-	return FALSE;
-}
 
 static void
 really_add_swallow(GtkWidget *d,int button, gpointer data)
@@ -245,8 +195,9 @@ ask_about_swallowing(PanelWidget *panel, int pos)
 
 	w = gtk_label_new(_("Width"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
-	adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 255.0, 1.0,
-						    5.0, 0.0);
+	adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 
+						    (float)gdk_screen_width(),
+						    1.0, 5.0, 0.0);
 	gtk_adjustment_set_value(adj, 48); 
 	width_s = gtk_spin_button_new(adj,0,0);
 	gtk_box_pack_start(GTK_BOX(box),width_s,FALSE,FALSE,0);
@@ -257,8 +208,9 @@ ask_about_swallowing(PanelWidget *panel, int pos)
 	
 	w = gtk_label_new(_("Height"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
-	adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 255.0, 1.0,
-						    5.0, 0.0);
+	adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 
+						    (float)gdk_screen_height(),
+						    1.0, 5.0, 0.0);
 	gtk_adjustment_set_value(adj, 48); 
 	height_s = gtk_spin_button_new(adj,0,0);
 	gtk_box_pack_start(GTK_BOX(box),height_s,FALSE,FALSE,0);
@@ -281,25 +233,25 @@ ask_about_swallowing(PanelWidget *panel, int pos)
 }
 
 static int
-ignore_1st_click(GtkWidget *w, GdkEvent *e)
+ignore_1st_click(GtkWidget *widget, GdkEvent *event)
 {
-	GdkEventButton *be = (GdkEventButton *)e;
-	if(e->type == GDK_BUTTON_PRESS &&
-	   be->button == 1) {
-		gtk_signal_emit_stop_by_name(GTK_OBJECT(w),"event");
-		return TRUE;
+	GdkEventButton *buttonevent = (GdkEventButton *)event;
+
+	if((event->type == GDK_BUTTON_PRESS &&
+	    buttonevent->button == 1) ||
+	   (event->type == GDK_BUTTON_RELEASE &&
+	    buttonevent->button == 1)) {
+		buttonevent->button = 2;
 	}
+	 
 	return FALSE;
 }
 
-
 static Swallow *
 create_swallow_applet(char *title, char *path, int width, int height, SwallowOrient orient)
 {
 	Swallow *swallow;
 	GtkWidget *w;
-	GtkWidget *table;
-	GtkWidget *handle_box;
 	
 	swallow = g_new(Swallow,1);
 
@@ -347,7 +299,6 @@ create_swallow_applet(char *title, char *path, int width, int height, SwallowOri
 	swallow->width = width;
 	swallow->height = height;
 	swallow->wid = -1;
-	swallow->timeout = -1;
 
 	set_swallow_applet_orient(swallow, orient);
 
